/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * Automatic Directory Listing Search
 * Henrik Engstr�m, henrikengstrom on home point se
 */

#ifndef ADL_SEARCH_FRAME_H
#define ADL_SEARCH_FRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MDIChildWindow.h"
#include "ExListViewCtrl.h"
#include "WinUtil.h"
#include "../client/ADLSearch.h"

///////////////////////////////////////////////////////////////////////////////
//
//	Class that represent an ADL search manager interface
//
///////////////////////////////////////////////////////////////////////////////
class ADLSearchFrame : public MDITabChildWindow, public StaticFrame<ADLSearchFrame, ResourceManager::ADL_SEARCH, IDC_FILE_ADL_SEARCH>, 
	private SettingsManagerListener {
public:
	// Constructor/destructor
	ADLSearchFrame();
	~ADLSearchFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK ADLSearchFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	LRESULT ADLSearchFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	// Message handlers
	void onCreate();
	bool onClose();
	void onAdd();
	void onEdit();
	void onRemove();
	void onHelp();
	void onMoveUp();
	void onMoveDown();
	bool onDoubleClickList(LPNMHDR pnmh);
	void onItemChanged(LPNMHDR pnmh);
	bool onKeyDown(LPNMHDR pnmh);
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onRefreshSettings();
	
	// Update colors
	LRESULT onCtlColor(WPARAM wParam, LPARAM lParam) {
		HWND hWnd = (HWND)lParam;
		HDC hDC   = (HDC)wParam;
		if(hWnd == ctrlList.m_hWnd) {
			::SetBkColor(hDC, WinUtil::bgColor);
			::SetTextColor(hDC, WinUtil::textColor);
			return (LRESULT)WinUtil::bgBrush;
		}

		return 0;
	}

	// Update control layouts
	void UpdateLayout();
private:
	// Communication with manager
	void LoadAll();
	void UpdateSearch(int index, BOOL doDelete = TRUE);

	// Contained controls
	ExListViewCtrl ctrlList;

	HWND hWndAdd, hWndEdit, hWndRemove, hWndMoveUp, hWndMoveDown, hWndHelp;
	HMENU contextMenu;

	bool closed;

	// Column order
	enum {
		COLUMN_FIRST = 0,
		COLUMN_ACTIVE_SEARCH_STRING = COLUMN_FIRST,
		COLUMN_SOURCE_TYPE,
		COLUMN_DEST_DIR,
		COLUMN_MIN_FILE_SIZE,
		COLUMN_MAX_FILE_SIZE,
		COLUMN_LAST
	};

	// Column parameters
	static int columnIndexes[];
	static int columnSizes[];
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();
};

#endif // ADL_SEARCH_FRAME_H

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
