/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "DownloadPage.h"
#include "WinUtil.h"
#include "PublicHubsListDlg.h"

#include "../client/SettingsManager.h"

DownloadPage * pDownloadPage = NULL;

DownloadPage::DownloadPage(SettingsManager *s) : PropPage(s, IDD_DOWNLOADPAGE) {
    pDownloadPage = this;
	SetTitle(CTSTRING(SETTINGS_DOWNLOADS));
	m_psp.pfnDlgProc = (DLGPROC)DownloadPageProc;
}

DownloadPage::~DownloadPage() {
    pDownloadPage = NULL;
}

BOOL DownloadPage::DownloadPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM /*lParam*/) {
    if(uMsg == WM_INITDIALOG) {
        pDownloadPage->m_hWnd = hWnd;
        pDownloadPage->onInitDialog();
        return TRUE;
    } else if(uMsg == WM_COMMAND) {
        switch(LOWORD(wParam)) {
            case IDC_BROWSEDIR:
                pDownloadPage->onClickedBrowseDir();
                return TRUE;
            case IDC_BROWSETEMPDIR:
                pDownloadPage->onClickedBrowseTempDir();
                return TRUE;
            case IDC_SETTINGS_LIST_CONFIG:
                pDownloadPage->onClickedListConfigure();
                return TRUE;
        }
    }

	return FALSE;
}

PropPage::TextItem DownloadPage::texts[] = {
	{ IDC_SETTINGS_DIRECTORIES, ResourceManager::SETTINGS_DIRECTORIES },
	{ IDC_SETTINGS_DOWNLOAD_DIRECTORY, ResourceManager::SETTINGS_DOWNLOAD_DIRECTORY },
	{ IDC_BROWSEDIR, ResourceManager::BROWSE_ACCEL },
	{ IDC_SETTINGS_UNFINISHED_DOWNLOAD_DIRECTORY, ResourceManager::SETTINGS_UNFINISHED_DOWNLOAD_DIRECTORY },
	{ IDC_BROWSETEMPDIR, ResourceManager::BROWSE },
	{ IDC_SETTINGS_DOWNLOAD_LIMITS, ResourceManager::SETTINGS_DOWNLOAD_LIMITS },
	{ IDC_SETTINGS_DOWNLOADS_MAX, ResourceManager::SETTINGS_DOWNLOADS_MAX },
	{ IDC_EXTRA_DOWNLOADS_MAX, ResourceManager::SETTINGS_CZDC_EXTRA_DOWNLOADS },
	{ IDC_SETTINGS_DOWNLOADS_SPEED_PAUSE, ResourceManager::SETTINGS_DOWNLOADS_SPEED_PAUSE },
	{ IDC_SETTINGS_SPEEDS_NOT_ACCURATE, ResourceManager::SETTINGS_SPEEDS_NOT_ACCURATE },
	{ IDC_SETTINGS_PUBLIC_HUB_LIST, ResourceManager::SETTINGS_PUBLIC_HUB_LIST },
	{ IDC_SETTINGS_PUBLIC_HUB_LIST_URL, ResourceManager::SETTINGS_PUBLIC_HUB_LIST_URL },
	{ IDC_SETTINGS_LIST_CONFIG, ResourceManager::SETTINGS_CONFIGURE_HUB_LISTS },
	{ IDC_SETTINGS_PUBLIC_HUB_LIST_HTTP_PROXY, ResourceManager::SETTINGS_PUBLIC_HUB_LIST_HTTP_PROXY },
	{ IDC_MAX_FILELIST2MATCH, ResourceManager::SETTINGS_MAX_FILELIST2MATCH },
	{ IDC_FILELIST_DOWNLOADS_MAX, ResourceManager::SETTINGS_FILELIST_DOWNLOADS },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item DownloadPage::items[] = {
	{ IDC_TEMP_DOWNLOAD_DIRECTORY, SettingsManager::TEMP_DOWNLOAD_DIRECTORY, PropPage::T_STR },
	{ IDC_DOWNLOADDIR,	SettingsManager::DOWNLOAD_DIRECTORY, PropPage::T_STR },
	{ IDC_DOWNLOADS, SettingsManager::DOWNLOAD_SLOTS, PropPage::T_INT },
	{ IDC_MAXSPEED, SettingsManager::MAX_DOWNLOAD_SPEED, PropPage::T_INT },
	{ IDC_EXTRA_DOWN_SLOT, SettingsManager::EXTRA_DOWNLOAD_SLOTS, PropPage::T_INT },
	{ IDC_PROXY, SettingsManager::HTTP_PROXY, PropPage::T_STR },
	{ IDC_FILELIST_DOWN_SLOT, SettingsManager::FILELIST_DOWNLOAD_SLOTS, PropPage::T_INT },
	{ IDC_MAXFILELIST2MATCH, SettingsManager::MAX_FILELIST2MATCH, PropPage::T_INT },
	{ 0, 0, PropPage::T_END }
};

void DownloadPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	PropPage::read(m_hWnd, items);
	
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SLOTSSPIN), UDM_SETRANGE, 0, MAKELPARAM(100, 0));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SPEEDSPIN), UDM_SETRANGE, 0, MAKELPARAM(10000, 0));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_EXTRASLOTSSPIN), UDM_SETRANGE, 0, MAKELPARAM(100, 1));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_FILELISTSPIN), UDM_SETRANGE, 0, MAKELPARAM(100, 0));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_MAXMATCHSPIN), UDM_SETRANGE, 0, MAKELPARAM(999, 0));
}

void DownloadPage::write() {
	PropPage::write(m_hWnd, items);

	const string& s = SETTING(DOWNLOAD_DIRECTORY);
	if(s.length() > 0 && s[s.length() - 1] != '\\') {
		SettingsManager::getInstance()->set(SettingsManager::DOWNLOAD_DIRECTORY, s + '\\');
	}
	const string& t = SETTING(TEMP_DOWNLOAD_DIRECTORY);
	if(t.length() > 0 && t[t.length() - 1] != '\\') {
		SettingsManager::getInstance()->set(SettingsManager::TEMP_DOWNLOAD_DIRECTORY, t + '\\');
	}
}

void DownloadPage::onClickedBrowseDir() {
	tstring dir = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
	if(WinUtil::browseDirectory(dir, m_hWnd)) {
		// Adjust path string
		if(dir.size() > 0 && dir[dir.size() - 1] != '\\')
			dir += '\\';

		::SetDlgItemText(m_hWnd, IDC_DOWNLOADDIR, dir.c_str());
	}
}

void DownloadPage::onClickedBrowseTempDir() {
	tstring dir = Text::toT(SETTING(TEMP_DOWNLOAD_DIRECTORY));
	if(WinUtil::browseDirectory(dir, m_hWnd)) {
		// Adjust path string
		if(dir.size() > 0 && dir[dir.size() - 1] != '\\')
			dir += '\\';

		::SetDlgItemText(m_hWnd, IDC_TEMP_DOWNLOAD_DIRECTORY, dir.c_str());
	}
}

void DownloadPage::onClickedListConfigure() {
	PublicHubListDlg dlg;
	dlg.DoModal(m_hWnd);
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
