/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//---------------------------------------------------------------------------
#include "stdafx.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
#include "../client/version.h"
//---------------------------------------------------------------------------
#include "ExceptionHandling.h"
//---------------------------------------------------------------------------
#include <Dbghelp.h>
#include <delayimp.h>
//---------------------------------------------------------------------------
#pragma comment(lib, "Dbghelp.lib")
//---------------------------------------------------------------------------
LPTOP_LEVEL_EXCEPTION_FILTER pOldTLEF = NULL;
tstring sPath = _T(""), sDebugSymbolsFile = _T("");
static TCHAR sDebugBuf[512];
char sExceptionTTH[40];
//---------------------------------------------------------------------------

FARPROC WINAPI CzDC_FailHook(unsigned /*dliNotify*/, PDelayLoadInfo /*pdli*/) {
    ::MessageBox(NULL, _T("Something bad happen and CzDC crashed. If you know why this happen then please report it as bug to bugs@CzDC.org!"),
        _T("CzDC crashed!"), MB_OK | MB_ICONERROR);
    exit(EXIT_FAILURE);
}

//---------------------------------------------------------------------------

void GetSourceFileInfo(DWORD64 dw64Address, FILE * fw) {
    IMAGEHLP_LINE64 il64LineInfo;
	memset(&il64LineInfo, 0, sizeof(IMAGEHLP_LINE64));
	il64LineInfo.SizeOfStruct = sizeof(IMAGEHLP_LINE64);

    DWORD dwDisplacement = 0;

	if(SymGetLineFromAddr64(GetCurrentProcess(), dw64Address, &dwDisplacement, &il64LineInfo) == TRUE) {
        // We have sourcefile and linenumber info, write it.
        fprintf(fw, "%s(%d): ", il64LineInfo.FileName, il64LineInfo.LineNumber);
	} else {
        // We don't have sourcefile and linenumber info, let's try module name

        IMAGEHLP_MODULE64 im64ModuleInfo;
        memset(&im64ModuleInfo, 0, sizeof(IMAGEHLP_MODULE64));
        im64ModuleInfo.SizeOfStruct = sizeof(IMAGEHLP_MODULE64);

        if(SymGetModuleInfo64(GetCurrentProcess(), dw64Address, &im64ModuleInfo)) {
            // We found module name, write module name and address
            fprintf(fw, "%s|0x%08I64X: ", im64ModuleInfo.ModuleName, dw64Address);
        } else {
            // We don't found module. Write address, it's better than nothing
            fprintf(fw, "0x%08I64X: ", dw64Address);
        }
	}
}
//---------------------------------------------------------------------------

void GetFunctionInfo(DWORD64 dw64Address, FILE * fw) {
	DWORD64 dw64Displacement = 0;

    ULONG64 ul64Buffer[(sizeof(SYMBOL_INFO) + MAX_SYM_NAME*sizeof(TCHAR) + sizeof(ULONG64) - 1) / sizeof(ULONG64)];
    PSYMBOL_INFO pSym = (PSYMBOL_INFO)ul64Buffer;

    pSym->SizeOfStruct = sizeof(SYMBOL_INFO);
    pSym->MaxNameLen = MAX_SYM_NAME;

	if(SymFromAddr(GetCurrentProcess(), dw64Address, &dw64Displacement, pSym) == TRUE) {
        // We have decorated name, try to make it readable
        if(UnDecorateSymbolName(pSym->Name, (PSTR)sDebugBuf, 512, UNDNAME_COMPLETE | UNDNAME_NO_THISTYPE | UNDNAME_NO_SPECIAL_SYMS | UNDNAME_NO_MEMBER_TYPE |
            UNDNAME_NO_MS_KEYWORDS | UNDNAME_NO_ACCESS_SPECIFIERS) > 0) {
            // We have readable name, write it
            fprintf(fw, (char *)sDebugBuf);
            fprintf(fw, "\n");
            return;
        }
	}

    // We don't found any info, write '?'
    fprintf(fw, "?\n");
}
//---------------------------------------------------------------------------

LONG WINAPI CzDC_UnhandledExceptionFilter(LPEXCEPTION_POINTERS ExceptionInfo) {
    static volatile LONG PermLock = 0;

    // When unhandled exception happen then permanently 'lock' here. We terminate after first exception.
    while(InterlockedExchange(&PermLock, 1) == 1) {
		::Sleep(10);
	}

    // Set failure hook
	__pfnDliFailureHook2 = CzDC_FailHook;

    // Check if we have debug symbols
	DWORD dwCode = GetFileAttributes(sDebugSymbolsFile.c_str());
	if(dwCode == INVALID_FILE_ATTRIBUTES || dwCode == FILE_ATTRIBUTE_DIRECTORY) {
        ::MessageBox(NULL, _T("Something bad happen and CzDC crashed. If you know why this happen then please report it as bug to bugs@CzDC.org!"),
            _T("CzDC crashed!"), MB_OK | MB_ICONERROR);

        ExceptionHandlingUnitialize();

		exit(EXIT_FAILURE);
	}

    // Initialize debug symbols
    SymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_FAIL_CRITICAL_ERRORS | SYMOPT_LOAD_LINES);
    if(SymInitialize(GetCurrentProcess(), (PCSTR)sPath.c_str(), TRUE) == FALSE) {
        ::MessageBox(NULL, _T("Something bad happen and CzDC crashed. If you know why this happen then please report it as bug to bugs@CzDC.org!"),
            _T("CzDC crashed!"), MB_OK | MB_ICONERROR);

        ExceptionHandlingUnitialize();

		exit(EXIT_FAILURE);
    }

    // Generate crash log filename
    time_t acc_time;
    time(&acc_time);

    struct tm *tm = localtime(&acc_time);
    _tcsftime(sDebugBuf, 128, _T("\\Crash-%d.%m.%Y-%H.%M.%S.log"), tm);

    sPath += sDebugBuf;

    // Open crash file
    FILE * fw = _tfopen(sPath.c_str(), _T("w"));
    if(fw == NULL) {
        ::MessageBox(NULL, _T("Something bad happen and CzDC crashed. If you know why this happen then please report it as bug to bugs@CzDC.org!"),
            _T("CzDC crashed!"), MB_OK | MB_ICONERROR);

        ExceptionHandlingUnitialize();
        SymCleanup(GetCurrentProcess());

		exit(EXIT_FAILURE);
    }

    tstring sCrashMsg = _T("Something bad happen and CzDC crashed. In CzDC directory was generated file ");
    sCrashMsg += tstring(sDebugBuf+1);
    sCrashMsg += _T(" with info why this happen, please send that file to bugs@CzDC.org!");

    // Write CzDC version and exception code
	fprintf(fw, "CzDC version: " VERSIONSTRING "[" CzDCVERSIONSTRING "]"
#ifdef _M_X64
        " (x64)"
#endif
        "\nException Code: %x\n", ExceptionInfo->ExceptionRecord->ExceptionCode);

    {
        // Write windoze version where we crashed if is possible
        OSVERSIONINFOEX ver;
        memset(&ver, 0, sizeof(OSVERSIONINFOEX));
        ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

        if(GetVersionEx((OSVERSIONINFO*)&ver) != 0) {
			fprintf(fw, "Windows version: %d.%d SP: %d\n", ver.dwMajorVersion, ver.dwMinorVersion, ver.wServicePackMajor);
        }
    }

    // Write date and time when crash happen
    _tcsftime(sDebugBuf, 128, _T("Date and time: %d.%m.%Y %H:%M:%S\n"), tm);
    _ftprintf(fw, sDebugBuf);

    // Write executable file TTH
	fprintf(fw, "TTH: %s\n\n", sExceptionTTH);

    STACKFRAME64 sf64CallStack;
    memset(&sf64CallStack, 0, sizeof(STACKFRAME64));

    sf64CallStack.AddrPC.Mode      = AddrModeFlat;
    sf64CallStack.AddrStack.Mode   = AddrModeFlat;
    sf64CallStack.AddrFrame.Mode   = AddrModeFlat;

#ifdef _M_X64
    sf64CallStack.AddrPC.Offset    = ExceptionInfo->ContextRecord->Rip;
    sf64CallStack.AddrStack.Offset = ExceptionInfo->ContextRecord->Rsp;
    sf64CallStack.AddrFrame.Offset = ExceptionInfo->ContextRecord->Rbp;
#else
    sf64CallStack.AddrPC.Offset    = ExceptionInfo->ContextRecord->Eip;
    sf64CallStack.AddrStack.Offset = ExceptionInfo->ContextRecord->Esp;
    sf64CallStack.AddrFrame.Offset = ExceptionInfo->ContextRecord->Ebp;
#endif

	// Write where crash happen
    fprintf(fw, "Exception location:\n");

    GetSourceFileInfo(sf64CallStack.AddrPC.Offset, fw);
	GetFunctionInfo(sf64CallStack.AddrPC.Offset, fw);

	// Try to write callstack
    fprintf(fw, "\nCall stack:\n");

    // We don't want it like never ending story, limit call stack to 100 lines
	for(uint32_t ui32i = 0; ui32i < 100; ui32i++) {
		if(StackWalk64(
#ifdef _M_X64
			IMAGE_FILE_MACHINE_AMD64,
#else
			IMAGE_FILE_MACHINE_I386,
#endif
			GetCurrentProcess(), GetCurrentThread(), &sf64CallStack, ExceptionInfo->ContextRecord, NULL,
            SymFunctionTableAccess64, SymGetModuleBase64, NULL) == FALSE || sf64CallStack.AddrFrame.Offset == 0) {
            break;
        }

        GetSourceFileInfo(sf64CallStack.AddrPC.Offset, fw);
        GetFunctionInfo(sf64CallStack.AddrPC.Offset, fw);
	}

	fclose(fw);

    ::MessageBox(NULL, sCrashMsg.c_str(),
        _T("CzDC crashed!"), MB_OK | MB_ICONERROR);

    ExceptionHandlingUnitialize();
    SymCleanup(GetCurrentProcess());

    exit(EXIT_FAILURE);
}
//---------------------------------------------------------------------------

void ExceptionHandlingInitialize() {
	TCHAR sBuf[MAX_PATH+1];
	::GetModuleFileName(NULL, sBuf, MAX_PATH);
	TCHAR * Path = _tcsrchr (sBuf, _T('\\'));
	if(Path != NULL) {
		sPath = tstring(sBuf, Path-sBuf);
	} else {
		sPath = sBuf;
	}

    size_t iBufLen = _tcslen(sBuf);
    if(iBufLen > 3 && tolower(sBuf[iBufLen-3]) == 'e' && tolower(sBuf[iBufLen-2]) == 'x' && tolower(sBuf[iBufLen-1]) == 'e') {
        sBuf[iBufLen-3] = 'p';
        sBuf[iBufLen-2] = 'd';
        sBuf[iBufLen-1] = 'b';
    }

    sDebugSymbolsFile = sBuf;

     // Set CzDC unhandled exception filter
    pOldTLEF = SetUnhandledExceptionFilter(&CzDC_UnhandledExceptionFilter);
}
//---------------------------------------------------------------------------

void ExceptionHandlingUnitialize() {
    // Restore old Top Level Exception Filter
    SetUnhandledExceptionFilter(pOldTLEF);
}
//---------------------------------------------------------------------------
