/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "FavoritesFrm.h"
#include "HubFrame.h"
#include "FavHubProperties.h"

#include "../client/ClientManager.h"
#include "../client/version.h"

int FavoriteHubsFrame::columnIndexes[] = { COLUMN_NAME, COLUMN_DESCRIPTION, COLUMN_NICK, COLUMN_PASSWORD, COLUMN_SERVER, COLUMN_USERDESCRIPTION };
int FavoriteHubsFrame::columnSizes[] = { 200, 290, 125, 100, 100, 125 };
static ResourceManager::Strings columnNames[] = { ResourceManager::AUTO_CONNECT, ResourceManager::DESCRIPTION, ResourceManager::NICK, ResourceManager::PASSWORD, ResourceManager::SERVER,
    ResourceManager::USER_DESCRIPTION
};
ATOM FavoriteHubsFrame::m_aAtom = 0;
UINT FavoriteHubsFrame::m_uiResourceID = IDR_FAVORITES;

FavoriteHubsFrame::FavoriteHubsFrame() : nosave(true), closed(false), hubsMenu(NULL) {
    RegisterWindowClass(m_aAtom, FavoriteHubsFrameWndProc, _T("FavoriteHubsFrame"), m_uiResourceID);
}

FavoriteHubsFrame::~FavoriteHubsFrame() {
    if(hubsMenu != NULL) {
		::DestroyMenu(hubsMenu);
	}
}

LRESULT CALLBACK FavoriteHubsFrame::FavoriteHubsFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	FavoriteHubsFrame * pParent;

    if(uMsg == WM_NCCREATE) {
		pParent = (FavoriteHubsFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (FavoriteHubsFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->FavoriteHubsFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT FavoriteHubsFrame::FavoriteHubsFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SETFOCUS:
            onSetFocus();
            break;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_CONNECT:
                    openSelected();
                    return 0;
                case IDC_REMOVE:
                    onRemove();
                    return 0;
                case IDC_EDIT:
                    onEdit();
                    return 0;
                case IDC_NEWFAV:
                    onNew();
                    return 0;
                case IDC_MOVE_UP:
                    onMoveUp();
                    return 0;
                case IDC_MOVE_DOWN:
                    onMoveDown();
                    return 0;
                case IDC_OPEN_HUB_LOG:
                    onOpenHubLog();
                    return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_HUBLIST) {
                switch(((LPNMHDR)lParam)->code) {
                    case NM_DBLCLK:
                        onDoubleClickHublist((LPNMHDR)lParam);
                        return 0;
                    case LVN_KEYDOWN:
                        if(onKeyDown((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case LVN_ITEMCHANGED:
                        onItemChanged((LPNMHDR)lParam);
                        return 0;
                    case LVN_COLUMNCLICK:
                        onColumnClickHublist((LPNMHDR)lParam);
                        return 0;
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

void FavoriteHubsFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_FAVORITES);
    created = true;

	ctrlHubs.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS, WS_EX_CLIENTEDGE, IDC_HUBLIST,
        LVS_EX_LABELTIP | LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES | LVS_EX_HEADERDRAGDROP);

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(FAVORITESFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(FAVORITESFRAME_WIDTHS), COLUMN_LAST);

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		ListViewInsertColumn(ctrlHubs.m_hWnd, ui8i, CTSTRING_I(columnNames[ui8i]), LVCFMT_LEFT, columnSizes[ui8i], ui8i);
	}

	::SendMessage(ctrlHubs.m_hWnd, LVM_SETCOLUMNORDERARRAY, COLUMN_LAST, (LPARAM)columnIndexes);

	hWndConnect = CreateWindowEx(0, WC_BUTTON, CTSTRING(CONNECT), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_DISABLED | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_CONNECT, g_hInstance, NULL);
	::SendMessage(hWndConnect, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndNew = CreateWindowEx(0, WC_BUTTON, CTSTRING(NEW), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, (HMENU)IDC_NEWFAV, g_hInstance, NULL);
	::SendMessage(hWndNew, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndProperties = CreateWindowEx(0, WC_BUTTON, CTSTRING(PROPERTIES), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_DISABLED | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_EDIT, g_hInstance, NULL);
	::SendMessage(hWndProperties, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndRemove = CreateWindowEx(0, WC_BUTTON, CTSTRING(REMOVE), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_DISABLED | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_REMOVE, g_hInstance, NULL);
	::SendMessage(hWndRemove, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndUp = CreateWindowEx(0, WC_BUTTON, CTSTRING(MOVE_UP), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_DISABLED | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_MOVE_UP, g_hInstance, NULL);
	::SendMessage(hWndUp, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndDown = CreateWindowEx(0, WC_BUTTON, CTSTRING(MOVE_DOWN), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_DISABLED | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_MOVE_DOWN, g_hInstance, NULL);
	::SendMessage(hWndDown, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	FavoriteManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);
	updateList(FavoriteManager::getInstance()->getFavoriteHubs());

	hubsMenu = ::CreatePopupMenu();
	::AppendMenu(hubsMenu, MF_STRING, IDC_OPEN_HUB_LOG, CTSTRING(OPEN_HUB_LOG));
	::AppendMenu(hubsMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(hubsMenu, MF_STRING, IDC_CONNECT, CTSTRING(CONNECT));
	::AppendMenu(hubsMenu, MF_STRING, IDC_NEWFAV, CTSTRING(NEW));
	::AppendMenu(hubsMenu, MF_STRING, IDC_EDIT, CTSTRING(PROPERTIES));
	::AppendMenu(hubsMenu, MF_STRING, IDC_MOVE_UP, CTSTRING(MOVE_UP));
	::AppendMenu(hubsMenu, MF_STRING, IDC_MOVE_DOWN, CTSTRING(MOVE_DOWN));
	::AppendMenu(hubsMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(hubsMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));
    ::SetMenuDefaultItem(hubsMenu, IDC_CONNECT, 0);

	nosave = false;
}

void FavoriteHubsFrame::openSelected() {
	if(!checkNick())
		return;

	int i = -1;
	while( (i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
		FavoriteHubEntry* entry = (FavoriteHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i);
		RecentHubEntry r;
		r.setName(entry->getName());
		r.setDescription(entry->getDescription());
		r.setUsers("*");
		r.setShared("*");
		r.setServer(entry->getServer());
		FavoriteManager::getInstance()->addRecent(r);
		HubFrame::openWindow(Text::toT(entry->getServer()), 
			entry->getWindowPosX(), entry->getWindowPosY(), entry->getWindowSizeX(), entry->getWindowSizeY(), entry->getWindowType(), 
			entry->getChatUserSplit(), entry->getUserListState(), entry->getColumsOrder(), entry->getColumsWidth(), entry->getColumsVisible());
	}
	return;
}

void FavoriteHubsFrame::addEntry(const FavoriteHubEntry* entry, int pos) {
	TStringList l;
	l.push_back(Text::toT(entry->getName()));
	l.push_back(Text::toT(entry->getDescription()));
	l.push_back(Text::toT(entry->getNick(false)));
	l.push_back(tstring(entry->getPassword().size(), '*'));
	l.push_back(Text::toT(entry->getServer()));
	l.push_back(Text::toT(entry->getUserDescription()));
	bool b = entry->getConnect();
	int i = ctrlHubs.insert(pos, l, 0, (LPARAM)entry);
	ListView_SetItemState(ctrlHubs.m_hWnd, i, INDEXTOSTATEIMAGEMASK(b ? 2 : 1), LVIS_STATEIMAGEMASK);
}

bool FavoriteHubsFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	if((HWND)wParam != ctrlHubs.m_hWnd || (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED) == -1) {
        return false;
    }

    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);

	if(iX == -1 && iY == -1) {
        ListViewGetContextMenuPos(ctrlHubs.m_hWnd, iX, iY);
	}

    ::TrackPopupMenuEx(hubsMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

	return true;
}

void FavoriteHubsFrame::onDoubleClickHublist(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;

	if(item->iItem != -1) {
		FavoriteHubEntry* entry = (FavoriteHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, item->iItem);
		RecentHubEntry r;
		r.setName(entry->getName());
		r.setDescription(entry->getDescription());
		r.setUsers("*");
		r.setShared("*");
		r.setServer(entry->getServer());
		FavoriteManager::getInstance()->addRecent(r);
		HubFrame::openWindow(Text::toT(entry->getServer()), 
			entry->getWindowPosX(), entry->getWindowPosY(), entry->getWindowSizeX(), entry->getWindowSizeY(), entry->getWindowType(), 
			entry->getChatUserSplit(), entry->getUserListState(), entry->getColumsOrder(), entry->getColumsWidth(), entry->getColumsVisible());
	} else {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_NEWFAV, 0);
	}
}

bool FavoriteHubsFrame::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*)pnmh;

	switch(kd->wVKey) {
        case VK_INSERT:
            ::PostMessage(m_hWnd, WM_COMMAND, IDC_NEWFAV, 0);
            return true;
        case VK_DELETE:
            ::PostMessage(m_hWnd, WM_COMMAND, IDC_REMOVE, 0);
            return true;
        case VK_RETURN:
            ::PostMessage(m_hWnd, WM_COMMAND, IDC_CONNECT, 0);
            return true;
        default:
            break;
	}

	return false;
}

void FavoriteHubsFrame::onRemove() {
	if(BOOLSETTING(CONFIRM_ITEM_REMOVAL) == false ||
		::MessageBox(m_hWnd, CTSTRING(CONFIRM_REMOVE), _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
		int i = -1;
		while( (i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
			FavoriteManager::getInstance()->removeFavorite((FavoriteHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i));
		}
	}
}

void FavoriteHubsFrame::onEdit() {
	int i = -1;
	if((i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
		FavoriteHubEntry* e = (FavoriteHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i);
		FavHubProperties dlg(e);
		if(dlg.DoModal(m_hWnd) == IDOK) {
			ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_NAME, LVIF_TEXT, Text::toT(e->getName()).c_str(), 0);
			ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_DESCRIPTION, LVIF_TEXT, Text::toT(e->getDescription()).c_str(), 0);
			ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_SERVER, LVIF_TEXT, Text::toT(e->getServer()).c_str(), 0);
			ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_NICK, LVIF_TEXT, Text::toT(e->getNick(false)).c_str(), 0);
			ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_PASSWORD, LVIF_TEXT, tstring(e->getPassword().size(), '*').c_str(), 0);
			ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_USERDESCRIPTION, LVIF_TEXT, Text::toT(e->getUserDescription()).c_str(), 0);
		}
	}
}

void FavoriteHubsFrame::onNew() {
	FavoriteHubEntry e;
	FavHubProperties dlg(&e);

	while(true) {
		if(dlg.DoModal(m_hWnd) == IDOK) {
			if(FavoriteManager::getInstance()->checkFavHubExists(e)) {
				::MessageBox(m_hWnd, CTSTRING(FAVORITE_HUB_ALREADY_EXISTS), _T(" "), MB_ICONWARNING | MB_OK);
			} else {
				FavoriteManager::getInstance()->addFavorite(e);
				break;
			}
		} else {
			break;
		}
	}
}

bool FavoriteHubsFrame::checkNick() {
	if(SETTING(NICK).empty()) {
		::MessageBox(m_hWnd, CTSTRING(ENTER_NICK), _T(CzDC_VERSION_STRING), MB_ICONSTOP | MB_OK);
		return false;
	}
	return true;
}

void FavoriteHubsFrame::onMoveUp() {
	int j = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
	FavoriteHubEntry::List& fh = FavoriteManager::getInstance()->getFavoriteHubs();

    nosave = true;
	::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for(int i = 1; i < j; ++i) {
		if((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMSTATE, i, LVIS_SELECTED)) {
			FavoriteHubEntry* e = fh[i];
			swap(fh[i], fh[i-1]);
			::SendMessage(ctrlHubs.m_hWnd, LVM_DELETEITEM, i, 0);
			addEntry(e, i-1);
			ListView_SetItemState(ctrlHubs.m_hWnd, i-1, LVIS_SELECTED, LVIS_SELECTED);
			::SendMessage(ctrlHubs.m_hWnd, LVM_ENSUREVISIBLE, i-1, FALSE);
		}
	}
	::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	nosave = false;
	FavoriteManager::getInstance()->save();
}

void FavoriteHubsFrame::onMoveDown() {
	int j = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0) - 2;
	FavoriteHubEntry::List& fh = FavoriteManager::getInstance()->getFavoriteHubs();

	nosave = true;
	::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for(int i = j; i >= 0; --i) {
		if((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMSTATE, i, LVIS_SELECTED)) {
			FavoriteHubEntry* e = fh[i];
			swap(fh[i], fh[i+1]);
			::SendMessage(ctrlHubs.m_hWnd, LVM_DELETEITEM, i, 0);
            addEntry(e, i+1);
            ListView_SetItemState(ctrlHubs.m_hWnd, i+1, LVIS_SELECTED, LVIS_SELECTED);
			::SendMessage(ctrlHubs.m_hWnd, LVM_ENSUREVISIBLE, i-1, FALSE);
		}
	}
	::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	nosave = false;
	FavoriteManager::getInstance()->save();
}

void FavoriteHubsFrame::onItemChanged(LPNMHDR pnmh) {
	NMITEMACTIVATE* l = (NMITEMACTIVATE*)pnmh;

    UINT uiState = (UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMSTATE, l->iItem, LVIS_SELECTED);

	::EnableWindow(hWndConnect, uiState);
	::EnableWindow(hWndRemove, uiState);
	::EnableWindow(hWndProperties, uiState);
	::EnableWindow(hWndUp, uiState);
	::EnableWindow(hWndDown, uiState);

	if(nosave == false && l->iItem != -1 && ((l->uNewState & LVIS_STATEIMAGEMASK) != (l->uOldState & LVIS_STATEIMAGEMASK))) {
		FavoriteHubEntry* f = (FavoriteHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, l->iItem);
		f->setConnect((((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMSTATE, l->iItem, LVIS_STATEIMAGEMASK)>> 12) - 1) != FALSE);
		FavoriteManager::getInstance()->save();
	}
}

bool FavoriteHubsFrame::onClose() {
	if(closed == false) {
		FavoriteManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		WinUtil::saveHeaderOrder(ctrlHubs.m_hWnd, SettingsManager::FAVORITESFRAME_ORDER,
			SettingsManager::FAVORITESFRAME_WIDTHS, COLUMN_LAST, columnIndexes, columnSizes);

		return false;
	}
}

void FavoriteHubsFrame::UpdateLayout() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	RECT rc = rect;
	rc.bottom -=28;
	::MoveWindow(ctrlHubs.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	const long bwidth = 90;
	const long bspace = 10;

	rc = rect;
	rc.bottom -= 2;
	rc.top = rc.bottom - 22;

	rc.left = 2;
	rc.right = rc.left + bwidth;
	::MoveWindow(hWndNew, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	::OffsetRect(&rc, bwidth + 2, 0);
	::MoveWindow(hWndProperties, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	::OffsetRect(&rc, bwidth + 2, 0);
	::MoveWindow(hWndRemove, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	::OffsetRect(&rc, bspace + bwidth + 2, 0);
	::MoveWindow(hWndUp, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	::OffsetRect(&rc, bwidth + 2, 0);
	::MoveWindow(hWndDown, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	::OffsetRect(&rc, bspace + bwidth + 2, 0);
	::MoveWindow(hWndConnect, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

void FavoriteHubsFrame::onOpenHubLog() {
	if((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		FavoriteHubEntry* entry = (FavoriteHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i);
		StringMap params;
		params["hubNI"] = entry->getName();
		params["hubURL"] = entry->getServer();
		params["myNI"] = entry->getNick(); 
		tstring filename = Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_MAIN_CHAT), params, true)));
		if(File::FileExist(Text::fromT(filename))){
			ShellExecute(NULL, NULL, filename.c_str(), NULL, NULL, SW_SHOWNORMAL);
		} else {
			::MessageBox(m_hWnd, CTSTRING(NO_LOG_FOR_HUB), CTSTRING(NO_LOG_FOR_HUB), MB_OK);	  
		}
	}
}

void FavoriteHubsFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void FavoriteHubsFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlHubs.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlHubs.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}
}

void FavoriteHubsFrame::onColumnClickHublist(LPNMHDR pnmh) {
	// On sort, disable move functionality...
	::EnableWindow(hWndUp, FALSE);
	::EnableWindow(hWndDown, FALSE);

	NMLISTVIEW* l = (NMLISTVIEW*)pnmh;

	if(l->iSubItem == ctrlHubs.getSortColumn()) {
		if(!ctrlHubs.isAscending()) {
			ctrlHubs.setSort(-1, ctrlHubs.getSortType());
		} else {
			ctrlHubs.setSortDirection(false);
        }
	} else {
		ctrlHubs.setSort(l->iSubItem, ExListViewCtrl::SORT_STRING_NOCASE);
	}
}
	
/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
