/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "FinishedFrame.h"
#include "WinUtil.h"
#include "TextFrame.h"
#include "FlatTabCtrl.h"

#include "../client/ClientManager.h"

int FinishedFrame::columnIndexes[] = { COLUMN_DONE, COLUMN_FILE, COLUMN_PATH, COLUMN_NICK, COLUMN_HUB, COLUMN_SIZE, COLUMN_SPEED, COLUMN_CRC_TTH };
int FinishedFrame::columnSizes[] = { 100, 110, 290, 125, 80, 80, 80, 80 };
static ResourceManager::Strings columnNames[] = { ResourceManager::FILENAME, ResourceManager::TIME, ResourceManager::PATH, ResourceManager::NICK, ResourceManager::HUB, ResourceManager::SIZE,
    ResourceManager::SPEED, ResourceManager::CRC_TTH
};
ATOM FinishedFrame::m_aAtom = 0;
UINT FinishedFrame::m_uiResourceID = IDR_FINISHED_DL;

FinishedFrame::FinishedFrame() : totalBytes(0), totalTime(0), closed(false), ctxMenu(NULL), tabMenu(NULL) {
    RegisterWindowClass(m_aAtom, FinishedFrameWndProc, _T("FinishedFrame"), m_uiResourceID);
}

FinishedFrame::~FinishedFrame() {
    if(ctxMenu != NULL) {
        ::DestroyMenu(ctxMenu);
    }

    if(tabMenu != NULL) {
        ::DestroyMenu(tabMenu);
    }
}

LRESULT CALLBACK FinishedFrame::FinishedFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	FinishedFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (FinishedFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (FinishedFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->FinishedFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT FinishedFrame::FinishedFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SETFOCUS:
            onSetFocus();
            break;
        case WM_SPEAKER:
            onSpeaker(wParam, lParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case FTM_CONTEXTMENU:
            onTabContextMenu(lParam);
            return TRUE;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_REMOVE:
                case IDC_TOTAL:
                    onRemove(LOWORD(wParam));
                    return 0;
                case IDC_VIEW_AS_TEXT:
                    onViewAsText();
                    return 0;
                case IDC_OPEN_FILE:
                    onOpenFile();
                    return 0;
                case IDC_OPEN_FOLDER:
                    onOpenFolder();
                    return 0;
                case IDC_DOWNLOAD_LOG:
                    onDownloadLog();
                    return 0;
                case IDC_CLOSE_WINDOW:
                    ::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
                    return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_FINISHED) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_COLUMNCLICK:
                        onColumnClickFinished((LPNMHDR)lParam);
                        return 0;
                    case LVN_KEYDOWN:
                        if(onKeyDown((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case NM_DBLCLK:
                        if(onDoubleClick((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case NM_CUSTOMDRAW:
                        return onCustomDraw((LPNMHDR)lParam);
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

void FinishedFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_FINISHED_DL);
    created = true;

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, 0, 0, 0, 0, m_hWnd, 0, g_hInstance, NULL);

	ctrlList.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE,
        IDC_FINISHED, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT);
	::SendMessage(ctrlList.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::fileImages);

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(FINISHED_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(FINISHED_WIDTHS), COLUMN_LAST);

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		int fmt = (ui8i == COLUMN_SIZE || ui8i == COLUMN_SPEED) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		ListViewInsertColumn(ctrlList.m_hWnd, ui8i, CTSTRING_I(columnNames[ui8i]), fmt, columnSizes[ui8i], ui8i);
	}

	::SendMessage(ctrlList.m_hWnd, LVM_SETCOLUMNORDERARRAY, COLUMN_LAST, (LPARAM)columnIndexes);
	ctrlList.setSort(COLUMN_DONE, ExListViewCtrl::SORT_STRING_NOCASE);

	UpdateLayout();

	FinishedManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);
	updateList(FinishedManager::getInstance()->lockList());
	FinishedManager::getInstance()->unlockList();

	ctxMenu = ::CreatePopupMenu();
	::AppendMenu(ctxMenu, MF_STRING, IDC_VIEW_AS_TEXT, CTSTRING(VIEW_AS_TEXT));
	::AppendMenu(ctxMenu, MF_STRING, IDC_OPEN_FILE, CTSTRING(OPEN));
	::AppendMenu(ctxMenu, MF_STRING, IDC_OPEN_FOLDER, CTSTRING(OPEN_FOLDER));
	::AppendMenu(ctxMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(ctxMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));
	::AppendMenu(ctxMenu, MF_STRING, IDC_TOTAL, CTSTRING(REMOVE_ALL));
	::SetMenuDefaultItem(ctxMenu, IDC_OPEN_FILE, 0);

	tabMenu = ::CreatePopupMenu();

	if(BOOLSETTING(LOG_DOWNLOADS)) {
        ::AppendMenu(tabMenu, MF_STRING, IDC_DOWNLOAD_LOG, CTSTRING(OPEN_DOWNLOAD_LOG));
		::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
	}

	::AppendMenu(tabMenu, MF_STRING, IDC_TOTAL, CTSTRING(REMOVE_ALL));
	::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(tabMenu, MF_STRING, IDC_CLOSE_WINDOW, CTSTRING(CLOSE));
}

bool FinishedFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	if((HWND)wParam != ctrlList.m_hWnd || (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED) == -1) {
        return false;
    }

    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);

	if(iX == -1 && iY == -1) {
        ListViewGetContextMenuPos(ctrlList.m_hWnd, iX, iY);
	}

	if(BOOLSETTING(SHOW_SHELL_MENU) && ((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1)) {
		string path = ((FinishedItem*)ListViewGetItem(ctrlList.m_hWnd,
            (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED)))->getTarget();

		if(File::getSize(path) != 1) {
			CShellContextMenu shellMenu;
			shellMenu.SetPath(Text::toT(path));

			HMENU pShellMenu = shellMenu.GetMenu();
			::AppendMenu(pShellMenu, MF_STRING, IDC_VIEW_AS_TEXT, CTSTRING(VIEW_AS_TEXT));
			::AppendMenu(pShellMenu, MF_STRING, IDC_OPEN_FOLDER, CTSTRING(OPEN_FOLDER));
			::AppendMenu(pShellMenu, MF_SEPARATOR, 0, NULL);
			::AppendMenu(pShellMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));
			::AppendMenu(pShellMenu, MF_STRING, IDC_TOTAL, CTSTRING(REMOVE_ALL));
			::AppendMenu(pShellMenu, MF_SEPARATOR, 0, NULL);

			UINT idCommand = shellMenu.ShowContextMenu(m_hWnd, iX, iY);
			if(idCommand != 0) {
				::PostMessage(m_hWnd, WM_COMMAND, idCommand, 0);
			}

			return true;
		}
	}

    ::TrackPopupMenuEx(ctxMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

	return true;
}

void FinishedFrame::onColumnClickFinished(LPNMHDR pnmh) {
	NMLISTVIEW* const l = (NMLISTVIEW*)pnmh;

	if(l->iSubItem == ctrlList.getSortColumn()) {
		if (!ctrlList.isAscending()) {
			ctrlList.setSort(-1, ctrlList.getSortType());
		} else {
			ctrlList.setSortDirection(false);
        }
	} else {
		switch(l->iSubItem) {
			case COLUMN_SIZE:
				ctrlList.setSort(l->iSubItem, ExListViewCtrl::SORT_FUNC, true, sortSize);
				break;
			case COLUMN_SPEED:
				ctrlList.setSort(l->iSubItem, ExListViewCtrl::SORT_FUNC, true, sortSpeed);
				break;
			default:
				ctrlList.setSort(l->iSubItem, ExListViewCtrl::SORT_STRING_NOCASE);
				break;
		}
	}
}

void FinishedFrame::UpdateLayout() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[4];
		::GetClientRect(m_hWndStatusBar, &sr);
		w[3] = sr.right - 16;
		w[2] = max(w[3] - 100, 0);
		w[1] = max(w[2] - 100, 0);
		w[0] = max(w[1] - 100, 0);

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 4, (LPARAM)w);
	}

	RECT rc(rect);
	::MoveWindow(ctrlList.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

bool FinishedFrame::onDoubleClick(LPNMHDR pnmh) {
	NMITEMACTIVATE * const item = (NMITEMACTIVATE*)pnmh;

	if(item->iItem != -1) {
		FinishedItem* entry = (FinishedItem*)ListViewGetItem(ctrlList.m_hWnd, item->iItem);
		WinUtil::openFile(Text::toT(entry->getTarget()));
		return true;
	}

	return false;
}

void FinishedFrame::onViewAsText() {
	int i = -1;
	if((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
		FinishedItem * const entry = (FinishedItem*)ListViewGetItem(ctrlList.m_hWnd, i);
		TextFrame::openWindow(Text::toT(entry->getTarget()));
	}
}

void FinishedFrame::onOpenFile() {
	int i = -1;
	if((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
		FinishedItem * const entry = (FinishedItem*)ListViewGetItem(ctrlList.m_hWnd, i);
		WinUtil::openFile(Text::toT(entry->getTarget()));
	}
}

void FinishedFrame::onOpenFolder() {
	int i = -1;
	if((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
		FinishedItem * const entry = (FinishedItem*)ListViewGetItem(ctrlList.m_hWnd, i);
		WinUtil::openFolder(Text::toT(entry->getTarget()));
	};
}

void FinishedFrame::onRemove(WORD wID) {
	if(wID == IDC_REMOVE) {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
			FinishedManager::getInstance()->remove((FinishedItem*)ListViewGetItem(ctrlList.m_hWnd, i));
			::SendMessage(ctrlList.m_hWnd, LVM_DELETEITEM, i, 0);
		}
    } else if(wID == IDC_TOTAL) {
		FinishedManager::getInstance()->removeAll();
	}
}

bool FinishedFrame::onClose() {
	if(!closed) {
		FinishedManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		WinUtil::saveHeaderOrder(ctrlList.m_hWnd, SettingsManager::FINISHED_ORDER,
			SettingsManager::FINISHED_WIDTHS, COLUMN_LAST, columnIndexes, columnSizes);

		return false;
	}
}

void FinishedFrame::onSpeaker(WPARAM wParam, LPARAM lParam) {
	if(wParam == SPEAK_ADD_LINE) {
		FinishedItem* entry = (FinishedItem*)lParam;
		addEntry(entry);
		if(BOOLSETTING(BOLD_FINISHED_DOWNLOADS)) {
			setDirty();
        }
		updateStatus();
	} else if(wParam == SPEAK_REMOVE) {
		updateStatus();
	} else if(wParam == SPEAK_REMOVE_ALL) {
		::SendMessage(ctrlList.m_hWnd, LVM_DELETEALLITEMS, 0, 0);
		updateStatus();
	}
}

void FinishedFrame::addEntry(FinishedItem* entry) {
	TStringList l;
	l.push_back(Text::toT(Util::getFileName(entry->getTarget())));
	l.push_back(Text::toT(Util::formatTime("%Y-%m-%d %H:%M:%S", entry->getTime())));
	l.push_back(Text::toT(Util::getFilePath(entry->getTarget())));	
	l.push_back(Text::toT(entry->getUser()));
	l.push_back(Text::toT(entry->getHub()));
	l.push_back(Text::toT(Util::formatBytes(entry->getSize())));
	l.push_back(Text::toT(Util::formatBytes(entry->getAvgSpeed()) + "/s"));
	if(entry->getCrcTTHOK()) {
		l.push_back(TSTRING(OK));
	} else if(entry->getCrcTTHBad()) {
		l.push_back(TSTRING(BAD));
	} else {
		l.push_back(TSTRING(NO));
	}

	totalBytes += entry->getChunkSize();
	totalTime += entry->getMilliSeconds();

	int image = WinUtil::getIconIndex(Text::toT(entry->getTarget()));
	int loc = ctrlList.insert(l, image, (LPARAM)entry);
	::SendMessage(ctrlList.m_hWnd, LVM_ENSUREVISIBLE, loc, FALSE);
}

void FinishedFrame::onTabContextMenu(LPARAM lParam) {
    ::TrackPopupMenuEx(tabMenu, TPM_LEFTALIGN | (BOOLSETTING(TABS_ON_TOP) == false ? TPM_BOTTOMALIGN : TPM_TOPALIGN) | TPM_RIGHTBUTTON,
        GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), m_hWnd, NULL);
}

void FinishedFrame::onDownloadLog() {
	tstring filename = Text::toT(SETTING(LOG_DIRECTORY) + SETTING(LOG_FILE_DOWNLOAD));
	if(File::FileExist(Text::fromT(filename))){
		ShellExecute(NULL, NULL, filename.c_str(), NULL, NULL, SW_SHOWNORMAL);
	} else {
		::MessageBox(m_hWnd, CTSTRING(NO_DOWNLOAD_LOG), CTSTRING(NO_DOWNLOAD_LOG), MB_OK);	  
	}
}

LRESULT FinishedFrame::onCustomDraw(LPNMHDR pnmh) {
	LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)pnmh;

	switch(cd->nmcd.dwDrawStage) {
		case CDDS_PREPAINT:
			return CDRF_NOTIFYITEMDRAW;
		case CDDS_ITEMPREPAINT: {
			FinishedItem* fi = (FinishedItem*)cd->nmcd.lItemlParam;
			if(fi->getCrcTTHBad()) {
				cd->clrText = SETTING(ERROR_COLOR);
				return CDRF_NEWFONT | CDRF_NOTIFYSUBITEMDRAW;
			}
		}
		default:
			return CDRF_DODEFAULT;
	}
}

void FinishedFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void FinishedFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlList.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlList.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlList.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlList.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
		::SendMessage(ctrlList.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
