/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef FINISHED_FRAME_H
#define FINISHED_FRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MDIChildWindow.h"
#include "ExListViewCtrl.h"
#include "ShellContextMenu.h"

#include "../client/FinishedManager.h"

class FinishedFrame : public MDITabChildWindow, public StaticFrame<FinishedFrame, ResourceManager::FINISHED_DOWNLOADS, IDC_FINISHED>,
	private FinishedManagerListener, private SettingsManagerListener {
public:
	FinishedFrame();
	~FinishedFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK FinishedFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT FinishedFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void onCreate();
	bool onClose();
	bool onDoubleClick(LPNMHDR pnmh);
	void onSpeaker(WPARAM wParam, LPARAM lParam);
	void onRemove(WORD wID);
	void onViewAsText();
	void onOpenFile();
	void onOpenFolder();
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onColumnClickFinished(LPNMHDR pnmh);
	void onTabContextMenu(LPARAM lParam);
	void onDownloadLog();
	LRESULT onCustomDraw(LPNMHDR pnmh);
	void onRefreshSettings();
	void UpdateLayout();

	void onSetFocus() {
		::SetFocus(ctrlList.m_hWnd);
	}

	static int sortSize(LPARAM a, LPARAM b) {
		FinishedItem* c = (FinishedItem*)a;
		FinishedItem* d = (FinishedItem*)b;
		return compare(c->getSize(), d->getSize());
	}

	static int sortSpeed(LPARAM a, LPARAM b) {
		FinishedItem* c = (FinishedItem*)a;
		FinishedItem* d = (FinishedItem*)b;
		return compare(c->getAvgSpeed(), d->getAvgSpeed());
	}

	bool onKeyDown(LPNMHDR pnmh) {
		NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;

		if(kd->wVKey == VK_DELETE) {
			onRemove(IDC_REMOVE);
			return true;
		}

		return false;
	}

private:
	enum {
		SPEAK_ADD_LINE,
		SPEAK_REMOVE,
		SPEAK_REMOVE_ALL
	};

	enum {
		COLUMN_FIRST,
		COLUMN_FILE = COLUMN_FIRST,
		COLUMN_DONE,
		COLUMN_PATH,
		COLUMN_NICK,
		COLUMN_HUB,
		COLUMN_SIZE,
		COLUMN_SPEED,
		COLUMN_CRC_TTH,
		COLUMN_LAST
	};

	HMENU ctxMenu;
	HMENU tabMenu;

	ExListViewCtrl ctrlList;

	int64_t totalBytes;
	int64_t totalTime;

	bool closed;

	static int columnSizes[COLUMN_LAST];
	static int columnIndexes[COLUMN_LAST];

	void updateStatus() {
        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0),
            (LPARAM)Text::toT(Util::toString((int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0)) + ' ' + STRING(ITEMS)).c_str());
        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)Text::toT(Util::formatBytes(totalBytes)).c_str());
        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (3 | 0),
            (LPARAM)Text::toT(Util::formatBytes((totalTime > 0) ? totalBytes * ((int64_t)1000) / totalTime : 0) + "/s").c_str());
	}

	void updateList(const FinishedItem::List& fl) {
		::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
		for(FinishedItem::List::const_iterator i = fl.begin(); i != fl.end(); ++i) {
			addEntry(*i);
		}
		::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
		::InvalidateRect(ctrlList.m_hWnd, NULL, TRUE);
		updateStatus();
	}

	void addEntry(FinishedItem* entry);

	void on(AddedDl, FinishedItem* entry) throw() {
		::PostMessage(m_hWnd, WM_SPEAKER, SPEAK_ADD_LINE, (WPARAM)entry);
	}
	void on(RemovedDl, FinishedItem* entry) throw() {
		totalBytes -= entry->getChunkSize();
		totalTime -= entry->getMilliSeconds();
		::PostMessage(m_hWnd, WM_SPEAKER, SPEAK_REMOVE, 0);
	}
	void on(RemovedAllDl) throw() {
		::PostMessage(m_hWnd, WM_SPEAKER, SPEAK_REMOVE_ALL, 0);
		totalBytes = 0;
		totalTime = 0;
	}
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();
};

#endif // FINISHED_FRAME_H

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
