/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef HUB_FRAME_H
#define HUB_FRAME_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "MDIChildWindow.h"
#include "TypedListViewCtrl.h"
#include "ChatCtrl.h"
#include "MainFrm.h"

#include "../client/Client.h"
#include "../client/User.h"
#include "../client/ClientManager.h"
#include "../client/TimerManager.h"

#include "WinUtil.h"
#include "UCHandler.h"
#include "UserInfo.h"
#include "WinApiUtil.h"
#include "BasicSplitter.h"

struct CompareItems;

class HubFrame : private MDITabChildWindow, private ClientListener, private BasicSplitter, private FavoriteManagerListener, private TimerManagerListener,
    private UCHandler<HubFrame>, private SettingsManagerListener {
public:
    static ATOM m_aAtom;
    static UINT m_uiResourceID;

	BOOL PreTranslateMessage(MSG* pMsg) {
		if(!::IsWindow(m_hWnd))
			return FALSE;

		::SendMessage(hWndLastLines, TTM_RELAYEVENT, 0, (LPARAM)pMsg);
		return FALSE;
	}

    static LRESULT CALLBACK HubFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT HubFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void OnForwardMsg(LPARAM lParam);
	void onSpeaker();
	void onCopyUserInfo(WORD wID);
	bool onClose();
	void onDoubleClickUsers(LPNMHDR pnmh);
	void OnCreate();
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onTabContextMenu(LPARAM lParam);
	bool onChar(UINT uMsg, WPARAM wParam);
	void onShowUsers(WPARAM wParam);
	void onFollow();
	bool onLButton(WPARAM wParam, LPARAM lParam);
	void onEnterUsers();
	void onGetToolTip(LPNMHDR pnmh);
	LRESULT onCtlColor(WPARAM wParam);
	void onFileReconnect();
	void onClientEnLink(LPNMHDR pnmh);
	void onEditCopy();
	void onEditSelectAll();
	void onEditClearAll();
	void onCopyActualLine();
	void onSelectUser();
	void onPublicMessage();
	void onAutoScrollChat();
	void onCopyURL();
	void onOpenHubLog();
	void onOpenUserLog();
	void onWhoisIP();
	bool onFilterChar(WPARAM wParam);
	LRESULT onCustomDraw(LPNMHDR pnmh, bool& bHandled);
	void onSearch(WORD wID);
	void onRefreshSettings();
	void onIgnore();
	void onUnignore();
	void onMatchQueue();
	void onGetList();
	void onAddToFavorites();
	void onPrivateMessage();
	void onGrantSlot();
	void onGrantSlotHour();
	void onGrantSlotDay();
	void onGrantSlotWeek();
	void onUnGrantSlot();
	void onSize(WPARAM wParam, LPARAM lParam);
	void UpdateLayout(bool bResizeStatusBar = true);
	void addLine(const Identity& i, const tstring& aLine, CHARFORMAT2& cf, bool bUseEmo = true);
	void addClientLine(const tstring& aLine, const CHARFORMAT2& cf, const bool &inChat, bool bNoBold = false);
	void addSystemLine(const tstring& aLine, const CHARFORMAT2& cf);
	void onEnter();
	void onTab();
	void runUserCommand(::UserCommand& uc);

	static void openWindow(const tstring& server, 
		int windowposx = 0, int windowposy = 0, int windowsizex = 0, int windowsizey = 0, int windowtype = 0, int chatusersplit = 0, bool userliststate = true, 
        string sColumsOrder = Util::emptyString, string sColumsWidth = Util::emptyString, string sColumsVisible = Util::emptyString);
    static void resortUsers();
	static void closeDisconnected();

	void onSetFocus() {
		::SetFocus(hWndMessage);
		ctrlClient.GoToEnd();
	}

	bool onKeyDownUsers(LPNMHDR pnmh) {
		NMLVKEYDOWN* l = (NMLVKEYDOWN*)pnmh;

		if(l->wVKey == VK_TAB) {
			onTab();
			return true;
		}

		return false;
	}
private:
	enum Tasks { UPDATE_USER_JOIN, UPDATE_USER, REMOVE_USER, ADD_CHAT_LINE, ADD_STATUS_LINE, SET_WINDOW_TITLE, GET_PASSWORD, PRIVATE_MESSAGE, STATS,
        CONNECTED, DISCONNECTED, KICK_MSG, PRIVATE_MESSAGE_NICK, RESORT_USERLIST
	};

	struct MsgTask {
		MsgTask(const OnlineUser& ou, const string& m) : msg(m) {
            if(&ou != NULL) {
                identity = ou.getIdentity();
                bNoId = false;
            } else {
                bNoId = true;
            }
        }
		Identity identity;
		string msg;
		bool bNoId;
	};

	struct PMTask {
		PMTask(const OnlineUser& ou, const OnlineUser& by, const string& m) : fromidentity(ou.getIdentity()), msg(m) {
            if(&by != NULL) {
                byidentity = by.getIdentity();
                bNoId = false;
            } else {
                bNoId = true;
            }
        }
        Identity fromidentity;
		Identity byidentity;
		string msg;
		bool bNoId;
	};

	struct PMNickTask {
		PMNickTask(const string& nick, const OnlineUser& by, const string& m) : nick(nick), msg(m) {
            if(&by != NULL) {
                byidentity = by.getIdentity();
                bNoId = false;
            } else {
                bNoId = true;
            }
        }
		string nick;
		Identity byidentity;
		string msg;
		bool bNoId;
	};

	HubFrame(const tstring& aServer, int chatusersplit, bool userliststate, 
        string scolumsorder, string scolumswidth, string scolumsvisible);
	~HubFrame();

	typedef HASH_MAP<tstring, HubFrame*> FrameMap;
	typedef FrameMap::iterator FrameIter;
	static FrameMap frames;

	typedef HASH_MAP<UserPtr, UserInfo*, User::HashFunction> UserMap;
	typedef UserMap::iterator UserMapIter;

	tstring redirect;
	bool timeStamps;
	bool showJoins;
	bool favShowJoins;
	tstring complete;
	bool inTabComplete;

	bool waitingForPW;
	bool extraSort;

	TStringList prevCommands;
	tstring currentCommand;
	TStringList::size_type curCommandPosition;

	struct CountAvailable {
		CountAvailable() : available(0) { }
		int64_t available;
		void operator()(UserInfo *ui) {
            if(ui != NULL)
                available += ui->getIdentity().getBytesShared();
		}
	};

	int64_t CountUserMapAvailable() {
		int64_t available = 0;
		for(UserMapIter i = userMap.begin(); i != userMap.end(); ++i) {
            if(i->second != NULL)
                available += i->second->getIdentity().getBytesShared();
		}
		return available;
	}

	Client* client;
	tstring server;

	HMENU tabMenu;
	
	HWND hWndShowUsers, hWndMessage, hWndFilterSel, hWndFilter;

    ChatCtrl ctrlClient;

	typedef TypedListViewCtrl<UserInfo, IDC_USERS> CtrlUsers;
	CtrlUsers ctrlUsers;

	bool closed;

	TStringMap tabParams;
	bool tabMenuShown, ipMenuShown;

	UserMap userMap;
	TaskQueue tasks;
	bool updateUsers;

	StringMap ucLineParams;

	enum { MAX_CLIENT_LINES = 5 };
	TStringList lastLinesList;
	tstring lastLines;
	HWND hWndLastLines;

	static int columnIndexes[COLUMN_LAST];
	static int columnSizes[COLUMN_LAST];

	bool updateUser(const UserTask& u, bool &bResort);
	void removeUser(const UserPtr& aUser);

	UserInfo* findUser(const tstring& nick);

	void addAsFavorite();
	void removeFavoriteHub();

	int hubchatusersplit;
	string filterstring;
	int filtercolumn;
	bool filterUser(const Identity& id);
	void updateUserList();

	void PreparePopupMenu(HWND hWndCtrl, tstring& sNick, HMENU pMenu);
	bool ShowUserList;
	string sColumsOrder;
    string sColumsWidth;
    string sColumsVisible;

	void clearUserList();
	void clearTaskList();

	// FavoriteManagerListener
	void on(FavoriteManagerListener::UserAdded, const FavoriteUser& /*aUser*/) throw();
	void on(FavoriteManagerListener::UserRemoved, const FavoriteUser& /*aUser*/) throw();
	void resortForFavsFirst(bool justDoIt = false);

	// TimerManagerListener
	void on(TimerManagerListener::Second, uint64_t /*aTick*/) throw();
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

	// ClientListener
	void on(Connecting, Client*) throw();
	void on(Connected, Client*) throw();
	void on(UserUpdated, Client*, const OnlineUser&) throw();
	void on(UsersUpdated, Client*, const OnlineUser::List&) throw();
	void on(ClientListener::UserRemoved, Client*, const OnlineUser&) throw();
	void on(Redirect, Client*, const string&) throw();
	void on(Failed, Client*, const string&) throw();
	void on(GetPassword, Client*) throw();
	void on(HubUpdated, Client*) throw();
	void on(Message, Client*, const OnlineUser&, const string&) throw();
	void on(PrivateMessage, Client*, const OnlineUser&, const OnlineUser&, const string&) throw();
	void on(PrivateMessageNick, Client*, const string&, const OnlineUser&, const string&) throw();
	void on(NickTaken, Client*) throw();
	void on(SearchFlood, Client*, const string&) throw();

	void speak(Tasks s) { tasks.add(s, NULL); ::PostMessage(m_hWnd, WM_SPEAKER, 0, 0); }
	void speak(Tasks s, const string& msg) { tasks.add(s, new StringTask(msg)); ::PostMessage(m_hWnd, WM_SPEAKER, 0, 0); }
	void speak(Tasks s, const OnlineUser& u) { tasks.add(s, new UserTask(u)); updateUsers = true; }
	void speak(Tasks s, const OnlineUser& u, const string& line) {
		tasks.add(s, new MsgTask(u, line));
		::PostMessage(m_hWnd, WM_SPEAKER, 0, 0);
	}
	void speak(Tasks s, const OnlineUser& u, const OnlineUser& by, const string& line) {
		tasks.add(s, new PMTask(u, by, line));
		::PostMessage(m_hWnd, WM_SPEAKER, 0, 0);
	}
	void speak(Tasks s, const string& nick, const OnlineUser& by, const string& line) {
		tasks.add(s, new PMNickTask(nick, by, line));
		::PostMessage(m_hWnd, WM_SPEAKER, 0, 0);
	}

    static LRESULT CALLBACK ShowUsersWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK EditWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK RichEditWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK FilterWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

    HWND GetWindowHandle();
    void UpdateSplitterParts();
};

#endif // HUB_FRAME_H

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
