/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "Resource.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "LimitPage.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/SettingsManager.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "WinUtil.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

LimitPage * pLimitPage = NULL;
//----------------------------------------------------------------------------------------------------------------------------------------------------

LimitPage::LimitPage(SettingsManager *s) : PropPage(s, IDD_LIMITPAGE) {
    pLimitPage = this;
	title = _tcsdup((TSTRING(SETTINGS_ADVANCED) + _T('\\') + TSTRING(SETTINGS_LIMIT)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)LimitPageProc;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LimitPage::~LimitPage() {
    pLimitPage = NULL;
	free(title);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

BOOL CALLBACK LimitPage::LimitPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM /*lParam*/) {
    if(uMsg == WM_INITDIALOG) {
        pLimitPage->m_hWnd = hWnd;
        pLimitPage->onInitDialog();
        return TRUE;
    } else if(uMsg == WM_COMMAND) {
        switch(LOWORD(wParam)) {
            case IDC_TIME_LIMITING:
            case IDC_THROTTLE_ENABLE:
            case IDC_DISCONNECTING_ENABLE:
                pLimitPage->fixControls();
                return TRUE;
        }
    }

	return FALSE;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

PropPage::TextItem LimitPage::texts[] = {
	{ IDC_THROTTLE_ENABLE, ResourceManager::SETCZDC_ENABLE_LIMITING },
	{ IDC_CZDC_TRANSFER_LIMITING, ResourceManager::SETCZDC_TRANSFER_LIMITING },
	{ IDC_CZDC_UP_SPEEED, ResourceManager::SETCZDC_UPLOAD_SPEED },
	{ IDC_CZDC_UP_SPEEED1, ResourceManager::SETCZDC_UPLOAD_SPEED },
	{ IDC_SETTINGS_KBPS1, ResourceManager::KBPS },
	{ IDC_SETTINGS_KBPS2, ResourceManager::KBPS },
	{ IDC_SETTINGS_KBPS3, ResourceManager::KBPS },
	{ IDC_SETTINGS_KBPS4, ResourceManager::KBPS },
	{ IDC_SETTINGS_KBPS5, ResourceManager::KBPS },
	{ IDC_SETTINGS_KBPS6, ResourceManager::KBPS },
	{ IDC_SETTINGS_MINUTES, ResourceManager::MINUTES },
	{ IDC_CZDC_DW_SPEEED, ResourceManager::SETCZDC_DOWNLOAD_SPEED },
	{ IDC_CZDC_DW_SPEEED1, ResourceManager::SETCZDC_DOWNLOAD_SPEED },
	{ IDC_TIME_LIMITING, ResourceManager::SETCZDC_ALTERNATE_LIMITING },
	{ IDC_CZDC_TO, ResourceManager::SETCZDC_TO },
	{ IDC_CZDC_SECONDARY_TRANSFER, ResourceManager::SETCZDC_SECONDARY_LIMITING },
	{ IDC_CZDC_UP_NOTE, ResourceManager::SETCZDC_NOTE_UPLOAD },
	{ IDC_CZDC_DW_NOTE, ResourceManager::SETCZDC_NOTE_DOWNLOAD },
	{ IDC_CZDC_SLOW_DISCONNECT, ResourceManager::SETCZDC_SLOW_DISCONNECT },
	{ IDC_CZDC_I_DOWN_SPEED, ResourceManager::SETCZDC_I_DOWN_SPEED },
	{ IDC_CZDC_TIME_DOWN, ResourceManager::SETCZDC_TIME_DOWN },
	{ IDC_CZDC_H_DOWN_SPEED, ResourceManager::SETCZDC_H_DOWN_SPEED },
	{ IDC_DISCONNECTING_ENABLE, ResourceManager::SETCZDC_DISCONNECTING_ENABLE },
	{ IDC_CZDC_MIN_FILE_SIZE, ResourceManager::SETCZDC_MIN_FILE_SIZE },
	{ IDC_SETTINGS_MB, ResourceManager::MB },
	{ IDC_REMOVE_SLOW_USER, ResourceManager::SETCZDC_REMOVE_SLOW_USER },
	{ IDC_CZDC_ALTERNATE_SLOTS, ResourceManager::SETCZDC_ALTERNATE_SLOTS },
	{ IDC_SETTINGS_ZERO_DISABLE, ResourceManager::SETCZDC_ZERO_DISABLE },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
}; 
//----------------------------------------------------------------------------------------------------------------------------------------------------

PropPage::Item LimitPage::items[] = {
	{ IDC_MX_UP_SP_LMT_NORMAL, SettingsManager::MAX_UPLOAD_SPEED_LIMIT_NORMAL, PropPage::T_INT },
	{ IDC_MX_DW_SP_LMT_NORMAL, SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT_NORMAL, PropPage::T_INT },
	{ IDC_TIME_LIMITING, SettingsManager::TIME_DEPENDENT_THROTTLE, PropPage::T_BOOL },
	{ IDC_MX_UP_SP_LMT_TIME, SettingsManager::MAX_UPLOAD_SPEED_LIMIT_TIME, PropPage::T_INT },
	{ IDC_MX_DW_SP_LMT_TIME, SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT_TIME, PropPage::T_INT },
	{ IDC_BW_START_TIME, SettingsManager::BANDWIDTH_LIMIT_START, PropPage::T_INT },
	{ IDC_BW_END_TIME, SettingsManager::BANDWIDTH_LIMIT_END, PropPage::T_INT },
	{ IDC_THROTTLE_ENABLE, SettingsManager::THROTTLE_ENABLE, PropPage::T_BOOL },
	{ IDC_I_DOWN_SPEED, SettingsManager::I_DOWN_SPEED, PropPage::T_INT },
	{ IDC_TIME_DOWN, SettingsManager::DOWN_TIME, PropPage::T_INT },
	{ IDC_H_DOWN_SPEED, SettingsManager::H_DOWN_SPEED, PropPage::T_INT },
	{ IDC_DISCONNECTING_ENABLE, SettingsManager::DISCONNECTING_ENABLE, PropPage::T_BOOL },
	{ IDC_MIN_FILE_SIZE, SettingsManager::MIN_FILE_SIZE, PropPage::T_INT },
	{ IDC_REMOVE_SLOW_USER, SettingsManager::REMOVE_SLOW_USER, PropPage::T_BOOL },
	{ IDC_ALTERNATE_UPLOAD_SLOTS, SettingsManager::ALTERNATE_UPLOAD_SLOTS, PropPage::T_INT },
	{ 0, 0, PropPage::T_END }
};
//----------------------------------------------------------------------------------------------------------------------------------------------------

void LimitPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	PropPage::read(m_hWnd, items);

	::SendMessage(::GetDlgItem(m_hWnd, IDC_I_DOWN_SPEED_SPIN), UDM_SETRANGE32, 0, 99999);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_TIME_DOWN_SPIN), UDM_SETRANGE32, 1, 99999);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_H_DOWN_SPEED_SPIN), UDM_SETRANGE32, 0, 99999);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_UPLOADSPEEDSPIN), UDM_SETRANGE32, 0, 99999);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_DOWNLOADSPEEDSPIN), UDM_SETRANGE32, 0, 99999);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_UPLOADSPEEDSPIN_TIME), UDM_SETRANGE32, 0, 99999);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_DOWNLOADSPEEDSPIN_TIME), UDM_SETRANGE32, 0, 99999);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_MIN_FILE_SIZE_SPIN), UDM_SETRANGE32, 0, 999);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_ALTERNATE_UPLOAD_SLOTS_SPIN), UDM_SETRANGE, 0, MAKELPARAM(100, 0));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_START_TIME), CB_ADDSTRING, 0, (LPARAM)_T("Midnight"));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_END_TIME), CB_ADDSTRING, 0, (LPARAM)_T("Midnight"));

	for(int i = 1; i < 12; ++i) {
		::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_START_TIME), CB_ADDSTRING, 0, (LPARAM)(Text::toT(Util::toString(i)) + _T(" AM")).c_str());
		::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_END_TIME), CB_ADDSTRING, 0, (LPARAM)(Text::toT(Util::toString(i)) + _T(" AM")).c_str());
	}
	::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_START_TIME), CB_ADDSTRING, 0, (LPARAM)_T("Noon"));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_END_TIME), CB_ADDSTRING, 0, (LPARAM)_T("Noon"));
	for(int i = 1; i < 12; ++i) {
		::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_START_TIME), CB_ADDSTRING, 0, (LPARAM)(Text::toT(Util::toString(i)) + _T(" PM")).c_str());
		::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_END_TIME), CB_ADDSTRING, 0, (LPARAM)(Text::toT(Util::toString(i)) + _T(" PM")).c_str());
	}

	::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_START_TIME), CB_SETCURSEL, SETTING(BANDWIDTH_LIMIT_START), 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_END_TIME), CB_SETCURSEL, SETTING(BANDWIDTH_LIMIT_START), 0);

	fixControls();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void LimitPage::write() {
	PropPage::write(m_hWnd, items);

	if( SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) > 0) {
		if( SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) < ((2 * Util::getSlots()) + 3) ) {
			settings->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT_NORMAL, ((2 * Util::getSlots()) + 3) );
		}
		if ( (SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL) > ( SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) * 7)) || ( SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL) == 0) ) {
			settings->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT_NORMAL, (SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL)*7) );
		}
	}

	if( SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) > 0) {
		if( SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) < ((2 * Util::getSlots()) + 3) ) {
			settings->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT_TIME, ((2 * Util::getSlots()) + 3) );
		}
		if ( (SETTING(MAX_DOWNLOAD_SPEED_LIMIT_TIME) > ( SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) * 7)) || ( SETTING(MAX_DOWNLOAD_SPEED_LIMIT_TIME) == 0) ) {
			settings->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT_TIME, (SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME)*7) );
		}
	}

	// Do specialized writing here
	// settings->set(XX, YY);

	settings->set(SettingsManager::BANDWIDTH_LIMIT_START, (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_START_TIME), CB_GETCURSEL, 0, 0));
	settings->set(SettingsManager::BANDWIDTH_LIMIT_END, (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_BW_END_TIME), CB_GETCURSEL, 0, 0));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void LimitPage::fixControls() {
	bool state = (::IsDlgButtonChecked(m_hWnd, IDC_THROTTLE_ENABLE) != 0);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_MX_UP_SP_LMT_NORMAL), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_UPLOADSPEEDSPIN), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_MX_DW_SP_LMT_NORMAL), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_DOWNLOADSPEEDSPIN), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_TIME_LIMITING), state);

	state = ((::IsDlgButtonChecked(m_hWnd, IDC_THROTTLE_ENABLE) != 0) && (::IsDlgButtonChecked(m_hWnd, IDC_TIME_LIMITING) != 0));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_BW_START_TIME), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_BW_END_TIME), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_MX_UP_SP_LMT_TIME), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_UPLOADSPEEDSPIN_TIME), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_MX_DW_SP_LMT_TIME), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_DOWNLOADSPEEDSPIN_TIME), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_ALTERNATE_UPLOAD_SLOTS), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_ALTERNATE_UPLOAD_SLOTS_SPIN), state);

	state = (::IsDlgButtonChecked(m_hWnd, IDC_DISCONNECTING_ENABLE) != 0);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_I_DOWN_SPEED), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_I_DOWN_SPEED_SPIN), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_TIME_DOWN), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_TIME_DOWN_SPIN), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_H_DOWN_SPEED), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_H_DOWN_SPEED_SPIN), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_MIN_FILE_SIZE), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_MIN_FILE_SIZE_SPIN), state);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_REMOVE_SLOW_USER), state);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
