/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "PrivateFrame.h"
#include "SearchFrame.h"
#include "WinUtil.h"
#include "FlatTabCtrl.h"

#include "../client/Client.h"
#include "../client/ClientManager.h"
#include "../client/Util.h"
#include "../client/LogManager.h"
#include "../client/UploadManager.h"
#include "../client/ShareManager.h"
#include "../client/FavoriteManager.h"
#include "../client/QueueManager.h"
#include "../client/StringTokenizer.h"

PrivateFrame::FrameMap PrivateFrame::frames;
tstring pSelectedLine = Util::emptyStringT;
tstring pSelectedURL = Util::emptyStringT;
ATOM PrivateFrame::m_aAtom = 0;
UINT PrivateFrame::m_uiResourceID = IDR_PRIVATE;

PrivateFrame::PrivateFrame(const UserPtr& aUser) : user(aUser), closed(false), isoffline(false),
    curCommandPosition(0), textMenu(NULL), tabMenu(NULL) {
    RegisterWindowClass(m_aAtom, PrivateFrameWndProc, _T("PrivateFrame"), m_uiResourceID);
}

PrivateFrame::~PrivateFrame() {
    if(textMenu != NULL) {
        ::DestroyMenu(textMenu);
    }

    if(tabMenu != NULL) {
        ::DestroyMenu(tabMenu);
    }
}

LRESULT CALLBACK PrivateFrame::PrivateFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	PrivateFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (PrivateFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (PrivateFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->PrivateFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT PrivateFrame::PrivateFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_SETFOCUS:
            onFocus();
            break;
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORSTATIC: {
            LRESULT lResult = onCtlColor(wParam, lParam);
            if(lResult != 0) {
                return lResult;
            }
            break;
        }
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_SPEAKER:
            updateTitle();
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_CONTEXTMENU:
            if(onContextMenu(lParam) == true) {
                return 0;
            }
            break;
        case FTM_CONTEXTMENU:
            onTabContextMenu(lParam);
            return TRUE;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_GETLIST:
                    onGetList();
                    return 0;
                case IDC_MATCH_QUEUE:
                    onMatchQueue();
                    return 0;
                case IDC_GRANTSLOT:
                    onGrantSlot();
                    return 0;
                case IDC_GRANTSLOT_HOUR:
                    onGrantSlotHour();
                    return 0;
                case IDC_GRANTSLOT_DAY:
                    onGrantSlotDay();
                    return 0;
                case IDC_GRANTSLOT_WEEK:
                    onGrantSlotWeek();
                    return 0;
                case IDC_UNGRANTSLOT:
                    onUnGrantSlot();
                    return 0;
                case IDC_ADD_TO_FAVORITES:
                    onAddToFavorites();
                    return 0;
                case IDC_SEND_MESSAGE:
                    onEnter();
                    return 0;
                case IDC_CLOSE_WINDOW:
                    onCloseWindow();
                    return 0;
                case ID_EDIT_COPY:
                    onEditCopy();
                    return 0;
                case ID_EDIT_SELECT_ALL:
                    onEditSelectAll();
                    return 0;
                case ID_EDIT_CLEAR_ALL:
                    onEditClearAll();
                    return 0;
                case IDC_COPY_ACTUAL_LINE:
                    onCopyActualLine();
                    return 0;
                case IDC_OPEN_USER_LOG:
                    onOpenUserLog();
                    return 0;
                case IDC_COPY_URL:
                    onCopyURL();
                    return 0;
                case IDC_SEARCH_DC:
                case IDC_SEARCH_GOOGLE:
                case IDC_SEARCH_YAHOO:
                case IDC_SEARCH_IMDB:
                case IDC_SEARCH_CSFD:
                case IDC_SEARCH_WIKI:
                    onSearch(LOWORD(wParam));
                    return 0;
            }
   
            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_CLIENT && ((LPNMHDR)lParam)->code == EN_LINK) {
                onClientEnLink((LPNMHDR)lParam);
                return 0;
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

	if(UCHandlerProc(uMsg, wParam) == true) {
		return 0;
    }

	LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK PrivateFrame::MessageWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CHAR:
        case WM_KEYDOWN:
        case WM_KEYUP: {
            PrivateFrame * pParent = (PrivateFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
            if(pParent != NULL) {
                if(pParent->onChar(uMsg, wParam) == TRUE) {
                    return 0;
                }
            }
        }
    }

    return ::CallWindowProc(OldEditProc, hWnd, uMsg, wParam, lParam);
}

void PrivateFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 255, 255), IDR_PRIVATE, IDR_PRIVATE_OFF);
    created = true;

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, 0, 0, 0, 0, m_hWnd, 0, g_hInstance, NULL);

	ctrlClient.m_hWnd = CreateWindowEx(WS_EX_CLIENTEDGE, RICHEDIT_CLASS, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VSCROLL | ES_AUTOVSCROLL | ES_MULTILINE |
        ES_NOHIDESEL | ES_READONLY, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_CLIENT, g_hInstance, NULL);

	::SendMessage(ctrlClient.m_hWnd, EM_EXLIMITTEXT, 0, 0);
	::SendMessage(ctrlClient.m_hWnd, EM_SETLANGOPTIONS, 0, 0);
	::SendMessage(ctrlClient.m_hWnd, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	::SendMessage(ctrlClient.m_hWnd, EM_SETBKGNDCOLOR, 0, WinUtil::bgColor);
	::SendMessage(ctrlClient.m_hWnd, EM_AUTOURLDETECT, FALSE, 0);
	::SendMessage(ctrlClient.m_hWnd, EM_SETEVENTMASK, 0, (DWORD)::SendMessage(ctrlClient.m_hWnd, EM_GETEVENTMASK, 0, 0) | ENM_LINK);

    hWndMessage = CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | ES_MULTILINE | ES_AUTOVSCROLL,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0, g_hInstance, NULL);

    ::SetWindowLongPtr(hWndMessage, GWLP_USERDATA, (LONG_PTR)this);
    OldEditProc = (WNDPROC)::SetWindowLongPtr(hWndMessage, GWLP_WNDPROC, (LONG_PTR)MessageWndProc);

	::SendMessage(hWndMessage, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
	::SendMessage(hWndMessage, EM_SETBKGNDCOLOR, 0, WinUtil::bgColor);
	::SendMessage(hWndMessage, EM_SETLIMITTEXT, 9999, 0);

	tabMenu = ::CreatePopupMenu();

	if(BOOLSETTING(LOG_PRIVATE_CHAT)) {
        ::AppendMenu(tabMenu, MF_STRING, IDC_OPEN_USER_LOG, CTSTRING(OPEN_USER_LOG));
		::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
	}

    ::AppendMenu(tabMenu, MF_STRING, ID_EDIT_CLEAR_ALL, CTSTRING(CLEAR));
	::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(tabMenu, MF_STRING, IDC_GETLIST, CTSTRING(GET_FILE_LIST));
	::AppendMenu(tabMenu, MF_STRING, IDC_MATCH_QUEUE, CTSTRING(MATCH_QUEUE));

	WinUtil::AddGrantSubMenu(tabMenu);

	::AppendMenu(tabMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));

	::PostMessage(m_hWnd, WM_SPEAKER, USER_UPDATED, 0);

	ClientManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);

	if(SETTING(PM_LOG_LINES) > 0) {
		readLog();
	}
}

void PrivateFrame::gotMessage(Identity& aUser, Identity& byUser, const tstring& aMessage) {
	PrivateFrame* p = NULL;
	const UserPtr& bUser = aUser.getUser();

	FrameIter i = frames.find(bUser);
	if(i == frames.end()) {
		p = new PrivateFrame(bUser);
		frames[bUser] = p;
		p->addLine(byUser, aMessage, WinUtil::m_ChatTextGeneral);
		if(Util::getAway()) {
			if(aUser.getConnection().empty() == false || aUser.getBytesShared() != 0)
				p->sendMessage(Text::toT(Util::getAwayMessage()));
		}

		if((BOOLSETTING(PRIVATE_MESSAGE_BEEP) || BOOLSETTING(PRIVATE_MESSAGE_BEEP_OPEN)) && BOOLSETTING(MUTE_SOUND) == false) {
			if(SETTING(BEEPFILE).empty()) {
				MessageBeep(MB_OK);
			} else {
				::PlaySound(Text::toT(SETTING(BEEPFILE)).c_str(), NULL, SND_FILENAME | SND_ASYNC);
			}
		}
	} else {
		if(BOOLSETTING(PRIVATE_MESSAGE_BEEP) && BOOLSETTING(MUTE_SOUND) == false) {
			if(SETTING(BEEPFILE).empty()) {
				MessageBeep(MB_OK);
			} else {
				::PlaySound(Text::toT(SETTING(BEEPFILE)).c_str(), NULL, SND_FILENAME | SND_ASYNC);
			}
		}
		i->second->addLine(byUser, aMessage, WinUtil::m_ChatTextGeneral);
	}
}

void PrivateFrame::openWindow(const UserPtr& aUser, const tstring& msg) {
	PrivateFrame* p = NULL;
	FrameIter i = frames.find(aUser);
	if(i == frames.end()) {
		p = new PrivateFrame(aUser);
		frames[aUser] = p;
		p->CreateEx(p, m_aAtom, m_uiResourceID);
	} else {
		p = i->second;
		if(::IsIconic(p->m_hWnd))
			::ShowWindow(p->m_hWnd, SW_RESTORE);
		::SendMessage(WinUtil::mdiClient, WM_MDIACTIVATE, (WPARAM)p->m_hWnd, 0);
	}

	if(!msg.empty()) {
		p->sendMessage(msg);
	}
}

BOOL PrivateFrame::onChar(UINT uMsg, WPARAM wParam) {
	if(uMsg != WM_KEYDOWN) {
		switch(wParam) {
			case VK_RETURN:
				if(WinUtil::isShift() || WinUtil::isCtrl() ||  WinUtil::isAlt()) {
					return FALSE;
				}

				break;
            case VK_TAB:
				return TRUE;
            default:
                return FALSE;
		}

		return TRUE;
	}

	switch(wParam) {
        case VK_RETURN:
            if( (GetKeyState(VK_SHIFT) & 0x8000) ||
                (GetKeyState(VK_CONTROL) & 0x8000) ||
                (GetKeyState(VK_MENU) & 0x8000) ) {
                return FALSE;
            } else {
                if(uMsg == WM_KEYDOWN) {
                    onEnter();
                }
            }

            break;
        case VK_UP:
            if((GetKeyState(VK_MENU) & 0x8000) || (((GetKeyState(VK_CONTROL) & 0x8000) == 0) ^ (BOOLSETTING(SETTINGS_USE_CTRL_FOR_LINE_HISTORY) == true))) {
                //scroll up in chat command history
                //currently beyond the last command?
                if(curCommandPosition > 0) {
                    //check whether current command needs to be saved
                    if(curCommandPosition == prevCommands.size()) {
                        auto_ptr<TCHAR> messageContents(new TCHAR[::GetWindowTextLength(hWndMessage)+2]);
                        ::GetWindowText(hWndMessage, messageContents.get(), ::GetWindowTextLength(hWndMessage)+1);
                        currentCommand = tstring(messageContents.get());
                    }

                    //replace current chat buffer with current command
                    ::SetWindowText(hWndMessage, prevCommands[--curCommandPosition].c_str());
                }
            } else {
                return FALSE;
            }

            break;
        case VK_DOWN:
            if((GetKeyState(VK_MENU) & 0x8000) || (((GetKeyState(VK_CONTROL) & 0x8000) == 0) ^ (BOOLSETTING(SETTINGS_USE_CTRL_FOR_LINE_HISTORY) == true))) {
                //scroll down in chat command history
                //currently beyond the last command?
                if(curCommandPosition + 1 < prevCommands.size()) {
                    //replace current chat buffer with current command
                    ::SetWindowText(hWndMessage, prevCommands[++curCommandPosition].c_str());
                } else if(curCommandPosition + 1 == prevCommands.size()) {
                    //revert to last saved, unfinished command
                    ::SetWindowText(hWndMessage, currentCommand.c_str());
                    ++curCommandPosition;
                }
            } else {
                return FALSE;
            }

            break;
        case VK_HOME:
            if(!prevCommands.empty() && (GetKeyState(VK_CONTROL) & 0x8000) || (GetKeyState(VK_MENU) & 0x8000)) {
                curCommandPosition = 0;
                auto_ptr<TCHAR> messageContents(new TCHAR[::GetWindowTextLength(hWndMessage)+2]);
                ::GetWindowText(hWndMessage, messageContents.get(), ::GetWindowTextLength(hWndMessage)+1);
                currentCommand = tstring(messageContents.get());
                ::SetWindowText(hWndMessage, prevCommands[curCommandPosition].c_str());
            } else {
                return FALSE;
            }

            break;
        case VK_END:
            if((GetKeyState(VK_CONTROL) & 0x8000) || (GetKeyState(VK_MENU) & 0x8000)) {
                curCommandPosition = prevCommands.size();
                ::SetWindowText(hWndMessage, currentCommand.c_str());
            } else {
                return FALSE;
            }

            break;
        default:
			return FALSE;
    }

	return TRUE;
}

void PrivateFrame::onEnter() {
	bool resetText = true;

	if(::GetWindowTextLength(hWndMessage) > 0) {
		AutoArray<TCHAR> msg(::GetWindowTextLength(hWndMessage)+1);
		::GetWindowText(hWndMessage, msg, ::GetWindowTextLength(hWndMessage)+1);
		tstring s(msg, ::GetWindowTextLength(hWndMessage));

		// save command in history, reset current buffer pointer to the newest command
		curCommandPosition = prevCommands.size();		//this places it one position beyond a legal subscript
		if(!curCommandPosition || curCommandPosition > 0 && prevCommands[curCommandPosition - 1] != s) {
			++curCommandPosition;
			prevCommands.push_back(s);
		}
		currentCommand = Util::emptyStringT;

		// Process special commands
		if(s[0] == '/') {
			tstring m = s;
			tstring param;
			tstring message;
			tstring status;
			if(WinUtil::checkCommand(s, param, message, status)) {
				if(!message.empty()) {
					sendMessage(message);
				}
				if(!status.empty()) {
					addClientLine(status);
				}
			} else if((Util::stricmp(s.c_str(), _T("clear")) == 0) || (Util::stricmp(s.c_str(), _T("cls")) == 0)) {
				ctrlClient.ClearChat();
			} else if(Util::stricmp(s.c_str(), _T("grant")) == 0) {
				UploadManager::getInstance()->reserveSlot(getUser());
				addClientLine(TSTRING(SLOT_GRANTED));
			} else if(Util::stricmp(s.c_str(), _T("close")) == 0) {
				::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
			} else if((Util::stricmp(s.c_str(), _T("favorite")) == 0) || (Util::stricmp(s.c_str(), _T("fav")) == 0)) {
				FavoriteManager::getInstance()->addFavoriteUser(getUser());
				addStatus(TSTRING(FAVORITE_USER_ADDED), WinUtil::m_ChatTextGeneral);
			} else if(Util::stricmp(s.c_str(), _T("getlist")) == 0) {
				onGetList();
			} else if(Util::stricmp(s.c_str(), _T("log")) == 0) {
				StringMap params;

				params["hubNI"] = Util::toString(ClientManager::getInstance()->getHubNames(user->getCID()));
				params["hubURL"] = Util::toString(ClientManager::getInstance()->getHubs(user->getCID()));
				params["userCID"] = user->getCID().toBase32(); 
				params["userNI"] = ClientManager::getInstance()->getNicks(user->getCID())[0];
				params["myCID"] = ClientManager::getInstance()->getMe()->getCID().toBase32();
				WinUtil::openFile(Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_PRIVATE_CHAT), params, true))));
			} else if((Util::stricmp(s.c_str(), _T("stat")) == 0) || (Util::stricmp(s.c_str(), _T("stats")) == 0)) {
				sendMessage(Text::toT(WinUtil::generateStats()));
			} else if(Util::stricmp(s.c_str(), _T("help")) == 0) {
				addStatus(_T("*** ") + WinUtil::commands + _T(", /getlist, /clear, /grant, /close, /favorite, /winamp, /log  <system, downloads, uploads>"), WinUtil::m_ChatTextSystem);
			} else {
				if(user->isOnline()) {
					sendMessage(m);
				} else {
					::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)CTSTRING(USER_WENT_OFFLINE));
					resetText = false;
				}
			}
		} else {
			if(user->isOnline()) {
				sendMessage(s);
			} else {
                ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)CTSTRING(USER_WENT_OFFLINE));
				resetText = false;
			}
		}
		if(resetText)
			::SetWindowText(hWndMessage, _T(""));
	} 
}

void PrivateFrame::addClientLine(const tstring& aLine) {
	if(!created) {
		CreateEx(this, m_aAtom, m_uiResourceID);
	}

	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)(_T("[") + Text::toT(Util::getShortTimeString()) + _T("] ") + aLine).c_str());

	if (BOOLSETTING(BOLD_PM)) {
		setDirty();
	}
}

void PrivateFrame::sendMessage(const tstring& msg) {
	ClientManager::getInstance()->privateMessage(user, Text::fromT(msg));

	if(user->isOnline()) {
        Identity MyIdentity = ClientManager::getInstance()->GetMyIdentity(user);

        tstring s;
        if(msg[0] == _T('/') && tolower(msg[1]) == _T('m') && tolower(msg[2]) == _T('e') && msg[3] == _T(' ')) {
    		s = _T("* ") + Text::toT(MyIdentity.getNick()) + msg.substr(3, msg.size());;
        } else {
            s = _T("<") + Text::toT(MyIdentity.getNick()) + _T("> ") + msg;
        }
		addLine(MyIdentity, s, WinUtil::m_ChatTextGeneral);
	}
}

bool PrivateFrame::onClose() {
	if(!closed) {
		ClientManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		frames.erase(user);

		return false;
	}
}

void PrivateFrame::addLine(const Identity& byUser, const tstring& aLine, CHARFORMAT2& cf) {
	if(!created) {
		if(BOOLSETTING(POPUNDER_PM))
			WinUtil::hiddenCreateEx(this);
		else
			CreateEx(this, m_aAtom, m_uiResourceID);
	}

	ctrlClient.AdjustTextSize();

	RECT r;
	::GetClientRect(ctrlClient.m_hWnd, &r);

	if(BOOLSETTING(LOG_PRIVATE_CHAT)) {
		StringMap params;
		params["message"] = Text::fromT(aLine);
		params["hubNI"] = Util::toString(ClientManager::getInstance()->getHubNames(user->getCID()));
		params["hubURL"] = Util::toString(ClientManager::getInstance()->getHubs(user->getCID()));
		params["userCID"] = user->getCID().toBase32(); 
		params["userNI"] = ClientManager::getInstance()->getNicks(user->getCID())[0];
		params["myCID"] = ClientManager::getInstance()->getMe()->getCID().toBase32();
		LOG(LogManager::PM, params);
	}

	ctrlClient.AppendText(byUser, Text::toT(ClientManager::getInstance()->getMe()->getFirstNick()), 
		BOOLSETTING(TIME_STAMPS) ? Text::toT("[" + Util::getShortTimeString() + "] ") : _T(""), Util::emptyStringT, aLine, cf);
	
	addClientLine(CTSTRING(LAST_CHANGE) +  Text::toT(Util::getTimeString()));

	if(BOOLSETTING(BOLD_PM)) {
		setDirty();
	}
}

void PrivateFrame::addStatus(const tstring& aLine, CHARFORMAT2& cf) {
	ctrlClient.AppendStatusText(BOOLSETTING(TIME_STAMPS) ? Text::toT("[" + Util::getShortTimeString() + "] ") : _T(""), aLine, cf);
}

void PrivateFrame::onEditCopy() {
	::SendMessage(ctrlClient.m_hWnd, WM_COPY, 0, 0);
}

void PrivateFrame::onEditSelectAll() {
	ctrlClient.SetSelAll();
}

void PrivateFrame::onEditClearAll() {
	ctrlClient.ClearChat();
}

void PrivateFrame::onCopyActualLine() {
	if(pSelectedLine.size() != 0) {
		WinUtil::setClipboard(pSelectedLine);
	}
}

void PrivateFrame::onTabContextMenu(LPARAM lParam) {
	StringList userhubs = ClientManager::getInstance()->getHubs(user->getCID());

	if(userhubs.size() > 0) {
		prepareMenu(tabMenu, UserCommand::CONTEXT_CHAT, userhubs[0], ClientManager::getInstance()->isOp(ClientManager::getInstance()->getMe(), userhubs[0]));
	} else {
		prepareMenu(tabMenu, UserCommand::CONTEXT_CHAT, "", false);
	}

	if(!(::GetMenuState(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION) & MF_SEPARATOR)) {	
		::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
	}

	::AppendMenu(tabMenu, MF_STRING, IDC_CLOSE_WINDOW, CTSTRING(CLOSE));

	::TrackPopupMenuEx(tabMenu, TPM_LEFTALIGN | (BOOLSETTING(TABS_ON_TOP) == false ? TPM_BOTTOMALIGN : TPM_TOPALIGN) | TPM_RIGHTBUTTON,
        GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), m_hWnd, NULL);

	::RemoveMenu(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION);
	::RemoveMenu(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION);

	cleanMenu(tabMenu);
}

void PrivateFrame::runUserCommand(UserCommand& uc) {
	if(!WinUtil::getUCParams(m_hWnd, uc, ucLineParams))
		return;

	StringMap ucParams = ucLineParams;

	ClientManager::getInstance()->userCommand(user, uc, ucParams, true);
}

void PrivateFrame::onGetList() {
	try {
		QueueManager::getInstance()->addList(user, QueueItem::FLAG_CLIENT_VIEW);
	} catch(const Exception& e) {
		addClientLine(Text::toT(e.getError()));
	}
}

void PrivateFrame::onMatchQueue() {
	try {
		QueueManager::getInstance()->addList(user, QueueItem::FLAG_MATCH_QUEUE);
	} catch(const Exception& e) {
		addClientLine(Text::toT(e.getError()));
	}
}

void PrivateFrame::onGrantSlot() {
	UploadManager::getInstance()->reserveSlot(user);
}

void PrivateFrame::onGrantSlotHour() {
	UploadManager::getInstance()->reserveSlotHour(user);
}

void PrivateFrame::onGrantSlotDay() {
	UploadManager::getInstance()->reserveSlotDay(user);
}

void PrivateFrame::onGrantSlotWeek() {
	UploadManager::getInstance()->reserveSlotWeek(user);
}

void PrivateFrame::onUnGrantSlot() {
	UploadManager::getInstance()->unreserveSlot(user);
}

void PrivateFrame::onAddToFavorites() {
	FavoriteManager::getInstance()->addFavoriteUser(user);
}

void PrivateFrame::UpdateLayout() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[1];
		::GetClientRect(m_hWndStatusBar, &sr);

		w[0] = sr.right - 16;

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 1, (LPARAM)w);
	}

	int h = WinUtil::fontHeight + 4;

	RECT rc = rect;
	rc.bottom -= (2*h) + 10;
	::MoveWindow(ctrlClient.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc = rect;
	rc.bottom -= 2;
	rc.top = rc.bottom - (2*h) - 5;
	rc.left +=2;
	rc.right -=2;
	::MoveWindow(hWndMessage, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

}

void PrivateFrame::updateTitle() {
	pair<tstring, bool> hubs = WinUtil::getHubNames(user);
	if(hubs.second) {
		unsetIconState();
		setTabColor(RGB(0, 255,	255));
		if(isoffline) {
			if(BOOLSETTING(STATUS_IN_CHAT)) {
				addStatus(_T(" *** ") + TSTRING(USER_WENT_ONLINE) + _T(" [") + Text::toT(user->getFirstNick()) + _T("] ***"), WinUtil::m_ChatTextServer);
			} else {
				addClientLine(_T(" *** ") + TSTRING(USER_WENT_ONLINE) + _T(" [") + Text::toT(user->getFirstNick()) + _T("] ***"));
			}
		}
		isoffline = false;
	} else {
		setIconState();
		setTabColor(RGB(255, 0, 0));
		if(BOOLSETTING(STATUS_IN_CHAT)) {
			addStatus(_T(" *** ") + TSTRING(USER_WENT_OFFLINE) + _T(" [") + Text::toT(user->getFirstNick()) + _T("] ***"), WinUtil::m_ChatTextServer);
		} else {
			addClientLine(_T(" *** ") + TSTRING(USER_WENT_OFFLINE) + _T(" [") + Text::toT(user->getFirstNick()) + _T("] ***"));
		}
		isoffline = true;
	}

	::SetWindowText(m_hWnd, (WinUtil::getNicks(user) + _T(" - ") + hubs.first).c_str());
}

void PrivateFrame::readLog() {
	StringMap params;	
	params["hubNI"] = Util::toString(ClientManager::getInstance()->getHubNames(user->getCID()));
	params["hubURL"] = Util::toString(ClientManager::getInstance()->getHubs(user->getCID()));
	params["userCID"] = user->getCID().toBase32(); 
	params["userNI"] = ClientManager::getInstance()->getNicks(user->getCID())[0];
	params["myCID"] = ClientManager::getInstance()->getMe()->getCID().toBase32();

	string path = Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_PRIVATE_CHAT), params, true));

	string sMyNick = ClientManager::getInstance()->GetMyNick(user);

	try {
		File f(path, File::READ, File::OPEN);
		
		int64_t size = f.getSize();

		if(size > 32*1024) {
			f.setPos(size - 32*1024);
		}
		string buf = f.read(32*1024);
		StringList lines;

		if(Util::strnicmp(buf.c_str(), "\xef\xbb\xbf", 3) == 0)
			lines = StringTokenizer<string>(buf.substr(3), "\r\n").getTokens();
		else
			lines = StringTokenizer<string>(buf, "\r\n").getTokens();

		size_t linesCount = lines.size();

		size_t i = linesCount > (size_t)(SETTING(PM_LOG_LINES) + 1) ? linesCount - (SETTING(PM_LOG_LINES)+1) : 0;

		for(; i < linesCount-1; ++i){
			if(lines[i].empty() == false)
				ctrlClient.AppendText(Text::toT(sMyNick), _T(""), Text::toT(lines[i]), WinUtil::m_ChatTextLog, _T(""));
		}

		f.close();
	} catch(const FileException&) {
		// ...
	}
}

bool PrivateFrame::onContextMenu(LPARAM lParam) {
    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);

	if(iX == -1 && iY == -1) {
		ctrlClient.GetContextMenuPos(iX, iY);
	}

	pSelectedLine = _T("");

	bool bHitURL = ctrlClient.HitURL();
	if(bHitURL == false) {
		pSelectedURL = _T("");
    }

    POINT p = { iX, iY };
	::ScreenToClient(ctrlClient.m_hWnd, &p);

	int i = ctrlClient.CharFromPos(p);
	int line = (int)::SendMessage(ctrlClient.m_hWnd, EM_EXLINEFROMCHAR, 0, i);
	int c = LOWORD(i) - (int)::SendMessage(ctrlClient.m_hWnd, EM_LINEINDEX, line, 0);
	int len = (int)::SendMessage(ctrlClient.m_hWnd, EM_LINELENGTH, i, 0) + 1;
	if(len < 3) {
		return false;
    }

    AutoArray<TCHAR> buf(len);
	ctrlClient.GetLine(line, buf, len);
	tstring x = tstring(buf, len-1);
	string::size_type start = x.find_last_of(_T(" <\t\r\n"), c);
	if(start == string::npos) {
        start = 0;
    }
	tstring nick = Text::toT(user->getFirstNick());
	if(x.substr(start, (nick.length() + 2) ) == (_T("<") + nick + _T(">"))) {
		if(!user->isOnline()) {
			return false;
		}
		StringList userhubs = ClientManager::getInstance()->getHubs(user->getCID());
		if(userhubs.size() > 0) {
			prepareMenu(tabMenu, UserCommand::CONTEXT_CHAT, userhubs[0], ClientManager::getInstance()->isOp(ClientManager::getInstance()->getMe(), userhubs[0]));
		} else {
			prepareMenu(tabMenu, UserCommand::CONTEXT_CHAT, "", false);
		}
		if(!(::GetMenuState(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION) & MF_SEPARATOR)) {	
			::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
		}
		::AppendMenu(tabMenu, MF_STRING, IDC_CLOSE_WINDOW, CTSTRING(CLOSE));
		::TrackPopupMenuEx(tabMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);
        ::RemoveMenu(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION);
        ::RemoveMenu(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION);
		cleanMenu(tabMenu);
	} else {
		if(textMenu != NULL) {
			::DestroyMenu(textMenu);
			textMenu = NULL;
		}

		textMenu = ::CreatePopupMenu();

		long lSelBegin, lSelEnd; ctrlClient.GetSel(lSelBegin, lSelEnd);
        if(lSelEnd > lSelBegin) {
            ::AppendMenu(textMenu, MF_SEPARATOR, 0, NULL);
            WinUtil::AddSearchSubMenu(textMenu);
        }
        ::AppendMenu(textMenu, MF_STRING, ID_EDIT_COPY, CTSTRING(COPY));
		::AppendMenu(textMenu, MF_STRING, IDC_COPY_ACTUAL_LINE, CTSTRING(COPY_LINE));
		if(pSelectedURL.empty() == false) {
            ::AppendMenu(textMenu, MF_STRING, IDC_COPY_URL, CTSTRING(COPY_URL));
        }
		::AppendMenu(textMenu, MF_SEPARATOR, 0, NULL);
		::AppendMenu(textMenu, MF_STRING, ID_EDIT_SELECT_ALL, CTSTRING(SELECT_ALL));
		::AppendMenu(textMenu, MF_STRING, ID_EDIT_CLEAR_ALL, CTSTRING(CLEAR));

		pSelectedLine = ctrlClient.LineFromPos(p);
		::TrackPopupMenuEx(textMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);
    }

	return true;
}

void PrivateFrame::onClientEnLink(LPNMHDR pnmh) {
	ENLINK* pEL = (ENLINK*)pnmh;

	if(pEL->msg == WM_LBUTTONUP) {
		LONG lBegin = pEL->chrg.cpMin, lEnd = pEL->chrg.cpMax;
		TCHAR* sURLTemp = new TCHAR[(lEnd - lBegin)+1];
		if(sURLTemp != NULL) {
			ctrlClient.GetTextRange(lBegin, lEnd, sURLTemp);
			tstring sURL(sURLTemp, lEnd - lBegin);
			WinUtil::openLink(sURL);
			delete[] sURLTemp;
		}
	} else if(pEL->msg == WM_RBUTTONUP) {
		pSelectedURL = _T("");
		long lBegin = pEL->chrg.cpMin, lEnd = pEL->chrg.cpMax;
		TCHAR* sURLTemp = new TCHAR[(lEnd - lBegin)+1];
		if(sURLTemp != NULL) {
			ctrlClient.GetTextRange(lBegin, lEnd, sURLTemp);
			pSelectedURL = sURLTemp;
			delete[] sURLTemp;
		}

		ctrlClient.SetSel(lBegin, lEnd);
		::InvalidateRect(ctrlClient.m_hWnd, NULL, TRUE);
	}
}

void PrivateFrame::onOpenUserLog() {
	StringMap params;
	params["hubNI"] = Util::toString(ClientManager::getInstance()->getHubNames(user->getCID()));
	params["hubURL"] = Util::toString(ClientManager::getInstance()->getHubs(user->getCID()));
	params["userCID"] = user->getCID().toBase32(); 
	params["userNI"] = user->getFirstNick();
	params["myCID"] = ClientManager::getInstance()->getMe()->getCID().toBase32();
	string file = Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_PRIVATE_CHAT), params, true));
	if(File::FileExist(file)) {
		ShellExecute(NULL, NULL, Text::toT(file).c_str(), NULL, NULL, SW_SHOWNORMAL);
	} else {
		::MessageBox(m_hWnd, CTSTRING(NO_LOG_FOR_USER), CTSTRING(NO_LOG_FOR_USER), MB_OK);	  
	}
}

void PrivateFrame::onCopyURL() {
	if(pSelectedURL.size() != 0) {
		WinUtil::setClipboard(pSelectedURL);
	}
}

void PrivateFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void PrivateFrame::onRefreshSettings() {
	::SendMessage(ctrlClient.m_hWnd, EM_SETBKGNDCOLOR, 0, WinUtil::bgColor);
	::SendMessage(hWndMessage, EM_SETBKGNDCOLOR, 0, WinUtil::bgColor);
	::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
}

void PrivateFrame::closeDisconnected() {
    for(FrameIter i = frames.begin(); i != frames.end(); ++i) {
		if(!i->first->isOnline()) {
			::PostMessage(i->second->m_hWnd, WM_CLOSE, 0, 0);
		}
	}
}

void PrivateFrame::closeAll() {
    for(FrameIter i=frames.begin(); i!= frames.end(); ++i) {
		::PostMessage(i->second->m_hWnd, WM_CLOSE, 0, 0);
	}
}

void PrivateFrame::onSearch(WORD wID) {
    long lSelBegin, lSelEnd; ctrlClient.GetSel(lSelBegin, lSelEnd);
    if(lSelEnd > lSelBegin) { 
        long iLen = lSelEnd - lSelBegin; 
        TCHAR *sTemp = new TCHAR[iLen+1];
        ctrlClient.GetTextRange(lSelBegin, lSelEnd, sTemp);
        sTemp[iLen] = 0;
        tstring sSearchString(sTemp, iLen);
        delete [] sTemp;
    	switch(wID) {
    		case IDC_SEARCH_DC:
    			WinUtil::SearchOnDC(sSearchString);
    			break;
    		case IDC_SEARCH_GOOGLE:
    			WinUtil::SearchOnGoogle(sSearchString);
    			break;
    		case IDC_SEARCH_YAHOO:
    			WinUtil::SearchOnYahoo(sSearchString);
    			break;
    		case IDC_SEARCH_IMDB:
    			WinUtil::SearchOnIMDB(sSearchString);
    			break;
    		case IDC_SEARCH_CSFD:
    			WinUtil::SearchOnCSFD(sSearchString);
    			break;
    		case IDC_SEARCH_WIKI:
    			WinUtil::SearchOnWikipedia(sSearchString);
    			break;
    		default:
    			break;
    	}
    }
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
