/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#ifndef RECENTS_FRAME_H
#define RECENTS_FRAME_H
//----------------------------------------------------------------------------------------------------------------------------------------------------
#pragma once
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/FavoriteManager.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "MDIChildWindow.h"
#include "ExListViewCtrl.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

class RecentHubsFrame : public MDITabChildWindow, public StaticFrame<RecentHubsFrame, ResourceManager::RECENT_HUBS, IDC_RECENTS>, private FavoriteManagerListener,
    private SettingsManagerListener {
public:
	RecentHubsFrame();
	~RecentHubsFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;
private:
	enum {
		COLUMN_NAME,
		COLUMN_DESCRIPTION,
		COLUMN_USERS,
		COLUMN_SHARED,
		COLUMN_SERVER,
		COLUMN_LAST
	};
	
	HWND hWndConnect, hWndRemove, hWndRemoveAll;
	HMENU hubsMenu;
	
	ExListViewCtrl ctrlHubs;

	bool closed;
	
	static int columnSizes[COLUMN_LAST];
	static int columnIndexes[COLUMN_LAST];

    static LRESULT CALLBACK RecentHubsFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT RecentHubsFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void onCreate();
	bool onClose();
	void onDoubleClickHublist(LPNMHDR pnmh);
	void onEnter();
	void onAdd();
	void onClickedConnect();
	void onRemove();
	void onRemoveAll();
	void onEdit();
	void onOpenHubLog();
	void onRefreshSettings();
	void onItemchanged(LPNMHDR pnmh);
	void UpdateLayout();
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onSetFocus();
	void onColumnClickHublist(LPNMHDR pnmh);
	void updateList(const RecentHubEntry::List& fl);
	void addEntry(const RecentHubEntry* entry, int pos);
	bool onKeyDown(LPNMHDR pnmh);

	void on(RecentAdded, const RecentHubEntry* entry) throw();
	void on(RecentRemoved, const RecentHubEntry* entry) throw();
	void on(RecentUpdated, const RecentHubEntry* entry) throw();

	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();
};
//----------------------------------------------------------------------------------------------------------------------------------------------------

#endif
