/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
#include "../client/QueueManager.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "Resource.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "SearchFrame.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "CountryFlags.h"
#include "FlatTabCtrl.h"
#include "WinUtil.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
static ATOM aAtom = 0;
static SearchFrame * pFirstSearchFrame = NULL;
//----------------------------------------------------------------------------------------------------------------------------------------------------
int SearchFrame::iColumnIndexes[] = { SearchInfo::COLUMN_FILE_NAME, SearchInfo::COLUMN_HITS_COUNT, SearchInfo::COLUMN_USER_NICK, SearchInfo::COLUMN_FILE_TYPE, 
	SearchInfo::COLUMN_FILE_SIZE, SearchInfo::COLUMN_FILE_PATH, SearchInfo::COLUMN_USER_SLOTS, SearchInfo::COLUMN_USER_CONNECTION, SearchInfo::COLUMN_USER_HUB, 
	SearchInfo::COLUMN_FILE_EXACT_SIZE, SearchInfo::COLUMN_USER_COUNTRY, SearchInfo::COLUMN_FILE_HASH };
static int iColumnSizes[] = { 200, 80, 100, 50, 80, 100, 40, 70, 150, 80, 100, 125 };
static ResourceManager::Strings rsColumnNames[] = { ResourceManager::FILE, ResourceManager::HIT_COUNT, ResourceManager::USER, ResourceManager::TYPE, ResourceManager::SIZE,
    ResourceManager::PATH, ResourceManager::SLOTS, ResourceManager::CONNECTION, ResourceManager::HUB, ResourceManager::EXACT_SIZE, ResourceManager::COUNTRY, ResourceManager::TTH_ROOT };
//----------------------------------------------------------------------------------------------------------------------------------------------------

SearchFrame::SearchFrame() : bClosed(false), bExpandGroups(BOOLSETTING(SEARCH_EXPAND_GROUPS)), bOnlyFreeSlots(BOOLSETTING(FREE_SLOTS_DEFAULT)), bSearchingHash(false),
    bSearchOnlyTTH(BOOLSETTING(SEARCH_ONLY_TTH)), bSearchPaused(false), bShowSearchUI(BOOLSETTING(SEARCH_SHOW_OPTIONS)), bUserWarned(false), ui64ResultsCount(0), ui64ResultsDropped(0),
    ui64ResultsPaused(0), hResultsMenu(NULL), hTargetMenu(NULL), hTargetDirMenu(NULL), uipTimer(NULL), pPausedResults(NULL), pNewResults(NULL), smtmInitialType(SearchManager::TYPE_ANY) {
    tlvResults.bGroupResults = BOOLSETTING(GROUP_SEARCH_RESULTS);

    memset(&hWindowItems, 0, (sizeof(hWindowItems) / sizeof(hWindowItems[0])) * sizeof(HWND));

    if(aAtom == 0) {
        RegisterWindowClass(aAtom, FrameWndProc, _T("SearchFrame"), IDR_SEARCH);
    }

    pPrevSearchFrame = NULL;
    pNextSearchFrame = pFirstSearchFrame;
    if(pFirstSearchFrame != NULL) {
        pFirstSearchFrame->pPrevSearchFrame = this;
    }
    pFirstSearchFrame = this;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
SearchFrame::~SearchFrame() {
    if(hResultsMenu != NULL) {
        ::DestroyMenu(hResultsMenu);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT SearchFrame::SearchFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_SETFOCUS:
            ::SetFocus(hWindowItems[CB_SEARCH_STRING]);
            return 0;
        case WM_CONTEXTMENU:
            if(OnContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SPEAKER:
            OnSpeaker(wParam, lParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                OnSize();
            }
            break;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORLISTBOX:
            return OnCtlColor(wParam, lParam);
        case WM_CLOSE:
            if(OnClose() == true) {
                return 0;
            }
            break;
        case WM_TIMER:
            OnTimer();
            return 0;
        case WM_REFRESH_SETTINGS:
            OnRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_DOWNLOAD:
                    DownloadSelected(Text::toT(SETTING(DOWNLOAD_DIRECTORY)), false);
                    return 0;
                case IDC_DOWNLOADTO:
                    OnDownloadBrowseTo();
                    return 0;
                case IDC_DOWNLOADDIR:
                    DownloadWholeSelected(Text::toT(SETTING(DOWNLOAD_DIRECTORY)));
                    return 0;
                case IDC_DOWNLOADDIRTO:
                    OnDownloadWholeBrowseTo();
                    return 0;
                case IDC_VIEW_AS_TEXT:
                    OnViewAsText();
                    return 0;
                case IDC_REMOVE:
                    ui64ResultsCount -= tlvResults.DeleteSelectedItems();
                    UpdateStatusBarResultsCount();
                    return 0;
                case IDC_SEARCH:
                    OnEnter();
                    return 0;
                case IDC_SEARCH_PAUSE:
                    OnPauseButton();
                    return 0;
                case IDC_FREESLOTS:
                    bOnlyFreeSlots = ::SendMessage(hWindowItems[CHB_ONLY_FREE_SLOTS], BM_GETCHECK, 0, 0) == BST_CHECKED ? true : false;
                    SettingsManager::getInstance()->set(SettingsManager::FREE_SLOTS_DEFAULT, bOnlyFreeSlots);

                    return 0;
                case IDC_ONLYTTH:
                    bSearchOnlyTTH = ::SendMessage(hWindowItems[CHB_ONLY_WITH_HASH], BM_GETCHECK, 0, 0) == BST_CHECKED ? true : false;
                    SettingsManager::getInstance()->set(SettingsManager::SEARCH_ONLY_TTH, bSearchOnlyTTH);

                    return 0;
                case IDC_GETLIST:
                    OnGetFileList();
                    return 0;
                case IDC_MATCH_QUEUE:
                    OnMatchQueue();
                    return 0;
                case IDC_SEARCH_ALTERNATES:
                    OnSearchForAlternates();
                    return 0;
                case IDC_COLLAPSED:
                    bExpandGroups = ::SendMessage(hWindowItems[CHB_EXPAND_GROUPS], BM_GETCHECK, 0, 0) == BST_CHECKED ? true : false;
                    SettingsManager::getInstance()->set(SettingsManager::SEARCH_EXPAND_GROUPS, bExpandGroups);

                    return 0;
                case IDC_COPY_NICK:
                case IDC_COPY_FILENAME:
                case IDC_COPY_PATH:
                case IDC_COPY_SIZE:
                case IDC_COPY_LINK:
                case IDC_COPY_TTH:
                    OnCopy(LOWORD(wParam));
                    return 0;
            }

            if(LOWORD(wParam) >= IDC_DOWNLOAD_FAVORITE_DIRS && LOWORD(wParam) <= IDC_DOWNLOAD_FAVORITE_DIRS + FavoriteManager::getInstance()->getFavoriteDirs().size()) {
                OnDownloadToFavoriteDir(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS && LOWORD(wParam) <= IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + FavoriteManager::getInstance()->getFavoriteDirs().size()) {
                OnDownloadWholeToFavoriteDir(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_TARGET && LOWORD(wParam) <= IDC_DOWNLOAD_TARGET + sTargetsList.size() + WinUtil::lastDirs.size()) {
                OnDownloadToTarget(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_WHOLE_TARGET && LOWORD(wParam) <= IDC_DOWNLOAD_WHOLE_TARGET + WinUtil::lastDirs.size()) {
                OnDownloadWholeToTarget(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_PRIORITY_PAUSED && LOWORD(wParam) <= IDC_PRIORITY_HIGHEST) {
                OnDownloadWithPriority(LOWORD(wParam));
                return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_RESULTS) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_GETDISPINFO: {
                        tlvResults.OnGetDispInfo(lParam);
                        return 0;
                    }
                    case LVN_COLUMNCLICK: {
                        tlvResults.OnColumnClick(lParam);
                        return 0;
					}
                    case NM_DBLCLK:
                        OnDoubleClickResults(lParam);
                        return 0;
                    case LVN_KEYDOWN: {
                        LPNMLVKEYDOWN lvKeyDown = (LPNMLVKEYDOWN)lParam;

                        if(lvKeyDown->wVKey == VK_DELETE) {
                            ui64ResultsCount -= tlvResults.DeleteSelectedItems();
                            UpdateStatusBarResultsCount();
                            return 0;
                        }

                        break;
                    }
                    case NM_CUSTOMDRAW:
                        return OnCustomDraw(lParam);
                }
            } else if(((LPNMHDR)lParam)->idFrom == IDC_HUB) {
                if(((LPNMHDR)lParam)->code == LVN_GETDISPINFO) {
                    lvHubs.onGetDispInfo((LPNMHDR)lParam);
                    return 0;
                } else if(((LPNMHDR)lParam)->code == LVN_ITEMCHANGED) {
                    OnHubItemChanged(lParam);
                    return 0;
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

	if(UCHandlerProc(uMsg, wParam) == true) {
		return 0;
	}

	LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OpenWindow(tstring tsInitialString/* = Util::emptyStringT*/, uint64_t ui64InitialSize/* = 0*/, SearchManager::SizeModes smInitialMode/* = SearchManager::SIZE_ATLEAST*/,
    SearchManager::TypeModes tmInitialType/* = SearchManager::TYPE_ANY*/) {
    SearchFrame * pNewSearchFrame = new SearchFrame();
    pNewSearchFrame->CreateEx(pNewSearchFrame, aAtom, IDR_SEARCH);

    pNewSearchFrame->smtmInitialType = tmInitialType;

	if(tsInitialString.empty() == false) {
		::SendMessage(pNewSearchFrame->hWindowItems[CB_SEARCH_STRING], CB_INSERTSTRING, 0, (LPARAM)tsInitialString.c_str());
		::SendMessage(pNewSearchFrame->hWindowItems[CB_SEARCH_STRING], CB_SETCURSEL, 0, 0);
		::SendMessage(pNewSearchFrame->hWindowItems[CB_SIZE_MODE], CB_SETCURSEL, smInitialMode, 0);
		::SetWindowText(pNewSearchFrame->hWindowItems[EDT_SIZE], Text::toT(Util::toString(ui64InitialSize)).c_str());
		::SendMessage(pNewSearchFrame->hWindowItems[CB_FILE_TYPE], CB_SETCURSEL, tmInitialType, 0);

		pNewSearchFrame->OnEnter();
	} else {
		::SetWindowText(pNewSearchFrame->m_hWnd, CTSTRING(SEARCH));
		::EnableWindow(pNewSearchFrame->hWindowItems[BTN_PAUSE_SEARCH], FALSE);
	}

	if(ui64InitialSize == 0) {
		::SendMessage(pNewSearchFrame->hWindowItems[CB_SIZE_UNITS], CB_SETCURSEL, 2, 0);
	} else {
		::SendMessage(pNewSearchFrame->hWindowItems[CB_SIZE_UNITS], CB_SETCURSEL, 0, 0);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::CloseAllWindows() {
    SearchFrame * pNext = pFirstSearchFrame,
        * pCur = NULL;

    while(pNext != NULL) {
        pCur = pNext;
        pNext = pCur->pNextSearchFrame;

        ::PostMessage(pCur->m_hWnd, WM_CLOSE, 0, 0);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::runUserCommand(UserCommand & uc) {
    StringMap ucParams;

	if(WinUtil::getUCParams(m_hWnd, uc, ucParams) == false) {
		return;
    }

	set<CID> users;

	int iSel = -1;
	while((iSel = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, iSel, LVNI_SELECTED)) != -1) {
        SearchInfo * si = tlvResults.GetItemData(iSel);

        runUserCommand(uc, si->pSR, ucParams, users);

        if(tlvResults.bGroupResults == true && si->pRoot->bExpanded == false) {
            SearchInfo * pNextSubItem = ((SearchInfo *)si->pRoot->pFirstSubItem)->pNext,
                * pCurSubItem = NULL;

            while(pNextSubItem != NULL) {
                pCurSubItem = pNextSubItem;
                pNextSubItem = pCurSubItem->pNext;

				runUserCommand(uc, pCurSubItem->pSR, ucParams, users);
            }
        }
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT CALLBACK SearchFrame::FrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	SearchFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (SearchFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		::SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (SearchFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->SearchFrameWndProc(hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT CALLBACK SearchFrame::KeyPressCheckWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    SearchFrame * pParent = (SearchFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
    if(pParent == NULL) { // should never happen, but ... :-D
        return 0;
    }

    if(uMsg == WM_CHAR || uMsg == WM_KEYDOWN || uMsg == WM_KEYUP) {
        if(pParent->OnChar(uMsg, wParam) == true) {
            return 0;
        }
    }

    if(hWnd == pParent->tlvResults.m_hWnd) {
        return pParent->tlvResults.TreeListViewProc(hWnd, uMsg, wParam, lParam);
    } else if(hWnd == pParent->lvHubs.m_hWnd) {
		return pParent->lvHubs.TypedListViewProc(hWnd, uMsg, wParam, lParam);
    } else if(hWnd == pParent->hWindowItems[BTN_SEARCH] || hWnd == pParent->hWindowItems[BTN_PAUSE_SEARCH] || hWnd == pParent->hWindowItems[CHB_HIDE_OPTIONS] ||
        hWnd == pParent->hWindowItems[CHB_ONLY_FREE_SLOTS] || 	hWnd == pParent->hWindowItems[CHB_ONLY_WITH_HASH] ||
        (pParent->hWindowItems[CHB_EXPAND_GROUPS] != NULL && hWnd == pParent->hWindowItems[CHB_EXPAND_GROUPS])) {
        return ::CallWindowProc(OldButtonProc, hWnd, uMsg, wParam, lParam);
    } else if(hWnd == pParent->hWindowItems[CB_SIZE_MODE] || hWnd == pParent->hWindowItems[CB_SIZE_UNITS] || 
		hWnd == pParent->hWindowItems[CB_FILE_TYPE]) {
        return ::CallWindowProc(OldComboBoxProc, hWnd, uMsg, wParam, lParam);
    }

    return ::CallWindowProc(OldEditProc, hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT CALLBACK SearchFrame::SearchUIWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
		SearchFrame * pParent = (SearchFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
            pParent->bShowSearchUI = (wParam == BST_CHECKED);
            pParent->UpdateSearchUI();
            SettingsManager::getInstance()->set(SettingsManager::SEARCH_SHOW_OPTIONS, pParent->bShowSearchUI);
        }
    }

    return ::CallWindowProc(OldButtonProc, hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnCreate() {
    created = true;

    WinUtil::tabCtrl->addTab(m_hWnd, RGB(127, 127, 255), IDR_SEARCH);

    m_hWndStatusBar = ::CreateWindowEx(0, STATUSCLASSNAME, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, 0, 0, 0, 0, m_hWnd, 0, g_hInstance, NULL);

    hWindowItems[CHB_SHOW_SEARCH_OPTIONS] = CreateWindowEx(0, WC_BUTTON, CTSTRING(SHOW_HIDE_SEARCH_OPTIONS), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_AUTOCHECKBOX,
        5, 5, 200, 16, m_hWndStatusBar, 0, g_hInstance, NULL);

	::SendMessage(hWindowItems[CHB_SHOW_SEARCH_OPTIONS], BM_SETCHECK, bShowSearchUI == true ? BST_CHECKED : BST_UNCHECKED, 0);

    ::SetWindowLongPtr(hWindowItems[CHB_SHOW_SEARCH_OPTIONS], GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[CHB_SHOW_SEARCH_OPTIONS], GWLP_WNDPROC, (LONG_PTR)SearchUIWndProc);

	WinUtil::splitTokens(iColumnIndexes, SETTING(SEARCHFRAME_ORDER), SearchInfo::COLUMN_LAST);
	WinUtil::splitTokens(iColumnSizes, SETTING(SEARCHFRAME_WIDTHS), SearchInfo::COLUMN_LAST);

    {
        RECT rect;
        ::GetClientRect(m_hWnd, &rect);

        tlvResults.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | WS_TABSTOP | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS,
            WS_EX_CLIENTEDGE, IDC_RESULTS, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT | LVS_EX_DOUBLEBUFFER, 0, 50, rect.right-rect.left, (rect.bottom-rect.top)-50,
            iColumnIndexes, rsColumnNames, iColumnSizes, SearchInfo::COLUMN_LAST, SearchInfo::COLUMN_HITS_COUNT);

        tlvResults.InsertColumn(SearchInfo::COLUMN_FILE_NAME, LVCFMT_LEFT, SearchInfo::COLUMN_FILE_NAME);

        for(uint8_t i = 2; i < SearchInfo::COLUMN_LAST; i++) {
            if(SETTING(SEARCHFRAME_VISIBLE).size() < i || SETTING(SEARCHFRAME_VISIBLE)[i] == '\0' || SETTING(SEARCHFRAME_VISIBLE)[i] == '1') {
                int fmt = (i == SearchInfo::COLUMN_FILE_SIZE || i == SearchInfo::COLUMN_FILE_EXACT_SIZE) ? LVCFMT_RIGHT : LVCFMT_LEFT;
                tlvResults.InsertColumn(i, fmt, i);
            }
        }

        if(tlvResults.bGroupResults == true) {
            tlvResults.InsertColumn(SearchInfo::COLUMN_HITS_COUNT, LVCFMT_LEFT, SearchInfo::COLUMN_HITS_COUNT);
        }
    }

	::SendMessage(tlvResults.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::fileImages);
    ::SetWindowLongPtr(tlvResults.m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
    ::SetWindowLongPtr(tlvResults.m_hWnd, GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

    if(tlvResults.bGroupResults == true) {
        tlvResults.SetAscending(false);
        tlvResults.SetSortColumn(SearchInfo::COLUMN_HITS_COUNT, true);
    } else {
        tlvResults.SetSortColumn(0);
    }

	hWindowItems[GB_SEARCH_FOR] = ::CreateWindowEx(WS_EX_TRANSPARENT, WC_BUTTON, CTSTRING(SEARCH_FOR), WS_CHILD | WS_VISIBLE | BS_GROUPBOX, 5, 1, 210, 44, m_hWnd, NULL, g_hInstance, NULL);

    hWindowItems[CB_SEARCH_STRING] = ::CreateWindowEx(0, WC_COMBOBOX, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_VSCROLL | WS_TABSTOP | CBS_DROPDOWN | CBS_AUTOHSCROLL,
        12, 16, 196, 21, m_hWnd, NULL, g_hInstance, NULL);
    ::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_SETEXTENDEDUI, TRUE, 0);

    {
        for(int i = 0; i <= (SettingsManager::SEARCH_MSG_RECENT_20-SettingsManager::SEARCH_MSG_RECENT_01); i++) {
            ::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_INSERTSTRING, (WPARAM)-1,
				(LPARAM)Text::toT(SettingsManager::getInstance()->get((SettingsManager::StrSetting)(SettingsManager::SEARCH_MSG_RECENT_01+i), true)).c_str());
        }

        COMBOBOXINFO cbi = { sizeof(COMBOBOXINFO) };

        if(::GetComboBoxInfo(hWindowItems[CB_SEARCH_STRING], &cbi) == TRUE) {
            ::SetWindowLongPtr(cbi.hwndItem, GWLP_USERDATA, (LONG_PTR)this);
            OldEditProc = (WNDPROC)::SetWindowLongPtr(cbi.hwndItem, GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);
        }
    }

	hWindowItems[GB_FILE_TYPE] = ::CreateWindowEx(WS_EX_TRANSPARENT, WC_BUTTON, CTSTRING(FILE_TYPE), WS_CHILD | WS_VISIBLE | BS_GROUPBOX, 220, 1, 110, 44, m_hWnd, 0, g_hInstance, NULL);

    hWindowItems[CB_FILE_TYPE] = ::CreateWindowEx(0, WC_COMBOBOX, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_HSCROLL | WS_VSCROLL | WS_TABSTOP | CBS_DROPDOWNLIST,
        227, 16, 96, 21, m_hWnd, NULL, g_hInstance, NULL);

	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(ANY));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(AUDIO));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(COMPRESSED));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(DOCUMENT));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(EXECUTABLE));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(PICTURE));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(VIDEO));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(DIRECTORY));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_ADDSTRING, 0, (LPARAM)_T("TTH"));
	::SendMessage(hWindowItems[CB_FILE_TYPE], CB_SETCURSEL, SETTING(LAST_SEARCH_TYPE), 0);

    ::SetWindowLongPtr(hWindowItems[CB_FILE_TYPE], GWLP_USERDATA, (LONG_PTR)this);
    OldComboBoxProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[CB_FILE_TYPE], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

	hWindowItems[GB_SIZE] = ::CreateWindowEx(WS_EX_TRANSPARENT, WC_BUTTON, CTSTRING(SIZE), WS_CHILD | WS_VISIBLE | BS_GROUPBOX, 335, 1, 210, 44, m_hWnd, NULL, g_hInstance, NULL);

    hWindowItems[CB_SIZE_MODE] = ::CreateWindowEx(0, WC_COMBOBOX, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_HSCROLL | WS_VSCROLL | WS_TABSTOP | CBS_DROPDOWNLIST,
        343, 16, 72, 21, m_hWnd, NULL, g_hInstance, NULL);

	::SendMessage(hWindowItems[CB_SIZE_MODE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(NORMAL));
	::SendMessage(hWindowItems[CB_SIZE_MODE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(AT_LEAST));
	::SendMessage(hWindowItems[CB_SIZE_MODE], CB_ADDSTRING, 0, (LPARAM)CTSTRING(AT_MOST));
	::SendMessage(hWindowItems[CB_SIZE_MODE], CB_SETCURSEL, 1, 0);

    ::SetWindowLongPtr(hWindowItems[CB_SIZE_MODE], GWLP_USERDATA, (LONG_PTR)this);
    OldComboBoxProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[CB_SIZE_MODE], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

    hWindowItems[EDT_SIZE] = ::CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_TABSTOP | ES_AUTOHSCROLL | ES_NUMBER, 420, 16, 72, 21, m_hWnd,
        NULL, g_hInstance, NULL);

    ::SetWindowLongPtr(hWindowItems[EDT_SIZE], GWLP_USERDATA, (LONG_PTR)this);
    OldEditProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[EDT_SIZE], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

    hWindowItems[CB_SIZE_UNITS] = ::CreateWindowEx(0, WC_COMBOBOX, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_HSCROLL | WS_VSCROLL | WS_TABSTOP | CBS_DROPDOWNLIST,
        497, 16, 40, 21, m_hWnd, NULL, g_hInstance, NULL);

	::SendMessage(hWindowItems[CB_SIZE_UNITS], CB_ADDSTRING, 0, (LPARAM)CTSTRING(B));
	::SendMessage(hWindowItems[CB_SIZE_UNITS], CB_ADDSTRING, 0, (LPARAM)CTSTRING(KB));
	::SendMessage(hWindowItems[CB_SIZE_UNITS], CB_ADDSTRING, 0, (LPARAM)CTSTRING(MB));
	::SendMessage(hWindowItems[CB_SIZE_UNITS], CB_ADDSTRING, 0, (LPARAM)CTSTRING(GB));

    ::SetWindowLongPtr(hWindowItems[CB_SIZE_UNITS], GWLP_USERDATA, (LONG_PTR)this);
    OldComboBoxProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[CB_SIZE_UNITS], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

    hWindowItems[BTN_SEARCH] = CreateWindowEx(0, WC_BUTTON, CTSTRING(SEARCH), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_TABSTOP | BS_PUSHBUTTON,
        549, 2, 105, 21, m_hWnd, (HMENU)IDC_SEARCH, g_hInstance, NULL);

    ::SetWindowLongPtr(hWindowItems[BTN_SEARCH], GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[BTN_SEARCH], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

	hWindowItems[BTN_PAUSE_SEARCH] = CreateWindowEx(0, WC_BUTTON, CTSTRING(PAUSE_SEARCH), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_TABSTOP | BS_PUSHBUTTON,
        549, 25, 105, 21, m_hWnd, (HMENU)IDC_SEARCH_PAUSE, g_hInstance, NULL);

    ::SetWindowLongPtr(hWindowItems[BTN_PAUSE_SEARCH], GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[BTN_PAUSE_SEARCH], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

	hWindowItems[GB_SEARCH_OPTIONS] = ::CreateWindowEx(WS_EX_TRANSPARENT, WC_BUTTON, CTSTRING(SEARCH_OPTIONS), WS_CHILD | BS_GROUPBOX, 5, 47, 210,
        tlvResults.bGroupResults == true ? 93 : 74, m_hWnd, NULL, g_hInstance, NULL);

    hWindowItems[CHB_HIDE_OPTIONS] = ::CreateWindowEx(0, WC_BUTTON, CTSTRING(HIDE_SEARCH_OPTIONS_ON_SEARCH), WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | BS_AUTOCHECKBOX,
        13, 61, 194, 16, m_hWnd, NULL, g_hInstance, NULL);

	::SendMessage(hWindowItems[CHB_HIDE_OPTIONS], BM_SETCHECK, BOOLSETTING(AUTO_HIDE_SEARCH_OPTIONS_ON_SEARCH) == true ? BST_CHECKED : BST_UNCHECKED, 0);

    ::SetWindowLongPtr(hWindowItems[CHB_HIDE_OPTIONS], GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[CHB_HIDE_OPTIONS], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

    hWindowItems[CHB_ONLY_FREE_SLOTS] = ::CreateWindowEx(0, WC_BUTTON, CTSTRING(ONLY_FREE_SLOTS), WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | BS_AUTOCHECKBOX, 13, 80, 194, 16,
        m_hWnd, (HMENU)IDC_FREESLOTS, g_hInstance, NULL);

	::SendMessage(hWindowItems[CHB_ONLY_FREE_SLOTS], BM_SETCHECK, (bOnlyFreeSlots == true ? BST_CHECKED : BST_UNCHECKED), 0);

    ::SetWindowLongPtr(hWindowItems[CHB_ONLY_FREE_SLOTS], GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[CHB_ONLY_FREE_SLOTS], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

    hWindowItems[CHB_ONLY_WITH_HASH] = ::CreateWindowEx(0, WC_BUTTON, CTSTRING(ONLY_TTH), WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | BS_AUTOCHECKBOX, 13, 99, 194, 16,
        m_hWnd, (HMENU)IDC_ONLYTTH, g_hInstance, NULL);

	::SendMessage(hWindowItems[CHB_ONLY_WITH_HASH], BM_SETCHECK, (bSearchOnlyTTH == true ? BST_CHECKED : BST_UNCHECKED), 0);

    ::SetWindowLongPtr(hWindowItems[CHB_ONLY_WITH_HASH], GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[CHB_ONLY_WITH_HASH], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

    if(tlvResults.bGroupResults == true) {
        hWindowItems[CHB_EXPAND_GROUPS] = ::CreateWindowEx(0, WC_BUTTON, CTSTRING(SEARCH_EXPAND_GROUPED_RESULTS), WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | BS_AUTOCHECKBOX,
            13, 118, 194, 16, m_hWnd, (HMENU)IDC_COLLAPSED, g_hInstance, NULL);

        ::SendMessage(hWindowItems[CHB_EXPAND_GROUPS], BM_SETCHECK, (bExpandGroups == true ? BST_CHECKED : BST_UNCHECKED), 0);

        ::SetWindowLongPtr(hWindowItems[CHB_EXPAND_GROUPS], GWLP_USERDATA, (LONG_PTR)this);
        OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWindowItems[CHB_EXPAND_GROUPS], GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);
    }

	hWindowItems[GB_HUBS] = CreateWindowEx(WS_EX_TRANSPARENT, WC_BUTTON, CTSTRING(HUBS), WS_CHILD | BS_GROUPBOX, 5, tlvResults.bGroupResults == true ? 140 : 121,
        210, 45, m_hWnd, NULL, g_hInstance, NULL);

	lvHubs.Create(m_hWnd, WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | WS_TABSTOP | LVS_REPORT | LVS_NOCOLUMNHEADER, WS_EX_CLIENTEDGE, IDC_HUB,
		LVS_EX_LABELTIP | LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT, 13, tlvResults.bGroupResults == true ? 155 : 136, 194, 21, false);

    ::SetWindowLongPtr(lvHubs.m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
    ::SetWindowLongPtr(lvHubs.m_hWnd, GWLP_WNDPROC, (LONG_PTR)KeyPressCheckWndProc);

	ListViewInsertDummyColumn(lvHubs.m_hWnd);

    {
        lvHubs.insertItem(new HubInfo(Util::emptyStringT, TSTRING(ONLY_WHERE_OP), false), 0);
        ListView_SetItemState(lvHubs.m_hWnd, 0, INDEXTOSTATEIMAGEMASK(1), LVIS_STATEIMAGEMASK);

        ClientManager::getInstance()->lock();

        Client::List & lClients = ClientManager::getInstance()->getClients();

        for(Client::List::iterator it = lClients.begin(); it != lClients.end(); it++) {
            if((*it)->isConnected() == false) {
                continue;
            }

            AddHub(new HubInfo(Text::toT((*it)->getHubUrl()), Text::toT((*it)->getHubName()), (*it)->getMyIdentity().isOp()));
        }

        ClientManager::getInstance()->addListener(this);

        ClientManager::getInstance()->unlock();

        ::SendMessage(lvHubs.m_hWnd, LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE);
    }

    for(uint8_t ui8i = 0; ui8i < (sizeof(hWindowItems) / sizeof(hWindowItems[0])); ui8i++) {
        ::SendMessage(hWindowItems[ui8i], WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    }

    SearchManager::getInstance()->addListener(this);
    SettingsManager::getInstance()->addListener(this);
    TimerManager::getInstance()->addListener(this);

    hTargetMenu = ::CreatePopupMenu();
    hTargetDirMenu = ::CreatePopupMenu();

    HMENU hCopyMenu = ::CreatePopupMenu();
    ::AppendMenu(hCopyMenu, MF_STRING, IDC_COPY_NICK, CTSTRING(COPY_NICK));
	::AppendMenu(hCopyMenu, MF_STRING, IDC_COPY_FILENAME, CTSTRING(FILENAME));
	::AppendMenu(hCopyMenu, MF_STRING, IDC_COPY_PATH, CTSTRING(PATH));
	::AppendMenu(hCopyMenu, MF_STRING, IDC_COPY_SIZE, CTSTRING(SIZE));
	::AppendMenu(hCopyMenu, MF_STRING, IDC_COPY_TTH, CTSTRING(TTH_ROOT));
	::AppendMenu(hCopyMenu, MF_STRING, IDC_COPY_LINK, CTSTRING(MAGNET_LINK));

    hResultsMenu = ::CreatePopupMenu();
    ::AppendMenu(hResultsMenu, MF_STRING, IDC_DOWNLOAD, CTSTRING(DOWNLOAD));
    ::AppendMenu(hResultsMenu, MF_POPUP, (UINT_PTR)hTargetMenu, CTSTRING(DOWNLOAD_TO));

    WinUtil::AddPrioritySubMenu(hResultsMenu, TSTRING(DOWNLOAD_WITH_PRIORITY));

    ::AppendMenu(hResultsMenu, MF_STRING, IDC_DOWNLOADDIR, CTSTRING(DOWNLOAD_WHOLE_DIR));
    ::AppendMenu(hResultsMenu, MF_POPUP, (UINT_PTR)hTargetDirMenu, CTSTRING(DOWNLOAD_WHOLE_DIR_TO));
    ::AppendMenu(hResultsMenu, MF_STRING, IDC_VIEW_AS_TEXT, CTSTRING(VIEW_AS_TEXT));
	::AppendMenu(hResultsMenu, MF_STRING, IDC_SEARCH_ALTERNATES, CTSTRING(SEARCH_FOR_ALTERNATES));
	::AppendMenu(hResultsMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(hResultsMenu, MF_STRING, IDC_GETLIST, CTSTRING(GET_FILE_LIST));
	::AppendMenu(hResultsMenu, MF_STRING, IDC_MATCH_QUEUE, CTSTRING(MATCH_QUEUE));
	::AppendMenu(hResultsMenu, MF_STRING, IDC_PRIVATEMESSAGE, CTSTRING(SEND_PRIVATE_MESSAGE));
	::AppendMenu(hResultsMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));
	::AppendMenu(hResultsMenu, MF_STRING, IDC_REMOVEALL, CTSTRING(REMOVE_FROM_ALL));
	::AppendMenu(hResultsMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(hResultsMenu, MF_POPUP, (UINT_PTR)hCopyMenu, CTSTRING(COPY));

    WinUtil::AddGrantSubMenu(hResultsMenu);

	::SetMenuDefaultItem(hResultsMenu, IDC_DOWNLOAD, 0);

	OnSize();
	UpdateSearchUI();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool SearchFrame::OnClose() {
	if(bClosed == false) {
        TimerManager::getInstance()->removeListener(this);
        SettingsManager::getInstance()->removeListener(this);
		SearchManager::getInstance()->removeListener(this);
 		ClientManager::getInstance()->removeListener(this);

        if(pPrevSearchFrame == NULL) {
            if(pNextSearchFrame == NULL) {
                pFirstSearchFrame = NULL;
            } else {
                pNextSearchFrame->pPrevSearchFrame = NULL;
                pFirstSearchFrame = pNextSearchFrame;
            }
        } else if(pNextSearchFrame == NULL) {
            pPrevSearchFrame->pNextSearchFrame = NULL;
        } else {
            pPrevSearchFrame->pNextSearchFrame = pNextSearchFrame;
            pNextSearchFrame->pPrevSearchFrame = pPrevSearchFrame;
        }

		bClosed = true;

		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);

		return true;
	} else {
        if(smtmInitialType == SearchManager::TYPE_ANY && (int)::SendMessage(hWindowItems[CB_FILE_TYPE], CB_GETCURSEL, 0, 0) != SETTING(LAST_SEARCH_TYPE)) {
            SettingsManager::getInstance()->set(SettingsManager::LAST_SEARCH_TYPE, (int)::SendMessage(hWindowItems[CB_FILE_TYPE], CB_GETCURSEL, 0, 0));
        }

        SettingsManager::getInstance()->set(SettingsManager::AUTO_HIDE_SEARCH_OPTIONS_ON_SEARCH,
            ::SendMessage(hWindowItems[CHB_HIDE_OPTIONS], BM_GETCHECK, 0, 0) == BST_CHECKED ? true : false);

        {
            for(int i = 0; i <= (SettingsManager::SEARCH_MSG_RECENT_20-SettingsManager::SEARCH_MSG_RECENT_01); i++) {
                int iLen = (int)::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_GETLBTEXTLEN, (WPARAM)i, 0);
                if(iLen == CB_ERR) {
                    SettingsManager::getInstance()->set((SettingsManager::StrSetting)(SettingsManager::SEARCH_MSG_RECENT_01+i), Util::emptyString);
                    continue;
                }

                TCHAR * tcBuf = new TCHAR[iLen+1];

                int iRet = (int)::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_GETLBTEXT, (WPARAM)i, (LPARAM)tcBuf);
                if(iRet == CB_ERR) {
                    SettingsManager::getInstance()->set((SettingsManager::StrSetting)(SettingsManager::SEARCH_MSG_RECENT_01+i), Util::emptyString);
                    continue;
                }

				SettingsManager::getInstance()->set((SettingsManager::StrSetting)(SettingsManager::SEARCH_MSG_RECENT_01+i), Text::fromT(tcBuf));

				delete[] tcBuf;
            }
        }

		tlvResults.DeleteAllItems();

        SearchInfo * nextPaused = pPausedResults,
            * curPaused = NULL;

        while(nextPaused != NULL) {
            curPaused = nextPaused;
            nextPaused = curPaused->pNext;
            
			delete curPaused;
		}

		pPausedResults = NULL;

        SearchInfo * nextNew = pNewResults,
            * curNew = NULL;

        while(nextNew != NULL) {
            curNew = nextNew;
            nextNew = curNew->pNext;

			delete curNew;
		}

		pNewResults = NULL;

		for(int i = 0; i < (int)::SendMessage(lvHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0); i++) {
			delete lvHubs.getItemData(i);
		}

		::SendMessage(lvHubs.m_hWnd, LVM_DELETEALLITEMS, 0, 0);

		tlvResults.SaveHeaderProperties(SettingsManager::SEARCHFRAME_ORDER, SettingsManager::SEARCHFRAME_WIDTHS,
			SettingsManager::SEARCHFRAME_VISIBLE);

        return false;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnTimer() {
    uint32_t ui32WaitTime = SearchManager::getInstance()->timeToSearch();
    if(ui32WaitTime > 0) {
		TCHAR * tcBuf = new TCHAR[TSTRING(SEARCHING_WAIT).size() + 16];
		_stprintf_s(tcBuf, TSTRING(SEARCHING_WAIT).size() + 16, CTSTRING(SEARCHING_WAIT), ui32WaitTime);

		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)tcBuf);
		::SetWindowText(m_hWnd, (TSTRING(SEARCH) + _T(" - ") + tstring(tcBuf)).c_str());

		delete[] tcBuf;
	} else {
		if(uipTimer != 0) {
			::KillTimer(m_hWnd, uipTimer);
			uipTimer = 0;
		}

		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)(TSTRING(SEARCHING_READY)).c_str());
		::SetWindowText(m_hWnd, (TSTRING(SEARCH) + _T(" - ") + TSTRING(SEARCHING_READY)).c_str());
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnSize() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		::GetClientRect(m_hWndStatusBar, &sr);

        int w[3];

		w[0] = 220;
		w[1] = sr.right-116;
		w[2] = sr.right-16;

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 3, (LPARAM)w);
	}

    if((rect.right - rect.left) > 659) {
        int iX = (rect.right - rect.left);
        ::SetWindowPos(hWindowItems[GB_SEARCH_FOR], NULL, 0, 0, iX-439-9, 44, SWP_NOMOVE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[CB_SEARCH_STRING], NULL, 0, 0, iX-439-23, 21, SWP_NOMOVE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[GB_FILE_TYPE], NULL, iX-438, 1, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[CB_FILE_TYPE], NULL, iX-431, 16, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[GB_SIZE], NULL, iX-323, 1, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[CB_SIZE_MODE], NULL, iX-315, 16, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[EDT_SIZE], NULL, iX-238, 16, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[CB_SIZE_UNITS], NULL, iX-161, 16, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[BTN_SEARCH], NULL, iX-109, 2, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[BTN_PAUSE_SEARCH], NULL, iX-109, 25, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
    } else {
        ::SetWindowPos(hWindowItems[GB_SEARCH_FOR], NULL, 0, 0, 210, 44, SWP_NOMOVE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[CB_SEARCH_STRING], NULL, 0, 0, 196, 21, SWP_NOMOVE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[GB_FILE_TYPE], NULL, 220, 1, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[CB_FILE_TYPE], NULL, 227, 16, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[GB_SIZE], NULL, 335, 1, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[CB_SIZE_MODE], NULL, 343, 16, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[EDT_SIZE], NULL, 420, 16, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[CB_SIZE_UNITS], NULL, 497, 16, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[BTN_SEARCH], NULL, 549, 2, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
        ::SetWindowPos(hWindowItems[BTN_PAUSE_SEARCH], NULL, 549, 25, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
    }

	if(bShowSearchUI == true) {
		::SetWindowPos(tlvResults.m_hWnd, NULL, rect.left+220, rect.top+50, rect.right - rect.left, (rect.bottom - rect.top)-50, SWP_NOZORDER);

        if((rect.bottom - rect.top) < (tlvResults.bGroupResults == true ? 189 : 170)) {
            ::ShowWindow(hWindowItems[GB_HUBS], SW_HIDE);
            ::ShowWindow(lvHubs.m_hWnd, SW_HIDE);
        } else {
            if(::IsWindowVisible(hWindowItems[GB_HUBS]) == FALSE) {
                ::ShowWindow(hWindowItems[GB_HUBS], SW_SHOW);
                ::ShowWindow(lvHubs.m_hWnd, SW_SHOW);
            }

            int iY = (rect.bottom - rect.top)-5;
            ::SetWindowPos(hWindowItems[GB_HUBS], NULL, 0, 0, 210, iY-(tlvResults.bGroupResults == true ? 140 : 121), SWP_NOMOVE | SWP_NOZORDER);
            ::SetWindowPos(lvHubs.m_hWnd, NULL, 0, 0, 194, iY-(tlvResults.bGroupResults == true ? 163 : 144), SWP_NOMOVE | SWP_NOZORDER);
        }
    } else {
        ::SetWindowPos(tlvResults.m_hWnd, NULL, rect.left, rect.top+50, rect.right - rect.left, (rect.bottom - rect.top)-50, SWP_NOZORDER);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::UpdateSearchUI() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect, false);

	if(bShowSearchUI == false) {
        ::SetWindowPos(tlvResults.m_hWnd, NULL, rect.left, rect.top+50, rect.right - rect.left, (rect.bottom - rect.top)-50, SWP_NOZORDER);

        ::ShowWindow(hWindowItems[GB_SEARCH_OPTIONS], SW_HIDE);
        ::ShowWindow(hWindowItems[CHB_HIDE_OPTIONS], SW_HIDE);
        ::EnableWindow(hWindowItems[CHB_HIDE_OPTIONS], FALSE);
        ::ShowWindow(hWindowItems[CHB_ONLY_FREE_SLOTS], SW_HIDE);
        ::EnableWindow(hWindowItems[CHB_ONLY_FREE_SLOTS], FALSE);
        ::ShowWindow(hWindowItems[CHB_ONLY_WITH_HASH], SW_HIDE);
        ::EnableWindow(hWindowItems[CHB_ONLY_WITH_HASH], FALSE);
        if(hWindowItems[CHB_EXPAND_GROUPS] != NULL) {
            ::ShowWindow(hWindowItems[CHB_EXPAND_GROUPS], SW_HIDE);
            ::EnableWindow(hWindowItems[CHB_EXPAND_GROUPS], FALSE);
        }
        ::ShowWindow(hWindowItems[GB_HUBS], SW_HIDE);
        ::ShowWindow(lvHubs.m_hWnd, SW_HIDE);
        ::EnableWindow(lvHubs.m_hWnd, FALSE);

		return;
	} else {
        ::ShowWindow(hWindowItems[GB_SEARCH_OPTIONS], SW_SHOW);
        ::ShowWindow(hWindowItems[CHB_HIDE_OPTIONS], SW_SHOW);
        ::EnableWindow(hWindowItems[CHB_HIDE_OPTIONS], TRUE);
        ::ShowWindow(hWindowItems[CHB_ONLY_FREE_SLOTS], SW_SHOW);
        ::EnableWindow(hWindowItems[CHB_ONLY_FREE_SLOTS], TRUE);
        ::ShowWindow(hWindowItems[CHB_ONLY_WITH_HASH], SW_SHOW);
        ::EnableWindow(hWindowItems[CHB_ONLY_WITH_HASH], TRUE);
        if(hWindowItems[CHB_EXPAND_GROUPS] != NULL) {
            ::ShowWindow(hWindowItems[CHB_EXPAND_GROUPS], SW_SHOW);
            ::EnableWindow(hWindowItems[CHB_EXPAND_GROUPS], TRUE);
        }
        ::ShowWindow(hWindowItems[GB_HUBS], SW_SHOW);
        ::ShowWindow(lvHubs.m_hWnd, SW_SHOW);
        ::EnableWindow(lvHubs.m_hWnd, TRUE);

        OnSize();
    }
}

//----------------------------------------------------------------------------------------------------------------------------------------------------

bool SearchFrame::OnChar(UINT uMsg, WPARAM wParam) {
    if(wParam == VK_TAB) {
        if(uMsg == WM_KEYDOWN) {
            OnTab(WinUtil::isShift());
            return true;
        }
    } else if(wParam == VK_RETURN) {
        if(WinUtil::isShift() || WinUtil::isCtrl() || WinUtil::isAlt()) {
            return false;
        } else if(uMsg == WM_KEYDOWN) {
            OnEnter();
            return true;
        }
    }

    return false;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool SearchFrame::OnContextMenu(WPARAM wParam, LPARAM lParam) {
    if((HWND)wParam != tlvResults.m_hWnd) {
        return false;
    }

    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);

    {
        RECT rc;
		::GetWindowRect((HWND)::SendMessage(tlvResults.m_hWnd, LVM_GETHEADER, 0, 0), &rc);

        POINT pt = { iX, iY };

		if(::PtInRect(&rc, pt)){
			tlvResults.ShowMenu(pt);
			return true;
		}
    }

	if((UINT)::SendMessage(tlvResults.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 0) {
        return false;
    }

	if(iX == -1 && iY == -1) {
		ListViewGetContextMenuPos(tlvResults.m_hWnd, iX, iY);
	}

	while(::GetMenuItemCount(hTargetMenu) > 0) {
        ::RemoveMenu(hTargetMenu, 0, MF_BYPOSITION);
	}

	while(::GetMenuItemCount(hTargetDirMenu) > 0) {
        ::RemoveMenu(hTargetDirMenu, 0, MF_BYPOSITION);
	}

	//Append favorite download dirs
	StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
	if(spl.size() != 0) {
        int iCount = 0;
		for(StringPairIter i = spl.begin(); i != spl.end(); i++) {
            ::AppendMenu(hTargetMenu, MF_STRING, IDC_DOWNLOAD_FAVORITE_DIRS + iCount, Text::toT(i->second).c_str());
			iCount++;
		}
		::AppendMenu(hTargetMenu, MF_SEPARATOR, 0, NULL);
	}

    ::AppendMenu(hTargetMenu, MF_STRING, IDC_DOWNLOADTO, CTSTRING(BROWSE));

    if(WinUtil::lastDirs.size() != 0) {
        ::AppendMenu(hTargetMenu, MF_SEPARATOR, 0, NULL);

        int iCount = 0;
        for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
            ::AppendMenu(hTargetMenu, MF_STRING, IDC_DOWNLOAD_TARGET + iCount, i->c_str());
            iCount++;
        }
    }

	ResultsCheck rc;

    CheckSelectedResults(&rc);

	if(rc.i64Size != -1 || rc.bHasHash) {
		sTargetsList.clear();
		if(rc.bHasHash) {
			QueueManager::getInstance()->getTargetsByRoot(sTargetsList, TTHValue(Text::fromT(rc.tsHash)));
		} else if(BOOLSETTING(USE_EXTENSION_DOWNTO)) {
			QueueManager::getInstance()->getTargetsBySize(sTargetsList, rc.i64Size, Text::fromT(rc.tsExtension));
		} else {
			QueueManager::getInstance()->getTargetsBySize(sTargetsList, rc.i64Size, Util::emptyString);
		}

		if(sTargetsList.size() != 0) {
			::AppendMenu(hTargetMenu, MF_SEPARATOR, 0, NULL);

            int iCount = 0;
			for(StringIter i = sTargetsList.begin(); i != sTargetsList.end(); ++i) {
                ::AppendMenu(hTargetMenu, MF_STRING, IDC_DOWNLOAD_TARGET + iCount, Text::toT(*i).c_str());
				iCount++;
			}
		}
	}

	//Append favorite download dirs
	if(spl.size() != 0) {
        int iCount = 0;
		for(StringPairIter i = spl.begin(); i != spl.end(); ++i) {
            ::AppendMenu(hTargetDirMenu, MF_STRING, IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + iCount, Text::toT(i->second).c_str());
			iCount++;
		}
		::AppendMenu(hTargetDirMenu, MF_SEPARATOR, 0, NULL);
	}

    ::AppendMenu(hTargetDirMenu, MF_STRING, IDC_DOWNLOADDIRTO, CTSTRING(BROWSE));

	if(WinUtil::lastDirs.size() != 0) {
    	::AppendMenu(hTargetDirMenu, MF_SEPARATOR, 0, NULL);

        int iCount = 0;
    	for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
            ::AppendMenu(hTargetDirMenu, MF_STRING, IDC_DOWNLOAD_WHOLE_TARGET + iCount, i->c_str());
			iCount++;
		}
	}

	// magnet link
	if((UINT)::SendMessage(tlvResults.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1){
		SearchResult* sr = tlvResults.GetSelectedItem()->pSR;
		if(sr->getType() == SearchResult::TYPE_FILE && sr->getTTH()) {
            ::EnableMenuItem(hResultsMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_ENABLED);
            ::EnableMenuItem(hResultsMenu, IDC_COPY_TTH, MF_BYCOMMAND | MFS_ENABLED);
            ::EnableMenuItem(hResultsMenu, IDC_COPY_LINK, MF_BYCOMMAND | MFS_ENABLED);
		} else {
            ::EnableMenuItem(hResultsMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_DISABLED);
            ::EnableMenuItem(hResultsMenu, IDC_COPY_TTH, MF_BYCOMMAND | MFS_DISABLED);
            ::EnableMenuItem(hResultsMenu, IDC_COPY_LINK, MF_BYCOMMAND | MFS_DISABLED);
		}
	}

    prepareMenu(hResultsMenu, UserCommand::CONTEXT_SEARCH, rc.sHub, rc.bOperator);

	if(!(::GetMenuState(hResultsMenu, ::GetMenuItemCount(hResultsMenu)-1, MF_BYPOSITION) & MF_SEPARATOR)) {
		::AppendMenu(hResultsMenu, MF_SEPARATOR, 0, NULL);
	}

	::AppendMenu(hResultsMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));

	::TrackPopupMenuEx(hResultsMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

	::RemoveMenu(hResultsMenu, ::GetMenuItemCount(hResultsMenu)-1, MF_BYPOSITION);
	::RemoveMenu(hResultsMenu, ::GetMenuItemCount(hResultsMenu)-1, MF_BYPOSITION);
	cleanMenu(hResultsMenu);

	return true;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnDoubleClickResults(LPARAM lParam) {
    LPNMITEMACTIVATE lpItemActivate = (LPNMITEMACTIVATE)lParam;

    if(lpItemActivate->iItem == -1) {
        return;
    }

    RECT rc = { LVIR_ICON, 0, 0, 0 };

    if(::SendMessage(tlvResults.m_hWnd, LVM_GETITEMRECT, lpItemActivate->iItem, (LPARAM)&rc) == TRUE &&
        lpItemActivate->ptAction.x < rc.left) {
        return;
    }

	DownloadSelected(Text::toT(SETTING(DOWNLOAD_DIRECTORY)), false);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnPauseButton() {
	if(bSearchPaused == true) {
		::SendMessage(tlvResults.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

        SearchInfo * nextPaused = pPausedResults,
            * curPaused = NULL;

        while(nextPaused != NULL) {
            curPaused = nextPaused;
            nextPaused = curPaused->pNext;

            curPaused->pNext = NULL;
            curPaused->pPrev = NULL;
            
			AddResult(curPaused);
		}

        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)Text::toT(Util::toString(ui64ResultsCount) + " " + STRING(FILES)).c_str());

		pPausedResults = NULL;
		ui64ResultsPaused = 0;

		::SendMessage(tlvResults.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	}

	bSearchPaused = !bSearchPaused;

	::SetWindowText(hWindowItems[BTN_PAUSE_SEARCH], bSearchPaused == true ? CTSTRING(CONTINUE_SEARCH) : CTSTRING(PAUSE_SEARCH));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT SearchFrame::OnCtlColor(WPARAM wParam, LPARAM lParam) {
	HWND hWnd = (HWND)lParam;
	HDC hDC = (HDC)wParam;

	if(hWnd == hWindowItems[CHB_HIDE_OPTIONS] || hWnd == hWindowItems[CHB_ONLY_FREE_SLOTS] || hWnd == hWindowItems[CHB_ONLY_WITH_HASH] ||
        (tlvResults.bGroupResults == true && hWnd == hWindowItems[CHB_EXPAND_GROUPS])) {

		::SetBkColor(hDC, ::GetSysColor(COLOR_3DFACE));
		::SetTextColor(hDC, ::GetSysColor(COLOR_BTNTEXT));
		return (LRESULT)::GetSysColorBrush(COLOR_3DFACE);
	} else {
		::SetBkColor(hDC, WinUtil::bgColor);
		::SetTextColor(hDC, WinUtil::textColor);
		return (LRESULT)WinUtil::bgBrush;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT SearchFrame::OnCustomDraw(LPARAM lParam) {
    LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)lParam;

	switch(cd->nmcd.dwDrawStage) {
        case CDDS_PREPAINT:
            return CDRF_NOTIFYITEMDRAW;
        case CDDS_ITEMPREPAINT:
            return CDRF_NOTIFYSUBITEMDRAW;
        case CDDS_SUBITEM | CDDS_ITEMPREPAINT: {
            RECT rc;
            int colIndex = tlvResults.GetRealColumn(cd->iSubItem);

            if(colIndex == SearchInfo::COLUMN_USER_NICK) {
                SearchInfo* si = (SearchInfo*)cd->nmcd.lItemlParam;
                rc.top = cd->iSubItem;
                rc.left = LVIR_BOUNDS;
                ::SendMessage(tlvResults.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);
                COLORREF color;
                if((UINT)::SendMessage(tlvResults.m_hWnd, LVM_GETITEMSTATE, (int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
				    if(tlvResults.m_hWnd == ::GetFocus()) {
                        color = GetSysColor(COLOR_HIGHLIGHT);
                        SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
                        SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
                    } else {
                        color = GetBkColor(cd->nmcd.hdc);
                        SetBkColor(cd->nmcd.hdc, color);
                    }
                } else {
				    color = WinUtil::bgColor;
				    SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
				    SetTextColor(cd->nmcd.hdc, WinUtil::textColor);
                }
                RECT rc2 = rc;
                rc2.left += 2;
                HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
                HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
                Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);
                DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
                DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));
                TCHAR buf[256];
                ListViewGetItemText(tlvResults.m_hWnd, (int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
                buf[255] = 0;
                POINT p = { rc2.left, rc2.top };
                string country = Text::fromT(buf);
                Identity UserIdentity = ClientManager::getInstance()->GetOnlineUserIdentity(si->pSR->getUser());
				ImageList_Draw(WinUtil::userImages, UserIdentity.getcType() == 11 ? 23 : WinUtil::getImage(UserIdentity, true), cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
                if(FavoriteManager::getInstance()->isFavoriteUser(si->pSR->getUser())) SetTextColor(cd->nmcd.hdc, SETTING(TEXT_FAV_FORE_COLOR));
                LONG top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
                ::ExtTextOut(cd->nmcd.hdc, rc2.left + 20, top, ETO_CLIPPED, &rc2, buf, (uint32_t)_tcslen(buf), NULL);
                return CDRF_NEWFONT | CDRF_SKIPDEFAULT;
            } else if(colIndex == SearchInfo::COLUMN_USER_CONNECTION) {
                SearchInfo* si = (SearchInfo*)cd->nmcd.lItemlParam;
                rc.top = cd->iSubItem;
                rc.left = LVIR_BOUNDS;
                ::SendMessage(tlvResults.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);
                COLORREF color;
                if((UINT)::SendMessage(tlvResults.m_hWnd, LVM_GETITEMSTATE, (int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
                    if(tlvResults.m_hWnd == ::GetFocus()) {
                        color = GetSysColor(COLOR_HIGHLIGHT);
                        SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
                        SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
				    } else {
                        color = GetBkColor(cd->nmcd.hdc);
                        SetBkColor(cd->nmcd.hdc, color);
                    }
                } else {
				    color = WinUtil::bgColor;
				    SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
				    SetTextColor(cd->nmcd.hdc, WinUtil::textColor);
                }
                RECT rc2 = rc;
                rc2.left += 2;
                HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
                HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
                Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);
                DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
                DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));
                POINT p = { rc2.left, rc2.top };
				Identity UserIdentity = ClientManager::getInstance()->GetOnlineUserIdentity(si->pSR->getUser());
                ImageList_Draw(WinUtil::speedImages, WinUtil::getSpeedImage(UserIdentity.getcType()), cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
                LONG top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
				tstring tsConn = Text::toT(TO_UTF8(UserIdentity.getConnection()));
				::ExtTextOut(cd->nmcd.hdc, rc2.left + 20, top, ETO_CLIPPED, &rc2, tsConn.c_str(), (uint32_t)tsConn.size(), NULL);

                return CDRF_NEWFONT | CDRF_SKIPDEFAULT;
            } else if(colIndex == SearchInfo::COLUMN_USER_COUNTRY) {
                SearchInfo* si = (SearchInfo*)cd->nmcd.lItemlParam;
                if(si->pSR->getIP() != "" && BOOLSETTING(GET_USER_COUNTRY)) {
                    rc.top = cd->iSubItem;
                    rc.left = LVIR_BOUNDS;
                    ::SendMessage(tlvResults.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);
                    COLORREF color;
                    if((UINT)::SendMessage(tlvResults.m_hWnd, LVM_GETITEMSTATE, (int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
                        if(tlvResults.m_hWnd == ::GetFocus()) {
                            color = GetSysColor(COLOR_HIGHLIGHT);
                            SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
                            SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
                        } else {
                            color = GetBkColor(cd->nmcd.hdc);
                            SetBkColor(cd->nmcd.hdc, color);
                        }
                    } else {
                        color = WinUtil::bgColor;
                        SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
                        SetTextColor(cd->nmcd.hdc, cd->clrText);
                    }
                    RECT rc2 = rc;
                    rc2.left += 2;

                    HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
                    HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
                    Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);
                    DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
                    DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));
                    LONG top = rc2.top + ((rc2.bottom - rc2.top) - 15)/2;
                    if((top - rc2.top) < 2)
                        top = rc2.top + 1;
                    TCHAR buf[256];
                    ListViewGetItemText(tlvResults.m_hWnd, (int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
                    buf[255] = 0;
                    POINT p = { rc2.left, top };
                    ImageList_Draw(CountryFlags::flagImages, si->ui8CountryFlagImage, cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
                    top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
                    ::ExtTextOut(cd->nmcd.hdc, rc2.left + 29, top + 1, ETO_CLIPPED, &rc2, buf, (uint32_t)_tcslen(buf), NULL);
                    return CDRF_SKIPDEFAULT;
                }
            }
        }
        default:
            return CDRF_DODEFAULT;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnSpeaker(WPARAM wParam, LPARAM lParam) {
 	switch(wParam) {
		case ADD_SEARCH_RESULTS: {
            SearchInfo * nextNew = NULL,
                * curNew = NULL;

            {
                Lock l(csSearch);
                nextNew = pNewResults;
                pNewResults = NULL;
            }

            if(nextNew == NULL) {
                return;
            }

            ::SendMessage(tlvResults.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

            while(nextNew != NULL) {
                curNew = nextNew;
                nextNew = curNew->pNext;

                curNew->pNext = NULL;
                curNew->pPrev = NULL;

                ui64ResultsCount++;

                if(IsDupeResult(curNew) == true) {
                    continue;
                }

                if(bSearchPaused == false) {
                    AddResult(curNew);
                } else {
                    if(pPausedResults != NULL) {
                        pPausedResults->pPrev = curNew;
                        curNew->pNext = pPausedResults;
                    }
                    pPausedResults = curNew;

                    ui64ResultsPaused++;
                }
			}

            if(BOOLSETTING(BOLD_SEARCH)) {
                setDirty();
            }

            UpdateStatusBarResultsCount();

			::SendMessage(tlvResults.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);

			return;
		}
		case SEARCH_RESULTS_FILTERED:
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(Util::toString(ui64ResultsDropped) + ' ' + STRING(FILTERED)).c_str());
			return;
 		case ADD_HUB:
 			AddHub((HubInfo*)(lParam));
			return;
 		case CHANGE_HUB:
 			ChangeHub((HubInfo*)(lParam));
			return;
 		case REMOVE_HUB:
 			RemoveHub((HubInfo*)(lParam));
 			return;
 	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnRefreshSettings() {
    bool refresh = false;

	if((COLORREF)::SendMessage(tlvResults.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(tlvResults.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(tlvResults.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(lvHubs.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(lvHubs.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);

		refresh = true;
	}
	if((COLORREF)::SendMessage(tlvResults.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
		::SendMessage(tlvResults.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		::SendMessage(lvHubs.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}

	::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_RESETCONTENT, 0, 0);

    for(int i = 0; i <= (SettingsManager::SEARCH_MSG_RECENT_20-SettingsManager::SEARCH_MSG_RECENT_01); i++) {
        ::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_INSERTSTRING, (WPARAM)-1,
			(LPARAM)Text::toT(SettingsManager::getInstance()->get((SettingsManager::StrSetting)(SettingsManager::SEARCH_MSG_RECENT_01+i), true)).c_str());
    }

	::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_SETCURSEL, 0, 0);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnCopy(WORD wID) {
	int pos = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
	if(pos != -1) {
		SearchResult* sr = tlvResults.GetItemData(pos)->pSR;
		switch(wID) {
			case IDC_COPY_NICK:
				WinUtil::setClipboard(Text::toT(sr->getUser()->getFirstNick()));
				break;
			case IDC_COPY_FILENAME:
				WinUtil::setClipboard(Text::toT(Util::getFileName(sr->getUtf8() ? sr->getFile() : Text::acpToUtf8(sr->getFile()))));
				break;
			case IDC_COPY_PATH:
				WinUtil::setClipboard(Text::toT(Util::getFilePath(sr->getUtf8() ? sr->getFile() : Text::acpToUtf8(sr->getFile()))));
				break;
			case IDC_COPY_SIZE:
				WinUtil::setClipboard(Text::toT(Util::formatBytes(sr->getSize())));
				break;
			case IDC_COPY_LINK:
				if(sr->getType() == SearchResult::TYPE_FILE) {
					WinUtil::copyMagnet(sr->getTTH(), sr->getSize(), Text::toT(sr->getUtf8() ? sr->getFileName() : Text::acpToUtf8(sr->getFileName())));
                }
				break;
			case IDC_COPY_TTH:
				if(sr->getType() == SearchResult::TYPE_FILE && sr->getTTH()) {
					WinUtil::setClipboard(Text::toT(sr->getTTH()->toBase32()));
                }
				break;
			default:
				break;
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnGetFileList() {
    int i = -1;
    while((i = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
        SearchInfo * si = tlvResults.GetItemData(i);
        try {
            WinUtil::addInitalDir(si->pSR->getUser(), Text::fromT(si->tsColumns[SearchInfo::COLUMN_FILE_PATH-1]));
            QueueManager::getInstance()->addList(si->pSR->getUser(), QueueItem::FLAG_CLIENT_VIEW);
        } catch(const Exception& e) {
            ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(e.getError()).c_str());
            continue;
        }

        if(tlvResults.bGroupResults == true && si->pRoot->bExpanded == false) {
            SearchInfo * pNextSubItem = ((SearchInfo *)si->pRoot->pFirstSubItem)->pNext,
                * pCurSubItem = NULL;

            while(pNextSubItem != NULL) {
                pCurSubItem = pNextSubItem;
                pNextSubItem = pCurSubItem->pNext;

				try {
                    WinUtil::addInitalDir(pCurSubItem->pSR->getUser(), Text::fromT(pCurSubItem->tsColumns[SearchInfo::COLUMN_FILE_PATH-1]));
                    QueueManager::getInstance()->addList(pCurSubItem->pSR->getUser(), QueueItem::FLAG_CLIENT_VIEW);
                } catch(const Exception& e) {
                    ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(e.getError()).c_str());
                }
            }
        }
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnMatchQueue() {
    int i = -1;
    while((i = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
        SearchInfo * si = tlvResults.GetItemData(i);
        try {
            QueueManager::getInstance()->addList(si->pSR->getUser(), QueueItem::FLAG_MATCH_QUEUE);
        } catch(const Exception& e) {
            ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(e.getError()).c_str());
            continue;
        }

        if(tlvResults.bGroupResults == true && si->pRoot->bExpanded == false) {
            SearchInfo * pNextSubItem = ((SearchInfo *)si->pRoot->pFirstSubItem)->pNext,
                * pCurSubItem = NULL;

            while(pNextSubItem != NULL) {
                pCurSubItem = pNextSubItem;
                pNextSubItem = pCurSubItem->pNext;

				try {
                    QueueManager::getInstance()->addList(pCurSubItem->pSR->getUser(), QueueItem::FLAG_MATCH_QUEUE);
                } catch(const Exception& e) {
                    ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(e.getError()).c_str());
                }
            }
        }
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnSearchForAlternates() {
	if((UINT)::SendMessage(tlvResults.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		SearchResult* sr = tlvResults.GetItemData(i)->pSR;

		if(sr->getTTH() != NULL) {
			SearchFrame::OpenWindow(Text::toT(sr->getTTH()->toBase32()), 0, SearchManager::SIZE_DONTCARE, SearchManager::TYPE_TTH);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnViewAsText() {
    int i = -1;
    while((i = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
        SearchInfo * si = tlvResults.GetItemData(i);
        string target = Util::getTempPath() + Text::fromT(si->tsColumns[SearchInfo::COLUMN_FILE_NAME]);
        if(si->pSR->getType() == SearchResult::TYPE_FILE) {
            try {
                QueueManager::getInstance()->add(target, si->pSR->getSize(), si->pSR->getTTH(), si->pSR->getUser(), si->pSR->getFile(), si->pSR->getUtf8(),
                    QueueItem::FLAG_CLIENT_VIEW | QueueItem::FLAG_TEXT);
            } catch(const Exception& e) {
                ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(e.getError()).c_str());
                continue;
            }

            if(tlvResults.bGroupResults == true && si->pRoot->bExpanded == false) {
                SearchInfo * pNextSubItem = ((SearchInfo *)si->pRoot->pFirstSubItem)->pNext,
                    * pCurSubItem = NULL;

                while(pNextSubItem != NULL) {
                    pCurSubItem = pNextSubItem;
                    pNextSubItem = pCurSubItem->pNext;

				    try {
                        QueueManager::getInstance()->add(target, pCurSubItem->pSR->getSize(), pCurSubItem->pSR->getTTH(), pCurSubItem->pSR->getUser(), pCurSubItem->pSR->getFile(),
                            pCurSubItem->pSR->getUtf8(), QueueItem::FLAG_CLIENT_VIEW | QueueItem::FLAG_TEXT);
                    } catch(const Exception& e) {
                        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(e.getError()).c_str());
                    }
                }
            }
        }
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnDownloadBrowseTo() {
	if((UINT)::SendMessage(tlvResults.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		SearchInfo* si = tlvResults.GetItemData(i);

		if(si->pSR->getType() == SearchResult::TYPE_FILE) {
    		tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY)) + si->tsColumns[SearchInfo::COLUMN_FILE_NAME];
    		if(WinUtil::browseFile(target, m_hWnd)) {
    			WinUtil::addLastDir(Util::getFilePath(target));

    			AddDownload(si, target, true);
    		}
		} else {
    		tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
    		if(WinUtil::browseDirectory(target, m_hWnd)) {
    			WinUtil::addLastDir(target);

    			AddDownload(si, target, false);
    		}
		}
	} else {
		tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
		if(WinUtil::browseDirectory(target, m_hWnd)) {
			WinUtil::addLastDir(target);

			DownloadSelected(target, false);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnDownloadToFavoriteDir(WORD wID) {
	size_t newId = (size_t)wID - IDC_DOWNLOAD_FAVORITE_DIRS;

	StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
	if(newId < spl.size()) {
        DownloadSelected(Text::toT(spl[newId].first), false);
	} else {
		DownloadSelected(Text::toT(sTargetsList[newId - spl.size()]), true);
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnDownloadToTarget(WORD wID) {
	size_t newId = (size_t)wID - IDC_DOWNLOAD_TARGET;

	if(newId < WinUtil::lastDirs.size()) {
        DownloadSelected(WinUtil::lastDirs[newId], false);
	} else {
		DownloadSelected(Text::toT(sTargetsList[newId - WinUtil::lastDirs.size()]), true);
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnDownloadWithPriority(WORD wID) {
	QueueItem::Priority p;

	switch(wID) {
		case IDC_PRIORITY_PAUSED:
            p = QueueItem::PAUSED;
            break;
		case IDC_PRIORITY_LOWEST:
            p = QueueItem::LOWEST;
            break;
		case IDC_PRIORITY_LOW:
            p = QueueItem::LOW;
            break;
		case IDC_PRIORITY_NORMAL:
            p = QueueItem::NORMAL;
            break;
		case IDC_PRIORITY_HIGH:
            p = QueueItem::HIGH;
            break;
		case IDC_PRIORITY_HIGHEST:
            p = QueueItem::HIGHEST;
            break;
		default:
            p = QueueItem::DEFAULT;
            break;
	}

	DownloadSelected(Text::toT(SETTING(DOWNLOAD_DIRECTORY)), false, p);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnDownloadWholeBrowseTo() {
	tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
	if(WinUtil::browseDirectory(target, m_hWnd)) {
		WinUtil::addLastDir(target);

		DownloadWholeSelected(target);
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnDownloadWholeToFavoriteDir(WORD wID) {
	StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();

	DownloadWholeSelected(Text::toT(spl[wID-IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS].first));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnDownloadWholeToTarget(WORD wID) {
    DownloadWholeSelected(WinUtil::lastDirs[wID-IDC_DOWNLOAD_WHOLE_TARGET]);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::AddDownload(SearchInfo * si, const tstring& tsDir, const bool &bWhole, QueueItem::Priority Priority/* = QueueItem::DEFAULT*/) {
	if(si->pSR->getType() == SearchResult::TYPE_FILE) {
        string target;
        if(bWhole == false) {
            target = Text::fromT(tsDir + si->tsColumns[SearchInfo::COLUMN_FILE_NAME]);
        } else {
            target = Text::fromT(tsDir);
        }

        try {
            QueueManager::getInstance()->add(target, si->pSR->getSize(), si->pSR->getTTH(), si->pSR->getUser(), si->pSR->getFile(), si->pSR->getUtf8());
        } catch(const Exception& e) {
            ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(e.getError()).c_str());
            return;
        }

        if(tlvResults.bGroupResults == true && si->pRoot->bExpanded == false) {
            SearchInfo * pNextSubItem = ((SearchInfo *)si->pRoot->pFirstSubItem)->pNext,
                * pCurSubItem = NULL;

            while(pNextSubItem != NULL) {
                pCurSubItem = pNextSubItem;
                pNextSubItem = pCurSubItem->pNext;

				try {
                    QueueManager::getInstance()->add(target, pCurSubItem->pSR->getSize(), pCurSubItem->pSR->getTTH(), pCurSubItem->pSR->getUser(), pCurSubItem->pSR->getFile(),
                        pCurSubItem->pSR->getUtf8());
                } catch(const Exception& e) {
                    ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(e.getError()).c_str());
                }
            }
        }

		if(WinUtil::isShift() || Priority != QueueItem::DEFAULT) {
            QueueManager::getInstance()->setPriority(target, WinUtil::isShift() ? QueueItem::HIGHEST : Priority);
		}
	} else {
		QueueManager::getInstance()->addDirectory(si->pSR->getFile(), si->pSR->getUser(), Text::fromT(tsDir), WinUtil::isShift() ? QueueItem::HIGHEST : Priority);
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::AddResult(SearchInfo * si) {
    if(tlvResults.bGroupResults == true && si->pSR->getTTH() != NULL) {
        RootItem * pNextRoot = tlvResults.pFirstRoot,
            * pCurRoot = NULL;

        while(pNextRoot != NULL) {
            pCurRoot = pNextRoot;
            pNextRoot = pCurRoot->pNext;

            TTHValue * pTTH = ((SearchInfo *)pCurRoot->pFirstSubItem)->pSR->getTTH();
            if(pTTH != NULL && *pTTH == *si->pSR->getTTH()) {
                si->pRoot = pCurRoot;

                break;
            }
        }
    }

	tlvResults.InsertItem(si, bExpandGroups);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::CheckSelectedResults(ResultsCheck * rc) {
    int i = -1;
    while((i = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
        SearchInfo * si = tlvResults.GetItemData(i);

        CheckResult(rc, si);

        if(tlvResults.bGroupResults == true && si->pRoot->bExpanded == false) {
            SearchInfo * pNextSubItem = ((SearchInfo *)si->pRoot->pFirstSubItem)->pNext,
                * pCurSubItem = NULL;

            while(pNextSubItem != NULL) {
                pCurSubItem = pNextSubItem;
                pNextSubItem = pCurSubItem->pNext;

				CheckResult(rc, pCurSubItem);
            }
        }
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::CheckResult(ResultsCheck * rc, SearchInfo * si) {
	if(si->tsColumns[SearchInfo::COLUMN_FILE_HASH-1].empty() == false) {
		if(rc->bFirstHash == true) {
			rc->tsHash = si->tsColumns[SearchInfo::COLUMN_FILE_HASH-1];
			rc->bHasHash = true;
			rc->bFirstHash = false;
		} else if(rc->bHasHash == true) {
			if(rc->tsHash != si->tsColumns[SearchInfo::COLUMN_FILE_HASH-1]) {
				rc->bHasHash = false;
			}
		}
	} else {
		rc->bFirstHash = false;
		rc->bHasHash = false;
	}

	if(si->pSR->getType() == SearchResult::TYPE_FILE) {
		if(rc->tsExtension.empty() == true) {
			rc->tsExtension = Util::getFileExt(si->tsColumns[SearchInfo::COLUMN_FILE_NAME]);
			rc->i64Size = si->pSR->getSize();
		} else if(rc->i64Size != -1) {
			if((si->pSR->getSize() != rc->i64Size) || (Util::stricmp(rc->tsExtension, Util::getFileExt(si->tsColumns[SearchInfo::COLUMN_FILE_NAME])) != 0)) {
				rc->i64Size = -1;
            }
        }
	} else {
		rc->i64Size = -1;
	}

	StringList userhubs = ClientManager::getInstance()->getHubs(si->pSR->getUser()->getCID());
	string userhub;
	if(userhubs.size() > 0) {
        userhub = userhubs[0];
    }

	if(rc->bOneHub == true && rc->sHub.empty() == true) {
		rc->sHub = userhub;
	} else if(rc->sHub != userhub) {
		rc->bOneHub = false;
		rc->sHub.clear();
	}
	if(rc->bOperator == true) {
		rc->bOperator = userhub.empty() ? false : ClientManager::getInstance()->isOp(ClientManager::getInstance()->getMe(), userhub);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::DownloadSelected(const tstring& tsDir, const bool &bWhole, QueueItem::Priority Priority/* = QueueItem::DEFAULT*/) {
    int i = -1;
    while((i = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
        AddDownload(tlvResults.GetItemData(i), tsDir, bWhole, Priority);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::DownloadWholeSelected(const tstring& tsDir) {
    int i = -1;
    while((i = (int)::SendMessage(tlvResults.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
        SearchInfo * si = tlvResults.GetItemData(i);
        if(si->pSR->getType() == SearchResult::TYPE_FILE) {
            QueueManager::getInstance()->addDirectory(Text::fromT(si->tsColumns[SearchInfo::COLUMN_FILE_PATH-1]), si->pSR->getUser(), Text::fromT(tsDir),
                WinUtil::isShift() ? QueueItem::HIGHEST : QueueItem::DEFAULT);
        } else {
            QueueManager::getInstance()->addDirectory(si->pSR->getFile(), si->pSR->getUser(), Text::fromT(tsDir), WinUtil::isShift() ? QueueItem::HIGHEST : QueueItem::DEFAULT);
        }
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnEnter() {
	if(::GetWindowTextLength(hWindowItems[CB_SEARCH_STRING]) == 0) {
		return;
    }

    StringList sClients;

	int iHubsCount = (int)::SendMessage(lvHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
	for(int i = 0; i < iHubsCount; i++) {
		if((((UINT)::SendMessage(lvHubs.m_hWnd, LVM_GETITEMSTATE, i, LVIS_STATEIMAGEMASK) >> 12) - 1)) {
			sClients.push_back(Text::fromT(lvHubs.getItemData(i)->tsHubUrl));
		}
	}

	if(sClients.size() == 0) {
		return;
    }

	if(SearchManager::getInstance()->okToSearch() == false) {
		bUserWarned = true;

		int32_t ui32WaitTime = SearchManager::getInstance()->timeToSearch();

		TCHAR * buf = new TCHAR[TSTRING(SEARCHING_WAIT).size() + 16];
		_stprintf_s(buf, TSTRING(SEARCHING_WAIT).size() + 16, CTSTRING(SEARCHING_WAIT), ui32WaitTime);

		::SetWindowText(m_hWnd, (TSTRING(SEARCH) + _T(" - ") + tstring(buf)).c_str());
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)buf);

		delete[] buf;

		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)_T(""));

		uipTimer = ::SetTimer(m_hWnd, 1, 1000, NULL);

        return;
    }

	tstring tsSearchString(::GetWindowTextLength(hWindowItems[CB_SEARCH_STRING]) + 1, _T('\0'));
	::GetWindowText(hWindowItems[CB_SEARCH_STRING], &tsSearchString[0], (int)tsSearchString.size());

	tsSearchString.resize(tsSearchString.size()-1);

	tstring tsSearchSize(::GetWindowTextLength(hWindowItems[EDT_SIZE]) + 1, _T('\0'));
	::GetWindowText(hWindowItems[EDT_SIZE], &tsSearchSize[0], (int)tsSearchSize.size());

	tsSearchSize.resize(tsSearchSize.size()-1);

	int64_t i64SearchSize = Util::toInt64(Text::fromT(tsSearchSize));

	switch((int)::SendMessage(hWindowItems[CB_SIZE_UNITS], CB_GETCURSEL, 0, 0)) {
		case 1:
			i64SearchSize *= 1024;
            break;
		case 2:
			i64SearchSize *= 1048576;
            break;
		case 3:
			i64SearchSize *= 1073741824;
            break;
	}

	tlvResults.DeleteAllItems();

	{
		Lock l(csSearch);

		SearchWordsList = StringTokenizer<tstring>(tsSearchString, ' ').getTokens();
	}

	SearchManager::SizeModes smsmMode((SearchManager::SizeModes)((int)::SendMessage(hWindowItems[CB_SIZE_MODE], CB_GETCURSEL, 0, 0)));

	if(i64SearchSize == 0) {
		smsmMode = SearchManager::SIZE_DONTCARE;
    }

	int iFileType = (int)::SendMessage(hWindowItems[CB_FILE_TYPE], CB_GETCURSEL, 0, 0);

    {
        int iRet = (int )::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_FINDSTRINGEXACT, (WPARAM)-1, (LPARAM)tsSearchString.c_str());
        if(iRet != CB_ERR) {
            ::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_DELETESTRING, (WPARAM)iRet, 0);
        }
    }

	if((int)::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_GETCOUNT, 0, 0) >= 20) {
		::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_DELETESTRING, 19, 0);
    }

    ::SendMessage(hWindowItems[CB_SEARCH_STRING], CB_INSERTSTRING, 0, (LPARAM)tsSearchString.c_str());

	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)(TSTRING(SEARCHING_FOR) + tsSearchString + _T("...")).c_str());
	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)_T(""));

	ui64ResultsDropped = 0;

	bSearchingHash = (iFileType == SearchManager::TYPE_TTH);

	::SetWindowText(m_hWnd, (TSTRING(SEARCH) + _T(" - ") + tsSearchString).c_str());

    SearchInfo * nextPaused = pPausedResults,
        * curPaused = NULL;

    while(nextPaused != NULL) {
        curPaused = nextPaused;
        nextPaused = curPaused->pNext;

		delete curPaused;
	}

	pPausedResults = NULL;

	SearchInfo * nextNew = NULL,
        * curNew = NULL;

	{
		Lock l(csSearch);
		nextNew = pNewResults;
		pNewResults = NULL;
    }

    while(nextNew != NULL) {
        curNew = nextNew;
        nextNew = curNew->pNext;

		delete curNew;
	}

	ui64ResultsCount = 0;

	bSearchPaused = bUserWarned = false;

	::EnableWindow(hWindowItems[BTN_PAUSE_SEARCH], TRUE);
	::SetWindowText(hWindowItems[BTN_PAUSE_SEARCH], CTSTRING(PAUSE_SEARCH));

    if(::SendMessage(hWindowItems[CHB_HIDE_OPTIONS], BM_GETCHECK, 0, 0) == BST_CHECKED && ::SendMessage(hWindowItems[CHB_SHOW_SEARCH_OPTIONS], BM_GETCHECK, 0, 0) == BST_CHECKED) {
        ::SendMessage(hWindowItems[CHB_SHOW_SEARCH_OPTIONS], BM_SETCHECK, BST_UNCHECKED, 0);
    }

	SearchManager::getInstance()->search(sClients, Text::fromT(tsSearchString), i64SearchSize, (SearchManager::TypeModes)iFileType, smsmMode);

	if(BOOLSETTING(CLEAR_SEARCH)){
		::SetWindowText(hWindowItems[CB_SEARCH_STRING], _T(""));
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnHubItemChanged(LPARAM lParam) {
	NMLISTVIEW* pLV = (NMLISTVIEW*)lParam;

	if(pLV->iItem == 0 && (pLV->uNewState ^ pLV->uOldState) & LVIS_STATEIMAGEMASK) {
		if(((pLV->uNewState & LVIS_STATEIMAGEMASK) >> 12) - 1) {
			for(int iItem = 0; (iItem = (int)::SendMessage(lvHubs.m_hWnd, LVM_GETNEXTITEM, iItem, LVNI_ALL)) != -1;) {
				HubInfo* client = lvHubs.getItemData(iItem);
				if(client->bOperator == false) {
					ListView_SetItemState(lvHubs.m_hWnd, iItem, INDEXTOSTATEIMAGEMASK(1), LVIS_STATEIMAGEMASK);
				}
			}
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::runUserCommand(UserCommand& uc, SearchResult * sr, StringMap &ucParams, set<CID> &users) {
	if(sr->getUser()->isOnline() == false) {
		return;
	}

    if(uc.getType() == UserCommand::TYPE_RAW_ONCE) {
		if(users.find(sr->getUser()->getCID()) != users.end()) {
            return;
        }

        users.insert(sr->getUser()->getCID());
    }

	ucParams["fileFN"] = sr->getFile();
	ucParams["fileSI"] = Util::toString(sr->getSize());
	ucParams["fileSIshort"] = Util::formatBytes(sr->getSize());

	if(sr->getTTH() != NULL) {
		ucParams["fileTR"] = sr->getTTH()->toBase32();
	} else {
		ucParams["fileTR"] = "NONE";
	}

	// compatibility with 0.674 and earlier
	ucParams["file"] = ucParams["fileFN"];
	ucParams["filesize"] = ucParams["fileSI"];
	ucParams["filesizeshort"] = ucParams["fileSIshort"];
	ucParams["tth"] = ucParams["fileTR"];

	ClientManager::getInstance()->userCommand(sr->getUser(), uc, ucParams, true);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::AddHub(HubInfo * pInfo) {
	int iPos = lvHubs.insertItem(pInfo, 0);

    if(pInfo->bOperator == true || (((UINT)::SendMessage(lvHubs.m_hWnd, LVM_GETITEMSTATE, 0, LVIS_STATEIMAGEMASK) >> 12) - 1) == FALSE) {
        ListView_SetItemState(lvHubs.m_hWnd, iPos, INDEXTOSTATEIMAGEMASK(2), LVIS_STATEIMAGEMASK);
    }

    ::SendMessage(lvHubs.m_hWnd, LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::OnTab(const bool & bShift) {
    ::SetFocus(::GetNextDlgTabItem(m_hWnd, ::GetFocus(), bShift == true ? TRUE : FALSE));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::ChangeHub(HubInfo * pInfo) {
    int iCount = (int)::SendMessage(lvHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0);

    for(int iPos = 0; iPos < iCount; iPos++) {
        if(lvHubs.getItemData(iPos)->tsHubUrl == pInfo->tsHubUrl) {
            delete lvHubs.getItemData(iPos);

            LVITEM lvi = { 0 };
            lvi.mask = LVIF_PARAM;
            lvi.iItem = iPos;
            lvi.iSubItem = 0;
            lvi.stateMask = 0;
            lvi.state = 0;
            lvi.pszText = NULL;
            lvi.iImage = 0;
            lvi.lParam = (LPARAM)pInfo;
            ::SendMessage(lvHubs.m_hWnd, LVM_SETITEM, 0, (LPARAM)&lvi);

            lvHubs.updateItem(iPos);

            if((((UINT)::SendMessage(lvHubs.m_hWnd, LVM_GETITEMSTATE, 0, LVIS_STATEIMAGEMASK) >> 12) - 1) == TRUE && pInfo->bOperator == false) {
                ListView_SetItemState(lvHubs.m_hWnd, iPos, INDEXTOSTATEIMAGEMASK(1), LVIS_STATEIMAGEMASK);
            }

            ::SendMessage(lvHubs.m_hWnd, LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE);

            return;
        }
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::RemoveHub(HubInfo * pInfo) {
    int iCount = (int)::SendMessage(lvHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0);

    for(int iPos = 0; iPos < iCount; iPos++) {
        if(lvHubs.getItemData(iPos)->tsHubUrl == pInfo->tsHubUrl) {
            delete lvHubs.getItemData(iPos);
            ::SendMessage(lvHubs.m_hWnd, LVM_DELETEITEM, iPos, 0);

            return;
        }
    }

    delete pInfo;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::on(ClientManagerListener::ClientConnected, Client * pClient) {
    HubInfo* pHubInfo = new HubInfo(Text::toT(pClient->getHubUrl()), Text::toT(pClient->getHubName()), pClient->getMyIdentity().isOp());
    ::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)ADD_HUB, (LPARAM)pHubInfo);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::on(ClientManagerListener::ClientUpdated, Client * pClient) {
    HubInfo* pHubInfo = new HubInfo(Text::toT(pClient->getHubUrl()), Text::toT(pClient->getHubName()), pClient->getMyIdentity().isOp());
    ::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)CHANGE_HUB, (LPARAM)pHubInfo);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::on(ClientManagerListener::ClientDisconnected, Client * pClient) {
    HubInfo* pHubInfo = new HubInfo(Text::toT(pClient->getHubUrl()), Text::toT(pClient->getHubName()), pClient->getMyIdentity().isOp());
    ::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)REMOVE_HUB, (LPARAM)pHubInfo);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::on(SearchManagerListener::SR, SearchResult * pResult) {
	// Check that this is really a relevant search result...
	{
		Lock l(csSearch);

		if(SearchWordsList.empty() == true) {
			return;
		}

		if(bSearchingHash == true) {
			if(pResult->getTTH() == NULL || Util::stricmp(Text::toT(pResult->getTTH()->toBase32()), SearchWordsList[0]) != 0) {
				return;
            }
		} else {
			// match all here
			for(TStringIter j = SearchWordsList.begin(); j != SearchWordsList.end(); ++j) {
				if(Util::findSubString(pResult->getUtf8() ? pResult->getFile() : Text::acpToUtf8(pResult->getFile()), Text::fromT(*j)) == -1) {
					ui64ResultsDropped++;
					::PostMessage(m_hWnd, WM_SPEAKER, SEARCH_RESULTS_FILTERED, NULL);

					return;
				}
			}
		}
	}

	// Reject results without free slots or tth if selected
	// but always show directories
	if((bOnlyFreeSlots == true && pResult->getFreeSlots() < 1) || ((bSearchOnlyTTH == true && pResult->getTTH() == NULL) && (pResult->getType() != SearchResult::TYPE_DIRECTORY))) {
		ui64ResultsDropped++;
		::PostMessage(m_hWnd, WM_SPEAKER, SEARCH_RESULTS_FILTERED, NULL);

		return;
	}

	SearchInfo* pSI = new SearchInfo(pResult);

	{
		Lock l(csSearch);
        if(pNewResults != NULL) {
            pNewResults->pPrev = pSI;
            pSI->pNext = pNewResults;
        }
        pNewResults = pSI;
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::on(SettingsManagerListener::Save, SimpleXML&) {
    ::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::on(TimerManagerListener::Second, uint64_t /*aTick*/) throw() {
	::PostMessage(m_hWnd, WM_SPEAKER, ADD_SEARCH_RESULTS, 0);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool SearchFrame::IsDupeResult(SearchInfo * si) {
    if(BOOLSETTING(FILTER_SEARCH) == true) {
        if(tlvResults.bGroupResults == true) {
            RootItem * pNextRoot = tlvResults.pFirstRoot,
                * pCurRoot = NULL;

            SearchInfo * pNextSubItem = NULL, * pCurSubItem = NULL;

            while(pNextRoot != NULL) {
                pCurRoot = pNextRoot;
                pNextRoot = pCurRoot->pNext;

                pNextSubItem = (SearchInfo *)pCurRoot->pFirstSubItem;

                if(si->pSR->getTTH() != pNextSubItem->pSR->getTTH()) {
                    continue;
                }

                while(pNextSubItem != NULL) {
                    pCurSubItem = pNextSubItem;
                    pNextSubItem = pCurSubItem->pNext;

                    if((si->pSR->getUser()->getCID() == pCurSubItem->pSR->getUser()->getCID()) && (si->pSR->getFile() == pCurSubItem->pSR->getFile())) {
                        delete si;
                        ui64ResultsCount--;
                        return true;
                    }
                }
            }
        } else {
            for(int i = 0; i < (int)::SendMessage(tlvResults.m_hWnd, LVM_GETITEMCOUNT, 0, 0); i++) {
                SearchInfo* si2 = tlvResults.GetItemData(i);
                if((si->pSR->getUser()->getCID() == si2->pSR->getUser()->getCID()) && (si->pSR->getFile() == si2->pSR->getFile())) {
                    delete si;
                    ui64ResultsCount--;
                    return true;
                }
            }
        }

        if(pPausedResults != NULL) {
            SearchInfo * nextPaused = pPausedResults,
                * curPaused = NULL;

            while(nextPaused != NULL) {
                curPaused = nextPaused;
                nextPaused = curPaused->pNext;

                if((si->pSR->getUser()->getCID() == curPaused->pSR->getUser()->getCID()) && (si->pSR->getFile() == curPaused->pSR->getFile())) {
                    delete si;
                    ui64ResultsCount--;
                    return true;
                }
            }
        }
    }

    return false;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchFrame::UpdateStatusBarResultsCount() {
    if(bSearchPaused == false) {
        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)Text::toT(Util::toString(ui64ResultsCount) + " " + STRING(FILES)).c_str());
    } else {
        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0),
            (LPARAM)Text::toT(Util::toString(ui64ResultsCount-ui64ResultsPaused) + "/" + Util::toString(ui64ResultsCount) + " " + STRING(FILES)).c_str());
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
