/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
#include "../client/SearchManager.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "Resource.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "SearchInfo.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "CountryFlags.h"
#include "TreeListViewCtrl.h"
#include "WinUtil.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

SearchInfo::SearchInfo(SearchResult * pSearchResult) : pSR(pSearchResult), pRoot(NULL), pNext(NULL), pPrev(NULL) {
    pSR->incRef();
    Update();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

SearchInfo::~SearchInfo() {
    pSR->decRef();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

int SearchInfo::GetImageIndex() {
    return (pSR->getType() == SearchResult::TYPE_FILE) ? WinUtil::getIconIndex(tsColumns[COLUMN_FILE_NAME]) : WinUtil::getDirIconIndex();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

const TCHAR * SearchInfo::GetText(const int & iColumn) const {
    return tsColumns[iColumn].c_str();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

int SearchInfo::CompareItems(const SearchInfo * pFirst, const SearchInfo * pSecond, const int & iColumn) {
    switch(iColumn) {
        case COLUMN_HITS_COUNT:
            return compare(pFirst->pRoot != NULL ? pFirst->pRoot->iItemCount : 0, pSecond->pRoot != NULL ? pSecond->pRoot->iItemCount : 0);
        case COLUMN_FILE_NAME:
            return _tcsicmp(pFirst->tsColumns[iColumn].c_str(), pSecond->tsColumns[iColumn].c_str());
        case COLUMN_FILE_TYPE:
            if(pFirst->pSR->getType() == pSecond->pSR->getType()) {
                return _tcsicmp(pFirst->tsColumns[COLUMN_FILE_TYPE-1].c_str(), pSecond->tsColumns[COLUMN_FILE_TYPE-1].c_str());
            } else {
                return (pFirst->pSR->getType() != SearchResult::TYPE_DIRECTORY) ? 1 : -1;
            }
        case COLUMN_USER_SLOTS:
            if(pFirst->pSR->getFreeSlots() == pSecond->pSR->getFreeSlots()) {
                return compare(pFirst->pSR->getSlots(), pSecond->pSR->getSlots());
            } else {
                return compare(pFirst->pSR->getFreeSlots(), pSecond->pSR->getFreeSlots());
            }
        case COLUMN_FILE_SIZE:
        case COLUMN_FILE_EXACT_SIZE:
            return compare(pFirst->pSR->getSize(), pSecond->pSR->getSize());
        default:
            return _tcsicmp(pFirst->tsColumns[iColumn-1].c_str(), pSecond->tsColumns[iColumn-1].c_str());
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void SearchInfo::Update() {
	if(pSR->getType() == SearchResult::TYPE_FILE) {
		if(pSR->getFile().rfind(_T('\\')) == tstring::npos) {
			tsColumns[COLUMN_FILE_NAME] = Text::toT(pSR->getUtf8() ? pSR->getFile() : Text::acpToUtf8(pSR->getFile()));
		} else {
			tsColumns[COLUMN_FILE_NAME] = Text::toT(Util::getFileName(pSR->getUtf8() ? pSR->getFile() : Text::acpToUtf8(pSR->getFile())));
			tsColumns[COLUMN_FILE_PATH-1] = Text::toT(Util::getFilePath(pSR->getUtf8() ? pSR->getFile() : Text::acpToUtf8(pSR->getFile())));
		}

		tsColumns[COLUMN_FILE_TYPE-1] = Text::toT(Util::getFileExt(Text::fromT(tsColumns[COLUMN_FILE_NAME])));
		if(tsColumns[COLUMN_FILE_TYPE-1].empty() == false && tsColumns[COLUMN_FILE_TYPE-1][0] == _T('.')) {
			tsColumns[COLUMN_FILE_TYPE-1].erase(0, 1);
        }
		tsColumns[COLUMN_FILE_SIZE-1] = Text::toT(Util::formatBytes(pSR->getSize()));
		tsColumns[COLUMN_FILE_EXACT_SIZE-1] = Text::toT(Util::formatExactSize(pSR->getSize()));
	} else {
		tsColumns[COLUMN_FILE_NAME] = Text::toT(pSR->getUtf8() ? pSR->getFileName() : Text::acpToUtf8(pSR->getFileName()));
		tsColumns[COLUMN_FILE_PATH-1] = Text::toT(pSR->getUtf8() ? pSR->getFile() : Text::acpToUtf8(pSR->getFile()));
		tsColumns[COLUMN_FILE_TYPE-1] = TSTRING(DIRECTORY);
		if(pSR->getSize() > 0) {
			tsColumns[COLUMN_FILE_SIZE-1] = Text::toT(Util::formatBytes(pSR->getSize()));
			tsColumns[COLUMN_FILE_EXACT_SIZE-1] = Text::toT(Util::formatExactSize(pSR->getSize()));
		}
	}
	tsColumns[COLUMN_USER_NICK-1] = WinUtil::getNicks(pSR->getUser());
	tsColumns[COLUMN_USER_CONNECTION-1] = Text::toT(TO_UTF8(ClientManager::getInstance()->getConnection(pSR->getUser()->getCID())));
	tsColumns[COLUMN_USER_HUB-1] = Text::toT(pSR->getHubName());
	tsColumns[COLUMN_USER_SLOTS-1] = Text::toT(pSR->getSlotString());

    if(ClientManager::getInstance()->IsMeOpOnUserHub(pSR->getUser())) {
        tsColumns[COLUMN_USER_COUNTRY-1] = Text::toT(pSR->getIP());
    }

	if(pSR->getTTH() != NULL) {
		tsColumns[COLUMN_FILE_HASH-1] = Text::toT(pSR->getTTH()->toBase32());
    }

	ui8CountryFlagImage = CountryFlags::GetFlagImageIndex(Util::getIpCountry(pSR->getIP().c_str()).c_str());
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
