/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "SpyFrame.h"
#include "SearchFrame.h"
#include "WinUtil.h"
#include "FlatTabCtrl.h"

#include "../client/ShareManager.h"
#include "../client/ResourceManager.h"

int SpyFrame::columnSizes[] = { 305, 70, 85 };
int SpyFrame::columnIndexes[] = { COLUMN_STRING, COLUMN_COUNT, COLUMN_TIME };
static ResourceManager::Strings columnNames[] = { ResourceManager::SEARCH_STRING, ResourceManager::COUNT, ResourceManager::TIME };

ATOM SpyFrame::m_aAtom = 0;
UINT SpyFrame::m_uiResourceID = IDR_SPY;

SpyFrame::SpyFrame() : total(0), cur(0), closed(false), ignoreTth(BOOLSETTING(SPY_FRAME_IGNORE_TTH_SEARCHES)), mnu(NULL) {
	ZeroMemory(perSecond, sizeof(perSecond));
	ClientManager::getInstance()->addListener(this);
	TimerManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);

    RegisterWindowClass(m_aAtom, SpyFrameWndProc, _T("SpyFrame"), m_uiResourceID);
}

SpyFrame::~SpyFrame() {
    if(mnu != NULL) {
        ::DestroyMenu(mnu);
    }
}

LRESULT CALLBACK SpyFrame::SpyFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	SpyFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (SpyFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (SpyFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->SpyFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT SpyFrame::SpyFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_SPEAKER:
            onSpeaker(wParam, lParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CONTEXTMENU:
			if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDC_SEARCH) {
                onSearch();
                return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_RESULTS && ((LPNMHDR)lParam)->code == LVN_COLUMNCLICK) {
                onColumnClickResults((LPNMHDR)lParam);
                return 0;
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

    return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK SpyFrame::IgnoreTigerWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
		SpyFrame * pParent = (SpyFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->ignoreTth = (wParam == BST_CHECKED);
        }
    }

    return ::CallWindowProc(OldButtonProc, hWnd, uMsg, wParam, lParam);
}

void SpyFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_SPY);
    created = true;

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, 0, 0, 0, 0, m_hWnd, 0, g_hInstance, NULL);

	ctrlSearches.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SINGLESEL, WS_EX_CLIENTEDGE,
        IDC_RESULTS, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT);

    hWndIgnoreTth = CreateWindowEx(0, WC_BUTTON, CTSTRING(IGNORE_TTH_SEARCHES), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_AUTOCHECKBOX,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, 0, g_hInstance, NULL);
	::SendMessage(hWndIgnoreTth, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndIgnoreTth, GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWndIgnoreTth, GWLP_WNDPROC, (LONG_PTR)IgnoreTigerWndProc);
	::SendMessage(hWndIgnoreTth, BM_SETCHECK, (ignoreTth == true ? BST_CHECKED : BST_UNCHECKED), 0);

	WinUtil::splitTokens(columnIndexes, SETTING(SPYFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(SPYFRAME_WIDTHS), COLUMN_LAST);

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		int fmt = (ui8i == COLUMN_COUNT) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		ListViewInsertColumn(ctrlSearches.m_hWnd, ui8i, CTSTRING_I(columnNames[ui8i]), fmt, columnSizes[ui8i], ui8i);
	}

	ctrlSearches.setSort(COLUMN_COUNT, ExListViewCtrl::SORT_INT, false);

	mnu = ::CreatePopupMenu();
	::AppendMenu(mnu, MF_STRING, IDC_SEARCH, CTSTRING(SEARCH));

	ShareManager::getInstance()->setHits(0);
}

bool SpyFrame::onClose() {
	if(!closed){
		ClientManager::getInstance()->removeListener(this);
		TimerManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);

		return true;
	} else {
		WinUtil::saveHeaderOrder(ctrlSearches.m_hWnd, SettingsManager::SPYFRAME_ORDER, SettingsManager::SPYFRAME_WIDTHS, COLUMN_LAST, columnIndexes, columnSizes);
		if (ignoreTth != BOOLSETTING(SPY_FRAME_IGNORE_TTH_SEARCHES))
			SettingsManager::getInstance()->set(SettingsManager::SPY_FRAME_IGNORE_TTH_SEARCHES, ignoreTth);

		return false;
	}
}

void SpyFrame::onColumnClickResults(LPNMHDR pnmh) {
	NMLISTVIEW* l = (NMLISTVIEW*)pnmh;
	if(l->iSubItem == ctrlSearches.getSortColumn()) {
		if (!ctrlSearches.isAscending()) {
			ctrlSearches.setSort(-1, ctrlSearches.getSortType());
		} else {
			ctrlSearches.setSortDirection(false);
        }
	} else {
		if(l->iSubItem == COLUMN_COUNT) {
			ctrlSearches.setSort(l->iSubItem, ExListViewCtrl::SORT_INT);
		} else {
			ctrlSearches.setSort(l->iSubItem, ExListViewCtrl::SORT_STRING_NOCASE);
		}
	}
}

void SpyFrame::UpdateLayout() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[5];
		::GetClientRect(m_hWndStatusBar, &sr);

		int tmp = (sr.right - sr.left) > 616 ? 516 : (((sr.right - sr.left) > 116) ? (sr.right - sr.left) - 100 : 16);

		w[0] = sr.right - tmp;
		w[1] = w[0] + (tmp-16)*1/4;
		w[2] = w[0] + (tmp-16)*2/4;
		w[3] = w[0] + (tmp-16)*3/4;
		w[4] = w[0] + (tmp-16)*4/4;

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 5, (LPARAM)w);

		::SendMessage(m_hWndStatusBar, SB_GETRECT, 0, (LPARAM)&sr);
		::MoveWindow(hWndIgnoreTth, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
	}

	::MoveWindow(ctrlSearches.m_hWnd, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, TRUE);
}

void SpyFrame::onSpeaker(WPARAM wParam, LPARAM lParam) {
	if(wParam == SEARCH) {
		tstring* x = (tstring*)lParam;

		total++;

		// Not thread safe, but who cares really...
		perSecond[cur]++;

		int j = ctrlSearches.find(*x);
		if(j == -1) {
			TStringList a;
			a.push_back(*x);
			a.push_back(Text::toT(Util::toString(1)));
			a.push_back(Text::toT(Util::getTimeString()));
			ctrlSearches.insert(a);
			if((int)::SendMessage(ctrlSearches.m_hWnd, LVM_GETITEMCOUNT, 0, 0) > 500) {
				::SendMessage(ctrlSearches.m_hWnd, LVM_DELETEITEM, (int)::SendMessage(ctrlSearches.m_hWnd, LVM_GETITEMCOUNT, 0, 0) - 1, 0);
			}
		} else {
			TCHAR tmp[32];
			ListViewGetItemText(ctrlSearches.m_hWnd, j, COLUMN_COUNT, tmp, 32);
			ListViewSetItem(ctrlSearches.m_hWnd, j, COLUMN_COUNT, LVIF_TEXT, Text::toT(Util::toString(Util::toInt(Text::fromT(tmp))+1)).c_str(), 0);
			ListViewGetItemText(ctrlSearches.m_hWnd, j, COLUMN_TIME, tmp, 32);
			ListViewSetItem(ctrlSearches.m_hWnd, j, COLUMN_TIME, LVIF_TEXT, Text::toT(Util::getTimeString()).c_str(), 0);
			if(ctrlSearches.getSortColumn() == COLUMN_COUNT )
				ctrlSearches.resort();
			if(ctrlSearches.getSortColumn() == COLUMN_TIME )
				ctrlSearches.resort();
		}
		delete x;

		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(STRING(TOTAL) + Util::toString(total)).c_str());
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (3 | 0), (LPARAM)Text::toT(STRING(HITS) + Util::toString(ShareManager::getInstance()->getHits())).c_str());
		double ratio = total > 0 ? ((double)ShareManager::getInstance()->getHits()) / (double)total : 0.0;
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (4 | 0), (LPARAM)Text::toT(STRING(HIT_RATIO) + Util::toString(ratio)).c_str());
	} else if(wParam == TICK_AVG) {
		float* x = (float*)lParam;
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)Text::toT(STRING(AVERAGE) + Util::toString(*x)).c_str());
		delete x;
	}
}

bool SpyFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	if((HWND)wParam != ctrlSearches.m_hWnd || (UINT)::SendMessage(ctrlSearches.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) != 1) {
        return false;
    }

    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);
		
	if(iX == -1 && iY == -1) {
		ListViewGetContextMenuPos(ctrlSearches.m_hWnd, iX, iY);
	}

	int i = (int)::SendMessage(ctrlSearches.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);

	AutoArray<TCHAR> buf(256);
	ListViewGetItemText(ctrlSearches.m_hWnd, i, COLUMN_STRING, buf, 256);
	searchString = buf;

    ::TrackPopupMenuEx(mnu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

	return true;
}

void SpyFrame::onSearch() {
	if(Util::strnicmp(searchString.c_str(), _T("TTH:"), 4) == 0) {
		SearchFrame::OpenWindow(searchString.substr(4), 0, SearchManager::SIZE_DONTCARE, SearchManager::TYPE_TTH);
	} else {
		SearchFrame::OpenWindow(searchString);
    }
}

void SpyFrame::on(ClientManagerListener::IncomingSearch, const string& s) throw() {
	if(ignoreTth && s.compare(0, 4, "TTH:") == 0)
		return;
	tstring* x = new tstring(Text::toT(s));
	tstring::size_type i = 0;
	while( (i=x->find(_T('$'))) != string::npos) {
		(*x)[i] = _T(' ');
	}
	::PostMessage(m_hWnd, WM_SPEAKER, SEARCH, (LPARAM)x);
}

void SpyFrame::on(TimerManagerListener::Second, uint64_t) throw() {
	float* f = new float(0.0);
	for(int i = 0; i < AVG_TIME; ++i) {
		(*f) += (float)perSecond[i];
	}
	(*f) /= AVG_TIME;

	cur = (cur + 1) % AVG_TIME;
	perSecond[cur] = 0;
	::PostMessage(m_hWnd, WM_SPEAKER, TICK_AVG, (LPARAM)f);
}

void SpyFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void SpyFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlSearches.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlSearches.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlSearches.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlSearches.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
		::SendMessage(ctrlSearches.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
