/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef SPY_FRAME_H
#define SPY_FRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../client/ClientManager.h"
#include "../client/TimerManager.h"

#include "MDIChildWindow.h"
#include "ExListViewCtrl.h"

class SpyFrame : public MDITabChildWindow, public StaticFrame<SpyFrame, ResourceManager::SEARCH_SPY, IDC_SEARCH_SPY>, private ClientManagerListener,
    private TimerManagerListener, private SettingsManagerListener {
public:
	SpyFrame();
	~SpyFrame();

	enum {
		COLUMN_FIRST,
		COLUMN_STRING = COLUMN_FIRST,
		COLUMN_COUNT,
		COLUMN_TIME,
		COLUMN_LAST
	};

	static int columnIndexes[COLUMN_LAST];
	static int columnSizes[COLUMN_LAST];

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK SpyFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT SpyFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void OnCreate();
	void onSpeaker(WPARAM wParam, LPARAM lParam);
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onSearch();
	bool onClose();
	void onColumnClickResults(LPNMHDR pnmh);
	void onRefreshSettings();
	void UpdateLayout();
private:

	enum { AVG_TIME = 60 };
	enum {
		SEARCH,
		TICK_AVG
	};

	ExListViewCtrl ctrlSearches;
	HWND hWndIgnoreTth;
	int total;
	int perSecond[AVG_TIME];
	int cur;
	tstring searchString;
	HMENU mnu;

	bool closed;
	bool ignoreTth;

	// ClientManagerListener
	void on(ClientManagerListener::IncomingSearch, const string& s) throw();

	// TimerManagerListener
	void on(TimerManagerListener::Second, uint64_t) throw();
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

    static LRESULT CALLBACK IgnoreTigerWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif // SPY_FRAME_H

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
