/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "../client/Socket.h"

#include "StatsFrame.h"
#include "WinUtil.h"
#include "WinApiUtil.h"
#include "FlatTabCtrl.h"

ATOM StatsFrame::m_aAtom = 0;
UINT StatsFrame::m_uiResourceID = IDR_NET_STATS;

StatsFrame::StatsFrame() : width(0), height(0), timerId(0), twidth(0), lastTick(GET_TICK()), scrollTick(0),
	lastUp(Socket::getTotalUp()), lastDown(Socket::getTotalDown()), max(0), closed(false) {
	backgr = ::CreateSolidBrush(WinUtil::bgColor);
	upload = ::CreatePen(PS_SOLID, 0, SETTING(UPLOAD_BAR_COLOR));
	download = ::CreatePen(PS_SOLID, 0, SETTING(DOWNLOAD_BAR_COLOR));
	foregr = ::CreatePen(PS_SOLID, 0, WinUtil::textColor);

    RegisterWindowClass(m_aAtom, StatsFrameWndProc, _T("StatsFrame"), m_uiResourceID);
}

StatsFrame::~StatsFrame() {
    ::DeleteObject(backgr);
    ::DeleteObject(upload);
    ::DeleteObject(download);
    ::DeleteObject(foregr);
}

LRESULT CALLBACK StatsFrame::StatsFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	StatsFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (StatsFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (StatsFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->StatsFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT StatsFrame::StatsFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_PAINT:
            onPaint();
            return 0;
        case WM_TIMER:
            onTimer();
            return 0;
        case WM_SIZE:
            onSize();
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

void StatsFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_NET_STATS);
    created = true;

	timerId = ::SetTimer(m_hWnd, 1, 1000, NULL);

	::SendMessage(m_hWnd, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	SettingsManager::getInstance()->addListener(this);
}

bool StatsFrame::onClose() {
	if(!closed) {
		if(timerId != 0) {
			::KillTimer(m_hWnd, timerId);
		}

        SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);

		return true;
	} else {
		return false;
	}
}

void StatsFrame::drawLine(HDC& dc, StatIter begin, StatIter end, RECT& rc, RECT& crc) {
	int x = crc.right;

	StatIter i;
	for(i = begin; i != end; ++i) {
		if((x - (int)i->scroll) < rc.right)
			break;
		x -= i->scroll;
	}
	if(i != end) {
		int y = (max == 0) ? 0 : (int)((i->speed * height) / max);
		::MoveToEx(dc, x, height - y, NULL);
		x -= i->scroll;
		++i;

		for(; i != end && x > twidth; ++i) {
			y = (max == 0) ? 0 : (int)((i->speed * height) / max);
			::LineTo(dc, x, height - y);
			if(x < rc.left)
				break;
			x -= i->scroll;
		}
	}
}

void StatsFrame::onPaint() {
	if(::GetUpdateRect(m_hWnd, NULL, FALSE)) {
        PAINTSTRUCT m_ps;
        HDC dc = ::BeginPaint(m_hWnd, &m_ps);
		RECT rc(m_ps.rcPaint);
		dcdebug("Update: %d, %d, %d, %d\n", rc.left, rc.top, rc.right, rc.bottom);

		::SelectObject(dc, backgr);
		::BitBlt(dc, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, NULL, 0, 0, PATCOPY);

		RECT clientRC;
		::GetClientRect(m_hWnd, &clientRC);

		::SetTextColor(dc, WinUtil::textColor);
		::SetBkColor(dc, WinUtil::bgColor);

		::SelectObject(dc, foregr);
		::SelectObject(dc, WinUtil::font);
		int lines = height / (WinUtil::fontHeight * LINE_HEIGHT);
		int lheight = height / (lines+1);

		for(int i = 0; i < lines; ++i) {
			int ypos = lheight * (i+1);
			if(ypos > WinUtil::fontHeight + 2) {
				::MoveToEx(dc, rc.left, ypos, NULL);
				::LineTo(dc, rc.right, ypos);
			}

			if(rc.left <= twidth) {
				ypos -= WinUtil::fontHeight + 2;
				if(ypos < 0)
					ypos = 0;
				if(height == 0)
					height = 1;
				tstring txt = Text::toT(Util::formatBytes(max * (height-ypos) / height) + "/s");
				int tw = WinUtil::getTextWidth(txt, dc);
				if(tw + 2 > twidth)
					twidth = tw + 2;
				::TextOut(dc, 1, ypos, txt.c_str(), (int)txt.size());
			}
		}

		if(rc.left < twidth) {
			tstring txt = Text::toT(Util::formatBytes(max) + "/s");
			int tw = WinUtil::getTextWidth(txt, dc);
			if(tw + 2 > twidth)
				twidth = tw + 2;
			::TextOut(dc, 1, 1, txt.c_str(), (int)txt.size());
		}

		::SelectObject(dc, upload);
		drawLine(dc, up.begin(), up.end(), rc, clientRC);

		::SelectObject(dc, download);
		drawLine(dc, down.begin(), down.end(), rc, clientRC);

        ::EndPaint(m_hWnd, &m_ps);
	}
}

void StatsFrame::addTick(uint64_t bdiff, uint64_t tdiff, StatList& lst, AvgList& avg, uint32_t scroll) {
	while((int)lst.size() > ((width / PIX_PER_SEC) + 1) ) {
		lst.pop_back();
	}
	while(avg.size() > AVG_SIZE ) {
		avg.pop_back();
	}
	uint64_t bspeed = bdiff * (uint64_t)1000 / tdiff;
	avg.push_front(bspeed);

	bspeed = 0;

	for(AvgIter ai = avg.begin(); ai != avg.end(); ++ai) {
		bspeed += *ai;
	}

	bspeed /= avg.size();
	lst.push_front(Stat(scroll, bspeed));
}

void StatsFrame::onTimer() {
	uint64_t tick = GET_TICK();
	uint64_t tdiff = tick - lastTick;
	if(tdiff == 0) {
		return;
    }

	uint64_t scrollms = (tdiff + scrollTick)*PIX_PER_SEC;
	uint32_t scroll = (uint32_t)(scrollms / 1000);

	if(scroll == 0) {
		return;
    }

	scrollTick = scrollms - (scroll * 1000);

	RECT rc;
	::GetClientRect(m_hWnd, &rc);
	rc.left = twidth;
	::ScrollWindow(m_hWnd, -((int)scroll), 0, &rc, &rc);

	uint64_t d = Socket::getTotalDown();
	uint64_t ddiff = d - lastDown;
	uint64_t u = Socket::getTotalUp();
	uint64_t udiff = u - lastUp;

	addTick(ddiff, tdiff, down, downAvg, scroll);
	addTick(udiff, tdiff, up, upAvg, scroll);

	uint64_t mspeed = 0;
	StatIter i;
	for(i = down.begin(); i != down.end(); ++i) {
		if(mspeed < i->speed)
			mspeed = i->speed;
	}
	for(i = up.begin(); i != up.end(); ++i) {
		if(mspeed < i->speed)
			mspeed = i->speed;
	}
	if(mspeed > max || ((max * 3 / 4) > mspeed) ) {
		max = mspeed;
		::InvalidateRect(m_hWnd, NULL, TRUE);
	}

	lastTick = tick;
	lastUp = u;
	lastDown = d;
}

void StatsFrame::onSize() {
	RECT rc;
	::GetClientRect(m_hWnd, &rc);
	width = rc.right - rc.left;
	height = (rc.bottom - rc.top) - 1;
	::InvalidateRect(m_hWnd, NULL, TRUE);
}

void StatsFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void StatsFrame::onRefreshSettings() {
    ::DeleteObject(backgr);
	backgr = ::CreateSolidBrush(WinUtil::bgColor);
	::DeleteObject(upload);
	upload = ::CreatePen(PS_SOLID, 0, SETTING(UPLOAD_BAR_COLOR));
	::DeleteObject(download);
	download = ::CreatePen(PS_SOLID, 0, SETTING(DOWNLOAD_BAR_COLOR));
	::DeleteObject(foregr);
	foregr = ::CreatePen(PS_SOLID, 0, WinUtil::textColor);
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
