/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef STATS_FRAME_H
#define STATS_FRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../client/TimerManager.h"

#include "MDIChildWindow.h"
#include "WinUtil.h"

class StatsFrame : public MDITabChildWindow, public StaticFrame<StatsFrame, ResourceManager::NETWORK_STATISTICS, IDC_NET_STATS>,
    private SettingsManagerListener {
public:
	StatsFrame();
	~StatsFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK StatsFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT StatsFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void onCreate();
	bool onClose();
	void onPaint();
	void onTimer();
	void onSize();
	void onRefreshSettings();
private:
	// Pixels per second
	enum { PIX_PER_SEC = 2 };
	enum { LINE_HEIGHT = 10 };
	enum { AVG_SIZE = 5 };

	HBRUSH backgr;
	HPEN upload;
	HPEN download;
	HPEN foregr;

	struct Stat {
		Stat() : scroll(0), speed(0) { }
		Stat(uint32_t aScroll, uint64_t aSpeed) : scroll(aScroll), speed(aSpeed) { }
		uint32_t scroll;
		uint64_t speed;
	};
	typedef deque<Stat> StatList;
	typedef StatList::iterator StatIter;
	typedef deque<uint64_t> AvgList;
	typedef AvgList::iterator AvgIter;

	StatList up;
	AvgList upAvg;
	StatList down;
	AvgList downAvg;

	int width;
	int height;
	UINT_PTR timerId;
	int twidth;

	uint64_t lastTick;
	uint64_t scrollTick;
	uint64_t lastUp;
	uint64_t lastDown;

	uint64_t max;
	bool closed;

	void drawLine(HDC& dc, StatIter begin, StatIter end, RECT& rc, RECT& crc);
	void addTick(uint64_t bdiff, uint64_t tdiff, StatList& lst, AvgList& avg, uint32_t scroll);

	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();
};

#endif // STATS_FRAME_H

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
