/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "WinUtil.h"
#include "SystemFrame.h"
#include "WinApiUtil.h"
#include "../client/File.h"
#include "FlatTabCtrl.h"

ATOM SystemFrame::m_aAtom = 0;
UINT SystemFrame::m_uiResourceID = IDR_SYSTEM_LOG;

SystemFrame::SystemFrame() : tabMenu(NULL) {
    RegisterWindowClass(m_aAtom, SystemFrameWndProc, _T("SystemFrame"), m_uiResourceID);
}

SystemFrame::~SystemFrame() {
    if(tabMenu != NULL) {
        ::DestroyMenu(tabMenu);
    }
}

LRESULT CALLBACK SystemFrame::SystemFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	SystemFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (SystemFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (SystemFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->SystemFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT SystemFrame::SystemFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_SETFOCUS:
            OnFocus();
            break;
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_CLOSE:
            onClose();
            break;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORSTATIC: {
            LRESULT lResult = onCtlColor(wParam, lParam);
            if(lResult != 0) {
                return lResult;
            }
            break;
        }
        case WM_SPEAKER:
            onSpeaker(wParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case FTM_CONTEXTMENU:
            onTabContextMenu(lParam);
            return TRUE;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDC_OPEN_SYSTEN_LOG) {
                onSystemLog();
                return 0;
            } else if(LOWORD(wParam) == IDC_CLOSE_WINDOW) {
                onCloseWindow();
                return 0;
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK SystemFrame::EditWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_LBUTTONDBLCLK) {
		SystemFrame * pParent = (SystemFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			if(pParent->onLButton(lParam) == true) {
                return 0;
            }
        }
    }

    return ::CallWindowProc(OldEditProc, hWnd, uMsg, wParam, lParam);
}

void SystemFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_SYSTEM_LOG);
    created = true;

	hWndPad = CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_VSCROLL | ES_AUTOVSCROLL | ES_MULTILINE | ES_NOHIDESEL | ES_READONLY,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0, g_hInstance, NULL);
	::SendMessage(hWndPad, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndPad, GWLP_USERDATA, (LONG_PTR)this);
    OldEditProc = (WNDPROC)::SetWindowLongPtr(hWndPad, GWLP_WNDPROC, (LONG_PTR)EditWndProc);

	deque<pair<time_t, string> > oldMessages = LogManager::getInstance()->getLastLogs();
	// Technically, we might miss a message or two here, but who cares...
	LogManager::getInstance()->addListener(this);

	for(deque<pair<time_t, string> >::iterator i = oldMessages.begin(); i != oldMessages.end(); ++i) {
		addLine(i->first, Text::toT(i->second));
	}

	tabMenu = ::CreatePopupMenu();
	if(BOOLSETTING(LOG_SYSTEM)) {
        ::AppendMenu(tabMenu, MF_STRING, IDC_OPEN_SYSTEN_LOG, CTSTRING(OPEN_SYSTEM_LOG));
		::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
	}
	::AppendMenu(tabMenu, MF_STRING, IDC_CLOSE_WINDOW, CTSTRING(CLOSE));

	SettingsManager::getInstance()->addListener(this);
}

void SystemFrame::onClose() {
	LogManager::getInstance()->removeListener(this);
	SettingsManager::getInstance()->removeListener(this);
}

void SystemFrame::UpdateLayout() {
	RECT rc;
	::GetClientRect(m_hWnd, &rc);

	rc.bottom -= 1;
	rc.top += 1;
	rc.left +=1;
	rc.right -=1;
	::MoveWindow(hWndPad, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

bool SystemFrame::onLButton(LPARAM lParam) {
	if(GetFocus() == hWndPad) {
		POINT pt = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };
		tstring x;
		tstring::size_type start = (tstring::size_type)WinUtil::textUnderCursor(pt, hWndPad, x);
		tstring::size_type end = x.find(_T(" "), start);

		if(end == tstring::npos)
			end = x.length();

		return WinUtil::parseDBLClick(x, start, end);
	}

	return false;
}

void SystemFrame::onSpeaker(WPARAM wParam) {
	auto_ptr<pair<time_t, tstring> > msg((pair<time_t, tstring>*)wParam);
	addLine(msg->first, msg->second);

	if(BOOLSETTING(BOLD_SYSTEM_LOG)) {
		setDirty();
    }
}

void SystemFrame::addLine(time_t t, const tstring& msg) {
	if(::GetWindowTextLength(hWndPad) > 25000) {
		// We want to limit the buffer to 25000 characters...after that, w95 becomes sad...
		::SendMessage(hWndPad, WM_SETREDRAW, (WPARAM)FALSE, 0);

		::SendMessage(hWndPad, EM_SETSEL, 0, (int)::SendMessage(hWndPad, EM_LINEINDEX, (int)::SendMessage(hWndPad, EM_LINEFROMCHAR, 2000, 0), 0));
		::SendMessage(hWndPad, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)_T(""));

		::SendMessage(hWndPad, WM_SETREDRAW, (WPARAM)TRUE, 0);
	}

    ::SendMessage(hWndPad, EM_SETSEL, ::GetWindowTextLength(hWndPad), ::GetWindowTextLength(hWndPad));
    ::SendMessage(hWndPad, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)(Text::toT("\r\n[" + Util::getShortTimeString(t) + "] ") + msg).c_str());
}

void SystemFrame::onTabContextMenu(LPARAM lParam) {
	::TrackPopupMenuEx(tabMenu, TPM_LEFTALIGN | (BOOLSETTING(TABS_ON_TOP) == false ? TPM_BOTTOMALIGN : TPM_TOPALIGN) | TPM_RIGHTBUTTON,
        GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), m_hWnd, NULL);
}

void SystemFrame::onSystemLog() {
	string filename = SETTING(LOG_DIRECTORY) + SETTING(LOG_FILE_SYSTEM);
	if(File::FileExist(filename)) {
		::ShellExecute(NULL, NULL, Text::toT(filename).c_str(), NULL, NULL, SW_SHOWNORMAL);
	} else {
		::MessageBox(m_hWnd, CTSTRING(NO_SYSTEM_LOG), CTSTRING(NO_SYSTEM_LOG), MB_OK);
	}
}

void SystemFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void SystemFrame::onRefreshSettings() {
	::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
