/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#ifndef TREE_LIST_VIEW_CTRL_H
#define TREE_LIST_VIEW_CTRL_H
//----------------------------------------------------------------------------------------------------------------------------------------------------
#pragma once
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../Client/ResourceManager.h"
#include "../Client/SettingsManager.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "WinApiUtil.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

struct RootItem {
    RootItem * pNext, *pPrev;

    void * pFirstSubItem, *pLastSubItem;

    int iItemCount;

    bool bExpanded;

    RootItem(void * pItem, const bool &bExpandGroups) {
        pNext = NULL;
        pPrev = NULL;

        pFirstSubItem = pItem;
        pLastSubItem = pItem;

        iItemCount = 1;

        bExpanded = bExpandGroups;
    }

    ~RootItem() { }
};
//----------------------------------------------------------------------------------------------------------------------------------------------------

template<class T>
class TreeListViewCtrl {
public:
    RootItem * pFirstRoot;

    HWND m_hWnd;

    bool bGroupResults;

    typedef TreeListViewCtrl<T> ThisTreeListViewCtrl;

    TreeListViewCtrl() {
        pFirstRoot = NULL;

        m_hWnd = NULL;

        bGroupResults = false;

        ipColumnIndexes = NULL;
        ipColumnSizes = NULL;

        spColumnNames = NULL;

        iColumnCount = 0;
        iSortColumn = 0;
        iHitsColumn = 0;

        bSortAscending = true;
    }

    ~TreeListViewCtrl() {
        delete [] ipColumnIndexes;
        delete [] ipColumnSizes;
    }

	LRESULT TreeListViewProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
        switch(uMsg) {
            case WM_LBUTTONDOWN:
                if(bGroupResults == true) {
                    if(OnLButtonDown(lParam) == true) {
                        return 0;
                    }
                }

                break;
            case WM_MENUCOMMAND:
                OnHeaderMenu((int)wParam+2);
                break;
            case WM_CHAR:
                if(onChar() == true) {
                    return 0;
                }

                break;
        }

        return ::CallWindowProc(OldListViewProc, hWnd, uMsg, wParam, lParam);
    }

    void Create(const HWND &hWndParent, const DWORD &dwStyle, const DWORD &dwExStyle, const UINT &uiID, const DWORD &dwExListViewStyle, const int & iX, const int & iY, const int & iWidth,
        const int & iHeight, int * pColumnIndexes, ResourceManager::Strings * pColumnNames, int * pColumnSizes, const int &iColumnLast, const int &iColumnHits) {
        spColumnNames = pColumnNames;
        iColumnCount = iColumnLast;
        iHitsColumn = iColumnHits;

        ipColumnIndexes = new int[iColumnLast];
        memcpy(ipColumnIndexes, pColumnIndexes, iColumnCount * sizeof(int));

        ipColumnSizes = new int[iColumnLast];
        memcpy(ipColumnSizes, pColumnSizes, iColumnCount * sizeof(int));

        m_hWnd = ::CreateWindowEx(dwExStyle, WC_LISTVIEW, _T(""), dwStyle, iX, iY, iWidth, iHeight, hWndParent, (HMENU)uiID, g_hInstance, NULL);

        ::SendMessage(m_hWnd, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

        ::SendMessage(m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, dwExListViewStyle);
        ::SendMessage(m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
        ::SendMessage(m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
        ::SendMessage(m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);

        OldListViewProc = (WNDPROC)::GetWindowLongPtr(m_hWnd, GWLP_WNDPROC);

        if(bGroupResults == true) {
            if(WinUtil::hStateImages == NULL && (Util::OsMajor > 5 || (Util::OsMajor == 5 && Util::OsMinor >= 1))) {
                WinUtil::CreateStateImages();
            }

            if(WinUtil::hStateImages == NULL) {
                WinUtil::hStateImages = ImageList_LoadImage(g_hInstance, MAKEINTRESOURCE(IDB_STATE), 16, 1, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
            }

            ::SendMessage(m_hWnd, LVM_SETIMAGELIST, LVSIL_STATE, (LPARAM)WinUtil::hStateImages);
        }
    }

    void InsertColumn(const int &iCol, const int &iFormat, const int &iSubItem) {
        LVCOLUMN lvColumn = { 0 };
        lvColumn.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH | LVCF_SUBITEM | LVCF_ORDER;
        lvColumn.pszText = (LPTSTR)CTSTRING_I(spColumnNames[iSubItem]);
        lvColumn.fmt = iFormat;
        lvColumn.cx = ipColumnSizes[iSubItem];
        lvColumn.iSubItem = iSubItem;
        lvColumn.iOrder = ipColumnIndexes[iSubItem];

        ::SendMessage(m_hWnd, LVM_INSERTCOLUMN, iCol, (LPARAM)&lvColumn);
    }

    int GetRealColumn(const int &iColumn) {
        LVCOLUMN lvColumn = { 0 };
        lvColumn.mask = LVCF_SUBITEM;
        if(::SendMessage(m_hWnd, LVM_GETCOLUMN, iColumn, (LPARAM)&lvColumn) == TRUE) {
            return lvColumn.iSubItem;
        }

        return 0;
    }

    int GetVirtualColumn(const int &iColumn) {
        for(int i = 0; i < (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0); i++) {
            LVCOLUMN lvColumn = { 0 };
            lvColumn.mask = LVCF_SUBITEM;
            if(::SendMessage(m_hWnd, LVM_GETCOLUMN, i, (LPARAM)&lvColumn) == TRUE) {
                if(lvColumn.iSubItem == iColumn) {
                    return i;
                }
            }
        }

        return 0;
    }

    void ShowMenu(const POINT &pt) {
        HMENU hMenu = ::CreatePopupMenu();

        MENUINFO MenuInfo = { 0 };
        MenuInfo.cbSize = sizeof(MENUINFO);
        MenuInfo.fMask = MIM_STYLE;
        MenuInfo.dwStyle = MNS_NOTIFYBYPOS;
        ::SetMenuInfo(hMenu, &MenuInfo);

        for(int i = 2; i < iColumnCount; i++) {
            ::AppendMenu(hMenu, MF_STRING, IDC_HEADER_MENU, CTSTRING_I(spColumnNames[i]));
        }

        for(int j = 0; j < (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0); j++) {
            LVCOLUMN lvColumn = { 0 };
            lvColumn.mask = LVCF_SUBITEM;
            if(::SendMessage(m_hWnd, LVM_GETCOLUMN, j, (LPARAM)&lvColumn) == TRUE) {
                ::CheckMenuItem(hMenu, lvColumn.iSubItem-2, MF_BYPOSITION | MF_CHECKED);
            }
        }

        ::TrackPopupMenuEx(hMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, m_hWnd, NULL);

        ::DestroyMenu(hMenu);
    }

    void OnHeaderMenu(const int &iColumn) {
        ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

        for(int i = 0; i < (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0); i++) {
            LVCOLUMN lvColumn = { 0 };
            lvColumn.mask = LVCF_WIDTH | LVCF_SUBITEM | LVCF_ORDER;
            if(::SendMessage(m_hWnd, LVM_GETCOLUMN, i, (LPARAM)&lvColumn) == TRUE) {
                if(lvColumn.iSubItem == iColumn) {
                    ipColumnSizes[lvColumn.iSubItem] = lvColumn.cx;
                    ipColumnIndexes[lvColumn.iSubItem] = lvColumn.iOrder;

                    ::SendMessage(m_hWnd, LVM_DELETECOLUMN, i, 0);

                    if(lvColumn.iSubItem == iSortColumn) {
                        SetSortColumn(0);
                    }

                    ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
                    ::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);

                    return;
                }
            }
        }

        LVCOLUMN lvColumn = { 0 };
        lvColumn.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH | LVCF_SUBITEM | LVCF_ORDER;
        lvColumn.pszText = (LPTSTR)CTSTRING_I(spColumnNames[iColumn]);
        lvColumn.fmt = (iColumn == 4 || iColumn == 9) ? LVCFMT_RIGHT : LVCFMT_LEFT;
        lvColumn.cx = ipColumnSizes[iColumn];
        lvColumn.iSubItem = (int)iColumn;
        lvColumn.iOrder = ipColumnIndexes[iColumn];

        ::SendMessage(m_hWnd, LVM_INSERTCOLUMN, iColumn, (LPARAM)&lvColumn);

        ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
        ::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
    }

    void SaveHeaderProperties(SettingsManager::StrSetting ColumnsOrder, SettingsManager::StrSetting ColumnsWidths, SettingsManager::StrSetting ColumnsVisibility) {
        char * sVisibleColumns = new char[iColumnCount+1];
		memset(sVisibleColumns, '0', iColumnCount);
		sVisibleColumns[iColumnCount] = '\0';

        for(int i = 0; i < (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0); i++) {
            LVCOLUMN lvColumn = { 0 };
            lvColumn.mask = LVCF_WIDTH | LVCF_SUBITEM | LVCF_ORDER;
            if(::SendMessage(m_hWnd, LVM_GETCOLUMN, i, (LPARAM)&lvColumn) == TRUE) {
                ipColumnSizes[lvColumn.iSubItem] = lvColumn.cx;
                ipColumnIndexes[lvColumn.iSubItem] = lvColumn.iOrder;
				sVisibleColumns[lvColumn.iSubItem] = '1';
            }
        }

		string sColumnsOrder, sColumnsWidths;

		for(int j = 0; j < iColumnCount; j++) {
			if(sColumnsOrder.size() != 0) {
				sColumnsOrder += ',';
				sColumnsWidths += ',';
			}

			sColumnsOrder += Util::toString(ipColumnIndexes[j]);
			sColumnsWidths += Util::toString(ipColumnSizes[j]);
		}

        SettingsManager::getInstance()->set(ColumnsOrder, sColumnsOrder);
        SettingsManager::getInstance()->set(ColumnsWidths, sColumnsWidths);
        SettingsManager::getInstance()->set(ColumnsVisibility, sVisibleColumns);

        delete [] sVisibleColumns;
    }

    void OnColumnClick(LPARAM lParam) {
        LPNMLISTVIEW nmListView = (LPNMLISTVIEW)lParam;

        if(nmListView->iSubItem != iSortColumn) {
            bSortAscending = true;
            iSortColumn = nmListView->iSubItem;
        } else {
            bSortAscending = !bSortAscending;
        }

        ListViewUpdateArrow(m_hWnd, bSortAscending, GetVirtualColumn(iSortColumn));

        SortItems();
    }

    void SetAscending(const bool &bAscending) {
        bSortAscending = bAscending;
		ListViewUpdateArrow(m_hWnd, bSortAscending, GetVirtualColumn(iSortColumn));
    }

    void SetSortColumn(const int &iColumn, bool bIsReal = false) {
        if(bIsReal == true) {
            iSortColumn = iColumn;
        } else {
            iSortColumn = GetRealColumn(iColumn);
        }

        ListViewUpdateArrow(m_hWnd, bSortAscending, GetVirtualColumn(iSortColumn));
    }

    void InsertItem(T* pItem, const bool &bExpandGroups) {
        if(bGroupResults == true) {
            if(pItem->pRoot == NULL) {
                pItem->pRoot = new RootItem((void *)pItem, bExpandGroups);

                pItem->pRoot->pNext = pFirstRoot;
                if(pFirstRoot != NULL) {
                    pFirstRoot->pPrev = pItem->pRoot;
                }
                pFirstRoot = pItem->pRoot;

                ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE, GetRootInsertPos(pItem), LPSTR_TEXTCALLBACK, 0, 0, I_IMAGECALLBACK, (LPARAM)pItem);
            } else {
                pItem->pRoot->iItemCount++;

                int iCmpRes = T::CompareItems(pItem, (T*)pItem->pRoot->pFirstSubItem, iSortColumn);

                if(bSortAscending == false) {
                    iCmpRes = -iCmpRes;
                }

                if(iCmpRes < 0) {
                    int iPos = GetItemPos((T*)pItem->pRoot->pFirstSubItem);

                    pItem->pNext = (T*)pItem->pRoot->pFirstSubItem;
                    ((T*)pItem->pRoot->pFirstSubItem)->pPrev = pItem;
                    pItem->pRoot->pFirstSubItem = pItem;

                    if(pItem->pRoot->bExpanded == true && pItem->pRoot->iItemCount != 2) {
                        pItem->pRoot->iItemCount--;
                        DeleteSubItems(pItem->pRoot, iPos);
                        pItem->pRoot->iItemCount++;
                    }

                    ::SendMessage(m_hWnd, LVM_DELETEITEM, iPos, 0);

                    iPos = GetRootInsertPos(pItem);

                    ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE | LVIF_STATE, iPos, LPSTR_TEXTCALLBACK,
                        INDEXTOSTATEIMAGEMASK(pItem->pRoot->bExpanded == false ? 1 : 2), LVIS_STATEIMAGEMASK, I_IMAGECALLBACK, (LPARAM)pItem);

                    if(pItem->pRoot->bExpanded == true) {
                        InsertSubItems(pItem->pRoot, iPos);
                    }

                    return;
                }

                pItem->pPrev = (T*)pItem->pRoot->pLastSubItem;
                ((T*)pItem->pRoot->pLastSubItem)->pNext = pItem;
                pItem->pRoot->pLastSubItem = pItem;

                if(iSortColumn == iHitsColumn) {
                    int iPos = GetItemPos((T*)pItem->pRoot->pFirstSubItem);

                    if(pItem->pRoot->bExpanded == true && pItem->pRoot->iItemCount != 2) {
                        pItem->pRoot->iItemCount--;
                        DeleteSubItems(pItem->pRoot, iPos);
                        pItem->pRoot->iItemCount++;
                    }

                    ::SendMessage(m_hWnd, LVM_DELETEITEM, iPos, 0);

                    iPos = GetRootInsertPos((T*)pItem->pRoot->pFirstSubItem);

                    ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE | LVIF_STATE, iPos, LPSTR_TEXTCALLBACK,
                        INDEXTOSTATEIMAGEMASK(pItem->pRoot->bExpanded == false ? 1 : 2), LVIS_STATEIMAGEMASK, I_IMAGECALLBACK, (LPARAM)pItem->pRoot->pFirstSubItem);

                    if(pItem->pRoot->bExpanded == true) {
                        InsertSubItems(pItem->pRoot, iPos);
                    }

                    return;
                }

                if(pItem->pRoot->iItemCount == 2) {
                    ListView_SetItemState(m_hWnd, GetItemPos((T*)pItem->pRoot->pFirstSubItem), INDEXTOSTATEIMAGEMASK(pItem->pRoot->bExpanded == false ? 1 : 2), LVIS_STATEIMAGEMASK);
                }

                if(pItem->pRoot->bExpanded == true) {
                    if(iCmpRes == 0) {
                        ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE | LVIF_INDENT, GetItemPos((T*)pItem->pRoot->pFirstSubItem)+1, LPSTR_TEXTCALLBACK, 0, 0,
                            I_IMAGECALLBACK, (LPARAM)pItem);
                    } else {
                        ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE | LVIF_INDENT,
                            GetSubItemInsertPos(pItem, GetItemPos((T*)pItem->pRoot->pFirstSubItem), pItem->pRoot->iItemCount-1), LPSTR_TEXTCALLBACK, 0, 0, I_IMAGECALLBACK, (LPARAM)pItem);
                    }
                }
            }
        } else {
            ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE, GetInsertPos(pItem), LPSTR_TEXTCALLBACK, 0, 0, I_IMAGECALLBACK, (LPARAM)pItem);
        }
    }

    void OnGetDispInfo(LPARAM lParam) {
        NMLVDISPINFO * lvDispInfo = (NMLVDISPINFO *)lParam;

        if(lvDispInfo->item.mask & LVIF_TEXT) {
            int iColumn = GetRealColumn(lvDispInfo->item.iSubItem);

            if(iColumn == iHitsColumn) {
                T * pItem = (T*)lvDispInfo->item.lParam;

                if(pItem->pRoot->iItemCount == 1 || pItem != pItem->pRoot->pFirstSubItem) {
                    lvDispInfo->item.pszText = (LPTSTR)Util::emptyStringT.c_str();
                    return;
                }

                static TCHAR tcBuf[256];
                _stprintf(tcBuf, _T("%d %s"), pItem->pRoot->iItemCount, CTSTRING(USERS));
                lvDispInfo->item.pszText = tcBuf;

                return;
            } else if(iColumn > iHitsColumn) {
                iColumn--;
            }

            lvDispInfo->item.pszText = (LPTSTR)((T*)lvDispInfo->item.lParam)->GetText(iColumn);
        }

        if(lvDispInfo->item.mask & LVIF_IMAGE) {
            lvDispInfo->item.iImage = ((T*)lvDispInfo->item.lParam)->GetImageIndex();
        }

        lvDispInfo->item.mask |= LVIF_DI_SETITEM;
    }

    T* GetItemData(const int &iItem) {
        return (T*)ListViewGetItem(m_hWnd, iItem);
    }

	int GetItemPos(T * pItem) {
		LVFINDINFO lvFindInfo = { 0 };
        lvFindInfo.flags = LVFI_PARAM;
        lvFindInfo.lParam = (LPARAM)pItem;
		return (int)::SendMessage(m_hWnd, LVM_FINDITEM, (WPARAM)-1, (LPARAM)&lvFindInfo);
	}

    T* GetSelectedItem() {
        int iItem = (int)::SendMessage(m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
        if(iItem != -1) {
            return GetItemData(iItem);
        }

        return NULL;
    };

    void forEachSelected(void (T::*pFunction)()) {
        int i = -1;
        while((i = (int)::SendMessage(m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
            (GetItemData(i)->*pFunction)();
        }
    }

    void DeleteAllItems() {
        ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

        if(bGroupResults == true) {
            RootItem * pNextRoot = pFirstRoot,
                * pCurRoot = NULL;

            T * pNextSubItem = NULL, * pCurSubItem = NULL;

            while(pNextRoot != NULL) {
                pCurRoot = pNextRoot;
                pNextRoot = pCurRoot->pNext;

                pNextSubItem = (T*)pCurRoot->pFirstSubItem;

                while(pNextSubItem != NULL) {
                    pCurSubItem = pNextSubItem;
                    pNextSubItem = pCurSubItem->pNext;

                    delete pCurSubItem;
                }

                delete pCurRoot;
            }

            pFirstRoot = NULL;
        } else {
            for(int i = 0; i < (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0); i++) {
                delete GetItemData(i);
            }
        }

        ::SendMessage(m_hWnd, LVM_DELETEALLITEMS, 0, 0);

        ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
    }

    uint64_t DeleteSelectedItems() {
        uint64_t ui64DeletedCount = 0;

        ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

        int i = -1;
        while((i = (int)::SendMessage(m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
            T * pItem = GetItemData(i);

            if(bGroupResults == true) {
                if(pItem->pRoot->pFirstSubItem == pItem) {
                    if(pItem->pRoot->iItemCount != 1) {
                        if(pItem->pRoot->bExpanded == false) {
                            T * pNextSubItem = pItem->pNext,
                                * pCurSubItem = NULL;

                            while(pNextSubItem != NULL) {
                                pCurSubItem = pNextSubItem;
                                pNextSubItem = pCurSubItem->pNext;

                                delete pCurSubItem;

                                ui64DeletedCount++;
                            }
                        } else {
                            for(int j = i+1; j < i + pItem->pRoot->iItemCount; j++) {
                                delete GetItemData(i+1);
                                ::SendMessage(m_hWnd, LVM_DELETEITEM, i+1, 0);
                                ui64DeletedCount++;
                            }
                        }
                    }

                    if(pItem->pRoot->pPrev == NULL) {
                        if(pItem->pRoot->pNext == NULL) {
                            pFirstRoot = NULL;
                        } else {
                            pItem->pRoot->pNext->pPrev = NULL;
                            pFirstRoot = pItem->pRoot->pNext;
                        }
                    } else if(pItem->pRoot->pNext == NULL) {
                        pItem->pRoot->pPrev->pNext = NULL;
                    } else {
                        pItem->pRoot->pPrev->pNext = pItem->pRoot->pNext;
                        pItem->pRoot->pNext->pPrev = pItem->pRoot->pPrev;
                    }

                    delete pItem->pRoot;
                } else {
                    if(pItem->pNext == NULL) {
                        pItem->pPrev->pNext = NULL;
                        pItem->pRoot->pLastSubItem = pItem->pPrev;
                    } else {
                        pItem->pPrev->pNext = pItem->pNext;
                        pItem->pNext->pPrev = pItem->pPrev;
                    }

                    pItem->pRoot->iItemCount--;

                    if(pItem->pRoot->iItemCount == 1) {
                        ListView_SetItemState(m_hWnd, i-1, INDEXTOSTATEIMAGEMASK(0), LVIS_STATEIMAGEMASK);
                    }
                }
            }

            delete pItem;
            ::SendMessage(m_hWnd, LVM_DELETEITEM, i, 0);
            ui64DeletedCount++;
	   }

	   ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);

	   return ui64DeletedCount;
    }
private:
    int * ipColumnIndexes;
    int * ipColumnSizes;

    ResourceManager::Strings * spColumnNames;

    int iColumnCount;
    int iSortColumn;
    int iHitsColumn;

    bool bSortAscending;

	int GetInsertPos(T* pItem) {
		int iHighLim = (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0) - 1;

		if(iHighLim == -1) {
			return 0;
        }

        int iCmpRes = 0, iLowLim = 0, iInsertPos = 0;

        T * pOtherItem = NULL;

        while(iLowLim <= iHighLim) {
            iInsertPos = (iLowLim + iHighLim) / 2;
            pOtherItem = GetItemData(iInsertPos);

            iCmpRes = T::CompareItems(pItem, pOtherItem, iSortColumn);

            if(iCmpRes == 0) {
                return iInsertPos;
            }

			if(bSortAscending == false) {
				iCmpRes = -iCmpRes;
            }

            if(iCmpRes < 0) {
                iHighLim = iInsertPos-1;
            } else {
                iLowLim = iInsertPos+1;
            }
        }

        if(iCmpRes > 0) {
            iInsertPos++;
        }

        return iInsertPos;
	}

    int GetRootInsertPos(T* pItem) {
		int iHighLim = (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0) - 1;

		if(iHighLim == -1) {
			return 0;
        }

        int iCmpRes = 0, iLowLim = 0, iInsertPos = 0;

        T * pOtherItem = NULL;

        while(iLowLim <= iHighLim) {
            iInsertPos = (iLowLim + iHighLim) / 2;
            pOtherItem = GetItemData(iInsertPos);

            if((T *)pOtherItem->pRoot->pFirstSubItem != pOtherItem) {
                pOtherItem = (T *)pOtherItem->pRoot->pFirstSubItem;
                iInsertPos = GetItemPos(pOtherItem);
            }

            iCmpRes = T::CompareItems(pItem, pOtherItem, iSortColumn);

            if(iCmpRes == 0) {
                return iInsertPos;
            }

			if(bSortAscending == false) {
				iCmpRes = -iCmpRes;
            }

            if(iCmpRes < 0) {
                iHighLim = iInsertPos-1;
            } else {
                if(pOtherItem->pRoot->bExpanded == true) {
                    iLowLim = iInsertPos+pOtherItem->pRoot->iItemCount;
                } else {
                    iLowLim = iInsertPos+1;
                }

                if(iLowLim > iHighLim) {
                    return iLowLim;
                }
            }
        }

        if(iCmpRes > 0) {
            iInsertPos++;
        }

        return iInsertPos;
	}

    int GetSubItemInsertPos(T* pItem, const int &iRootPos, const int &iSubItemCount) {
        int iLowLim = iRootPos+1;
        int iHighLim = iRootPos+iSubItemCount;

        if(iHighLim == (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0)) {
            T * pOtherItem = GetItemData(iHighLim-1);

            int iCmpRes = T::CompareItems(pItem, pOtherItem, iSortColumn);

            if(bSortAscending == false) {
                iCmpRes = -iCmpRes;
            }

            if(iCmpRes < 0) {
                return iHighLim-1;
            }

            return iHighLim;
        }

        int iCmpRes = 0, iInsertPos = 0;

        T * pOtherItem = NULL;

        while(iLowLim <= iHighLim) {
            iInsertPos = (iLowLim + iHighLim) / 2;
            pOtherItem = GetItemData(iInsertPos);

            if(pOtherItem == pOtherItem->pRoot->pFirstSubItem) {
                return iInsertPos;
            }
            iCmpRes = T::CompareItems(pItem, pOtherItem, iSortColumn);

            if(iCmpRes == 0) {
                return iInsertPos;
            }

			if(bSortAscending == false) {
				iCmpRes = -iCmpRes;
            }

            if(iCmpRes < 0) {
                iHighLim = iInsertPos-1;
            } else {
                iLowLim = iInsertPos+1;
            }
        }

        if(iCmpRes > 0) {
            iInsertPos++;
        }

        return iInsertPos;
    }

    void SortItems() {
        if(bGroupResults == false) {
            ::SendMessage(m_hWnd, LVM_SORTITEMS, (WPARAM)this, (LPARAM)&CompareFunction);
            return;
        }

        ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

        ::SendMessage(m_hWnd, LVM_DELETEALLITEMS, 0, 0);

        RootItem * pNextRoot = pFirstRoot,
            * pCurRoot = NULL;

        T * pRootSubItem = NULL, * pNextSubItem = NULL, * pCurSubItem = NULL;

        while(pNextRoot != NULL) {
            pCurRoot = pNextRoot;
            pNextRoot = pCurRoot->pNext;

            pRootSubItem = pNextSubItem = (T*)pCurRoot->pFirstSubItem;

            while(pNextSubItem != NULL) {
                pCurSubItem = pNextSubItem;
                pNextSubItem = pCurSubItem->pNext;

                int iCmpRes = T::CompareItems(pCurSubItem, pRootSubItem, iSortColumn);

                if(bSortAscending == false) {
                    iCmpRes = -iCmpRes;
                }

                if(iCmpRes < 0) {
                    pRootSubItem = pCurSubItem;
                }
            }

            if(pRootSubItem != pCurRoot->pFirstSubItem) {
                if(pRootSubItem->pNext == NULL) {
                    pRootSubItem->pPrev->pNext = NULL;
                    pCurRoot->pLastSubItem = pRootSubItem->pPrev;
                } else {
                    pRootSubItem->pPrev->pNext = pRootSubItem->pNext;
                    pRootSubItem->pNext->pPrev = pRootSubItem->pPrev;
                }

                pRootSubItem->pNext = (T*)pCurRoot->pFirstSubItem;
                ((T*)pCurRoot->pFirstSubItem)->pPrev = pRootSubItem;
                pCurRoot->pFirstSubItem = pRootSubItem;
            }

            int iPos = GetRootInsertPos(pRootSubItem);

            ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE | LVIF_STATE, iPos, LPSTR_TEXTCALLBACK,
                INDEXTOSTATEIMAGEMASK(pCurRoot->iItemCount == 1 ? 0 : (pCurRoot->bExpanded == false ? 1 : 2)), LVIS_STATEIMAGEMASK, I_IMAGECALLBACK, (LPARAM)pRootSubItem);

            if(pCurRoot->bExpanded == true && pCurRoot->iItemCount != 1) {
                InsertSubItems(pCurRoot, iPos);
            }
        }

        ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
    }

    static int CALLBACK CompareFunction(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort) {
        ThisTreeListViewCtrl * pTLVC = (ThisTreeListViewCtrl *)lParamSort;

        int iResult = T::CompareItems((T*)lParam1, (T*)lParam2, pTLVC->iSortColumn);

        return (pTLVC->bSortAscending == true ? iResult : -iResult);
    }

	bool onChar() {
        if((::GetKeyState(LOBYTE(::VkKeyScan('A'))) & 0x8000) != 0 && WinUtil::isCtrl() == true) {
            for(int i = 0; i < (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
                ListView_SetItemState(m_hWnd, i, LVIS_SELECTED, LVIS_SELECTED);
            }

            return true;
        }

		return false;
	}

	bool OnLButtonDown(LPARAM lParam) {
        LVHITTESTINFO lvHitTestInfo = { 0 };
        lvHitTestInfo.pt.x = GET_X_LPARAM(lParam);
        lvHitTestInfo.pt.y = GET_Y_LPARAM(lParam);

        int iPos = (int)::SendMessage(m_hWnd, LVM_SUBITEMHITTEST, 0, (LPARAM)&lvHitTestInfo);
        if(iPos == -1) {
            return false;
        }

        T * pItem = GetItemData(iPos);
        if(pItem->pRoot->pFirstSubItem != pItem || pItem->pRoot->iItemCount == 1) {
            return false;
        }

        RECT rc = { LVIR_ICON, 0, 0, 0 };

        if(::SendMessage(m_hWnd, LVM_GETITEMRECT, iPos, (LPARAM)&rc) == TRUE && lvHitTestInfo.pt.x < rc.left) {
            ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

            if(pItem->pRoot->bExpanded == false) {
                pItem->pRoot->bExpanded = true;
                ListView_SetItemState(m_hWnd, iPos, INDEXTOSTATEIMAGEMASK(2), LVIS_STATEIMAGEMASK);
                InsertSubItems(pItem->pRoot, iPos);
            } else {
                pItem->pRoot->bExpanded = false;
                ListView_SetItemState(m_hWnd, iPos, INDEXTOSTATEIMAGEMASK(1), LVIS_STATEIMAGEMASK);
                DeleteSubItems(pItem->pRoot, iPos);
            }

            ::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);

            return true;
        }

        return false;
    }

    void InsertSubItems(RootItem * pRoot, const int &iRootPos) {
        T * pNextSubItem = ((T*)pRoot->pFirstSubItem)->pNext,
            * pCurSubItem = NULL;

        int iItemCount = 1;

        while(pNextSubItem != NULL) {
            pCurSubItem = pNextSubItem;
            pNextSubItem = pCurSubItem->pNext;

            ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE | LVIF_INDENT, GetSubItemInsertPos(pCurSubItem, iRootPos, iItemCount), LPSTR_TEXTCALLBACK, 0, 0,
                I_IMAGECALLBACK, (LPARAM)pCurSubItem);

            iItemCount++;
        }
    }

    void DeleteSubItems(RootItem * pRoot, const int &iRootPos) {
        for(int i = iRootPos+1; i < iRootPos + pRoot->iItemCount; i++) {
            ::SendMessage(m_hWnd, LVM_DELETEITEM, iRootPos+1, 0);
        }
    }
};

//----------------------------------------------------------------------------------------------------------------------------------------------------
#endif
