/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef TYPED_LIST_VIEW_CTRL_H
#define TYPED_LIST_VIEW_CTRL_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../Client/SettingsManager.h"
#include "../client/FavoriteManager.h"
#include "WinApiUtil.h"
#include "memdc.h"

class ColumnInfo {
public:
	ColumnInfo(const tstring &aName, int aPos, int aFormat, int aWidth): name(aName), pos(aPos), width(aWidth), 
		format(aFormat), visible(true) {}
	~ColumnInfo() {}
	
	tstring name;
	bool visible;
	int pos;
	int width;
	int format;
};

template<class T, int ctrlId>
class TypedListViewCtrl {
public:
	TypedListViewCtrl() : sortColumn(-1), sortAscending(true) { }
	~TypedListViewCtrl() {
        for_each(columnList.begin(), columnList.end(), DeleteFunction());
    }

	typedef TypedListViewCtrl<T, ctrlId> thisClass;

    static LRESULT CALLBACK StaticTypedListViewProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
        thisClass * pParent = (thisClass *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
        if(pParent != NULL) {
            return pParent->TypedListViewProc(hWnd, uMsg, wParam, lParam);
        }

        return ::CallWindowProc(OldListViewProc, hWnd, uMsg, wParam, lParam);
    }

    LRESULT TypedListViewProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
        switch(uMsg) {
            case WM_MENUCOMMAND:
                onHeaderMenu(wParam);
                break;
            case WM_CHAR:
                if(onChar() == true) {
                    return 0;
                }

                break;
            case WM_ERASEBKGND:
                return 0;
            case WM_PAINT:
                return onPaint(uMsg);
            case WM_CONTEXTMENU:
                if(onContextMenu(lParam) == true) {
                    return 0;
                }

                break;
        }

        return ::CallWindowProc(OldListViewProc, hWnd, uMsg, wParam, lParam);
    }

	class iterator : private ::iterator<random_access_iterator_tag, T*> {
	public:
		iterator() : typedList(NULL), cur(0), cnt(0) { }
		iterator(const iterator& rhs) : typedList(rhs.typedList), cur(rhs.cur), cnt(rhs.cnt) { }
		iterator& operator=(const iterator& rhs) { typedList = rhs.typedList; cur = rhs.cur; cnt = rhs.cnt; return *this; }

		bool operator==(const iterator& rhs) const { return cur == rhs.cur; }
		bool operator!=(const iterator& rhs) const { return !(*this == rhs); }
		bool operator<(const iterator& rhs) const { return cur < rhs.cur; }

		int operator-(const iterator& rhs) const {
			return cur - rhs.cur;
		}

		iterator& operator+=(int n) { cur += n; return *this; }
		iterator& operator-=(int n) { return (cur += -n); }

		T& operator*() { return *typedList->getItemData(cur); }
		T* operator->() { return &(*(*this)); }
		T& operator[](int n) { return *typedList->getItemData(cur + n); }

		iterator operator++(int) {
			iterator tmp(*this);
			operator++();
			return tmp;
		}
		iterator& operator++() {
			++cur;
			return *this;
		}

	private:
		iterator(thisClass* aTypedList) : typedList(aTypedList), cur(aTypedList->GetNextItem(-1, LVNI_ALL)), cnt(aTypedList->GetItemCount()) {
			if(cur == -1)
				cur = cnt;
		}
		iterator(thisClass* aTypedList, int first) : typedList(aTypedList), cur(first), cnt(aTypedList->GetItemCount()) {
			if(cur == -1)
				cur = cnt;
		}
		friend class thisClass;
		thisClass* typedList;
		int cur;
		int cnt;
	};

	bool onChar() {
		if((GetKeyState(VkKeyScan('A') & 0xFF) & 0xFF00) > 0 && (GetKeyState(VK_CONTROL) & 0xFF00) > 0){
			int count = (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0);
			for(int i = 0; i < count; ++i)
				ListView_SetItemState(m_hWnd, i, LVIS_SELECTED, LVIS_SELECTED);

			return true;
		}

		return false;
	}

	void onGetDispInfo(LPNMHDR pnmh) {
		NMLVDISPINFO* di = (NMLVDISPINFO*)pnmh;
		if(di->item.mask & LVIF_TEXT) {
			di->item.mask |= LVIF_DI_SETITEM;
			di->item.pszText = (TCHAR*)((T*)di->item.lParam)->getText(columnIndexes.size() != 0 ? columnIndexes[di->item.iSubItem] : di->item.iSubItem, di->item.pszText);
		}
	}

	void onInfoTip(LPNMHDR pnmh) {
        if(BOOLSETTING(SHOW_INFOTIPS) == false) {
            return;
        }

		NMLVGETINFOTIP* pInfoTip = (NMLVGETINFOTIP*) pnmh;
		BOOL NoColumnHeader = (BOOL)(::GetWindowLong(m_hWnd, GWL_STYLE) & LVS_NOCOLUMNHEADER);
		tstring InfoTip(Util::emptyStringT);
		TCHAR Buffer[258];
		int indexes[32];
		int ihdritmcnt = (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0);
		::SendMessage(m_hWnd, LVM_GETCOLUMNORDERARRAY, ihdritmcnt, (LPARAM)indexes);
		for(int i = 0; i < ihdritmcnt; i++) {
			if(NoColumnHeader == false) {
				LV_COLUMN lvCol;
				lvCol.mask = LVCF_TEXT;
				lvCol.pszText = Buffer;
				lvCol.cchTextMax = 256;
				::SendMessage(m_hWnd, LVM_GETCOLUMN, indexes[i], (LPARAM)&lvCol);
				_tcscat(Buffer, _T(": "));
				InfoTip += Buffer;
			}

			ListViewGetItemText(m_hWnd, pInfoTip->iItem, indexes[i], Buffer, 256);
			Buffer[255] = _T('\0');
			_tcscat(Buffer, _T("\r\n"));
			InfoTip += Buffer;
		}

		if(InfoTip.size() > 2)
			InfoTip.erase(InfoTip.size() - 2);
	
		pInfoTip->cchTextMax = (int)InfoTip.size();

 		_tcsncpy(pInfoTip->pszText, InfoTip.c_str(), INFOTIPSIZE);
		pInfoTip->pszText[INFOTIPSIZE - 1] = _T('\0');
	}

	// Sorting
	void onColumnClick(LPNMHDR pnmh) {
		NMLISTVIEW* l = (NMLISTVIEW*)pnmh;
		if(l->iSubItem != sortColumn) {
			sortAscending = true;
			sortColumn = l->iSubItem;
		} else if(sortAscending) {
			sortAscending = false;
		} else {
			sortColumn = -1;
		}
		ListViewUpdateArrow(m_hWnd, sortAscending, sortColumn);
		resort();
	}

	void resort() {
		if(sortColumn != -1) {
			::SendMessage(m_hWnd, LVM_SORTITEMS, (WPARAM)this, (LPARAM)&compareFunc);
		}
	}

	int insertItem(T* item, int image) {
		return insertItem(getSortPos(item), item, image);
	}
	int insertItem(int i, T* item, int image) {
		return ListViewInsertItem(m_hWnd, LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE, i, LPSTR_TEXTCALLBACK, 0, 0, image, (LPARAM)item);
	}
	T* getItemData(int iItem) { return (T*)ListViewGetItem(m_hWnd, iItem); }
	T* getSelectedItem() { return ((UINT)::SendMessage(m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) > 0 ?
			getItemData((int)::SendMessage(m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) : NULL); }

	int findItem(T* item) {
		LVFINDINFO fi = { LVFI_PARAM, NULL, (LPARAM)item };
		return (int)::SendMessage(m_hWnd, LVM_FINDITEM, (WPARAM)-1, (LPARAM)&fi);
	}

	int findItem(const tstring& b, int start = -1, bool aPartial = false) {
		LVFINDINFO fi = { aPartial ? LVFI_PARTIAL : LVFI_STRING, b.c_str() };
		return (int)::SendMessage(m_hWnd, LVM_FINDITEM, start, (LPARAM)&fi);
	}

	void forEach(void (T::*func)()) {
		int n = (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0);
		for(int i = 0; i < n; ++i)
			(getItemData(i)->*func)();
	}
	void forEachSelected(void (T::*func)()) {
		int i = -1;
		while( (i = (int)::SendMessage(m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1)
			(getItemData(i)->*func)();
	}
	template<class _Function>
	_Function forEachT(_Function pred) {
		int n = (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0);
		for(int i = 0; i < n; ++i)
			pred(getItemData(i));
		return pred;
	}
	template<class _Function>
	_Function forEachSelectedT(_Function pred) {
		int i = -1;
		while( (i = (int)::SendMessage(m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1)
			pred(getItemData(i));
		return pred;
	}
	void forEachAtPos(int iIndex, void (T::*func)()) {
		(getItemData(iIndex)->*func)();
	}

	void updateItem(int i) {
		int k = (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0);
		for(int j = 0; j < k; ++j) {
			ListViewSetItem(m_hWnd, i, j, LVIF_TEXT, LPSTR_TEXTCALLBACK, 0);
		}
	}
	void updateItem(T* item) { int i = findItem(item); if(i != -1) updateItem(i); }
	void deleteItem(T* item) { int i = findItem(item); if(i != -1) ::SendMessage(m_hWnd, LVM_DELETEITEM, i, 0); }

	int getSortPos(T* a) {
		int high = (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0);
		if((sortColumn == -1) || (high == 0))
			return high;

		high--;

		int low = 0;
		int mid = 0;
		T* b = NULL;
		int comp = 0;
		while( low <= high ) {
			mid = (low + high) / 2;
			b = getItemData(mid);
			comp = T::compareItems(a, b, sortColumn);

			if(!sortAscending)
				comp = -comp;

			if(comp == 0) {
				return mid;
			} else if(comp < 0) {
				high = mid - 1;
			} else if(comp > 0) {
				low = mid + 1;
			}
		}

		comp = T::compareItems(a, b, sortColumn);
		if(!sortAscending)
			comp = -comp;
		if(comp > 0)
			mid++;

		return mid;
	}

	void setSortColumn(int aSortColumn) {
		sortColumn = aSortColumn;
		ListViewUpdateArrow(m_hWnd, sortAscending, sortColumn);
	}
	int getSortColumn() const { return sortColumn; }
	int getRealSortColumn() { return findColumn(sortColumn); }
	bool isAscending() const { return sortAscending; }
	void setAscending(bool s) {
		sortAscending = s;
		ListViewUpdateArrow(m_hWnd, sortAscending, sortColumn);
	}

	iterator begin() { return iterator(this); }
	iterator end() { return iterator(this, (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0)); }

	int InsertColumn(int nCol, const tstring &columnHeading, int nFormat = LVCFMT_LEFT, int nWidth = -1, int nSubItem = -1) {
		if(nWidth == 0) nWidth = 80;
		columnList.push_back(new ColumnInfo(columnHeading, nCol, nFormat, nWidth));
		columnIndexes.push_back(nCol);
		return ListViewInsertColumn(m_hWnd, nCol, columnHeading.c_str(), nFormat, nWidth, nSubItem);
	}

	void showMenu(POINT &pt){
		HMENU headerMenu = ::CreatePopupMenu();

		MENUINFO inf;
		inf.cbSize = sizeof(MENUINFO);
		inf.fMask = MIM_STYLE;
		inf.dwStyle = MNS_NOTIFYBYPOS;
		::SetMenuInfo(headerMenu, &inf);

		int j = 0;
		for(ColumnIter i = columnList.begin(); i != columnList.end(); ++i, ++j) {
            ::AppendMenu(headerMenu, MF_STRING, IDC_HEADER_MENU, (*i)->name.c_str());
			if((*i)->visible){
                ::CheckMenuItem(headerMenu, j, MF_BYPOSITION | MF_CHECKED);
            }
		}
		::TrackPopupMenuEx(headerMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, m_hWnd, NULL);

		::DestroyMenu(headerMenu);
	}

	LRESULT onPaint(UINT msg) {
		RECT updateRect;
		RECT crc;
		::GetClientRect(m_hWnd, &crc);
		if(::GetUpdateRect(m_hWnd, &updateRect, FALSE)) {
            PAINTSTRUCT m_ps;
            HDC dc = ::BeginPaint(m_hWnd, &m_ps);
			int oldDCSettings = ::SaveDC(dc);
			::SetBkColor(dc, WinUtil::bgColor);
			CMemDC memDC(dc, &crc);

			//it seems like the default paint method actually
			//uses the HDC it's passed, saves a lot of work =)
			LRESULT ret = ::CallWindowProc(OldListViewProc, m_hWnd, msg, (WPARAM)memDC.m_hDC, NULL);

			//make sure to paint before HDC goes out of scope
			memDC.Paint();

			::RestoreDC(dc, oldDCSettings);

            ::EndPaint(m_hWnd, &m_ps);

			return ret;
		}

		return 0;
	}

	bool onContextMenu(LPARAM lParam) {
		POINT pt = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };
		//make sure we're not handling the context menu if it's invoked from the
		//keyboard
		if(pt.x == -1 && pt.y == -1) {
			return false;
		}

		RECT rc;
		::GetWindowRect((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), &rc);

		if(::PtInRect(&rc, pt)) {
			showMenu(pt);
			return true;
		}

		return false;
	}

	void onHeaderMenu(WPARAM wParam) {
		ColumnInfo * ci = columnList[wParam];
		ci->visible = ! ci->visible;

		::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

		if(!ci->visible){
			removeColumn(ci);
		} else {
			if(ci->width == 0) ci->width = 80;
			ListViewInsertColumn(m_hWnd, ci->pos, ci->name.c_str(), ci->format, ci->width, (int)wParam);
			LVCOLUMN lvcl = { 0 };
			lvcl.mask = LVCF_ORDER;
			lvcl.iOrder = ci->pos;
			::SendMessage(m_hWnd, LVM_SETCOLUMN, ci->pos, (LPARAM)&lvcl);
			for(int i = 0; i < (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0); i++) {
				LVITEM lvItem;
				lvItem.iItem = i;
				lvItem.iSubItem = 0;
				lvItem.mask = LVIF_IMAGE | LVIF_PARAM;
				::SendMessage(m_hWnd, LVM_GETITEM, 0, (LPARAM)&lvItem);
				lvItem.iImage = ((T*)lvItem.lParam)->imageIndex();
				::SendMessage(m_hWnd, LVM_SETITEM, 0, (LPARAM)&lvItem);
				updateItem(i);
			}
		}

		updateColumnIndexes();

		::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
		::InvalidateRect(m_hWnd, NULL, TRUE);
		::UpdateWindow(m_hWnd);
	}

	void saveHeaderOrder(SettingsManager::StrSetting order, SettingsManager::StrSetting widths,
		SettingsManager::StrSetting visible) {
		string tmp, tmp2, tmp3;
		saveHeaderOrder(tmp, tmp2, tmp3);
		SettingsManager::getInstance()->set(order, tmp);
		SettingsManager::getInstance()->set(widths, tmp2);
		SettingsManager::getInstance()->set(visible, tmp3);
	}

	void saveFavoriteHeaderOrder(FavoriteHubEntry* Entry) throw() {
		string tmp, tmp2, tmp3;
		saveHeaderOrder(tmp, tmp2, tmp3);
		Entry->setColumsOrder(tmp);
		Entry->setColumsWidth(tmp2);
		Entry->setColumsVisible(tmp3);
		FavoriteManager::getInstance()->save();
	}

	void saveHeaderOrder(string& order, string& widths, string& visible) throw() {
		TCHAR buf[512];
		int size = (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0);
		for(int i = 0; i < size; ++i){
			LVCOLUMN lvc;
			lvc.cchTextMax = 512;
			lvc.mask = LVCF_TEXT | LVCF_ORDER | LVCF_WIDTH;
			lvc.pszText = buf;
			::SendMessage(m_hWnd, LVM_GETCOLUMN, i, (LPARAM)&lvc);
			for(ColumnIter j = columnList.begin(); j != columnList.end(); ++j){
				if(_tcscmp(lvc.pszText, (*j)->name.c_str()) == 0){
					(*j)->pos = lvc.iOrder;
					(*j)->width = lvc.cx;
					break;
				}
			}
		}

		for(ColumnIter i = columnList.begin(); i != columnList.end(); ++i){
			ColumnInfo* ci = *i;

			if(ci->visible){
				visible += "1,";
			} else {
				ci->pos = size++;
				visible += "0,";
			}

			order += Util::toString(ci->pos);
			order += ',';

			widths += Util::toString(ci->width);
			widths += ',';
		}

		order.erase(order.size()-1, 1);
		widths.erase(widths.size()-1, 1);
		visible.erase(visible.size()-1, 1);
	}

	void setVisible(string vis){
		StringTokenizer<string> tok(vis, ',');
		StringList l = tok.getTokens();

		StringIter i = l.begin();
		for(ColumnIter j = columnList.begin(); j != columnList.end() && i != l.end(); ++i, ++j) {

			if(Util::toInt(*i) == 0){
				(*j)->visible = false;
				removeColumn(*j);
			}
		}

		updateColumnIndexes();
	}

	void setColumnOrderArray(int iCount, LPINT piArray ) {
		LVCOLUMN lvc;
		lvc.mask = LVCF_ORDER;
		for(int i = 0; i < iCount; ++i) {
			lvc.iOrder = columnList[i]->pos = piArray[i];
			::SendMessage(m_hWnd, LVM_SETCOLUMN, i, (LPARAM)&lvc);
		}
	}

	//find the current position for the column that was inserted at the specified pos
	int findColumn(int col) const {
		return columnIndexes[col];
	}

    HWND m_hWnd;

    void Create(HWND hWndParent, DWORD dwStyle, DWORD dwExStyle, UINT uiID, DWORD dwExListViewStyle, const int & iX, const int & iY, const int & iWidth, const int & iHeight,
        bool bSetWndProc = true) {
        m_hWnd = ::CreateWindowEx(dwExStyle, WC_LISTVIEW, _T(""), dwStyle, iX, iY, iWidth, iHeight, hWndParent, (HMENU)uiID, g_hInstance, NULL);

		::SendMessage(m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, dwExListViewStyle);
        ::SendMessage(m_hWnd, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
        ::SendMessage(m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
        ::SendMessage(m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
        ::SendMessage(m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);

        if(bSetWndProc == true) {
            ::SetWindowLongPtr(m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
            OldListViewProc = (WNDPROC)::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)StaticTypedListViewProc);
        } else {
            OldListViewProc = (WNDPROC)::GetWindowLongPtr(m_hWnd, GWLP_WNDPROC);
        }
    }
private:
	int sortColumn;

	bool sortAscending;

	static int CALLBACK compareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort) {
		thisClass* t = (thisClass*)lParamSort;
		int result = T::compareItems((T*)lParam1, (T*)lParam2, t->getRealSortColumn());
		return (t->sortAscending ? result : -result);
	}

	typedef vector< ColumnInfo* > ColumnList;
	typedef ColumnList::iterator ColumnIter;

	ColumnList columnList;
	vector<int> columnIndexes;

	void removeColumn(ColumnInfo* ci) {
		TCHAR buf[512];
		LVCOLUMN lvcl = { 0 };
		lvcl.cchTextMax = 512;
		lvcl.mask = LVCF_TEXT | LVCF_ORDER | LVCF_WIDTH;
		lvcl.pszText = buf;

        int ihdritmcnt = (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0);
		for(int k = 0; k < ihdritmcnt; ++k) {
			::SendMessage(m_hWnd, LVM_GETCOLUMN, k, (LPARAM)&lvcl);
			if(_tcscmp(ci->name.c_str(), lvcl.pszText) == 0) {
				ci->width = lvcl.cx;
				ci->pos = lvcl.iOrder;

	    		if(ihdritmcnt >= 0 && sortColumn > ihdritmcnt - 2) {
	    			setSortColumn(0);
                }

	    		if(sortColumn == ci->pos)
	    			setSortColumn(0);

					::SendMessage(m_hWnd, LVM_DELETECOLUMN, k, 0);

					for(int i = 0; i < (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
    	    			LVITEM lvItem;
    	    			lvItem.iItem = i;
    	    			lvItem.iSubItem = 0;
    	    			lvItem.mask = LVIF_PARAM | LVIF_IMAGE;
    					::SendMessage(m_hWnd, LVM_GETITEM, 0, (LPARAM)&lvItem);
    	    			lvItem.iImage = ((T*)lvItem.lParam)->imageIndex();
    					::SendMessage(m_hWnd, LVM_SETITEM, 0, (LPARAM)&lvItem);
    	    		}
					break;
			}
		}
	}

	void updateColumnIndexes() {
		columnIndexes.clear();

		int columns = (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0);

		columnIndexes.reserve(columns);

		TCHAR buf[128];
		LVCOLUMN lvcl;

		for(int i = 0; i < columns; ++i) {
			lvcl.cchTextMax = 128;
			lvcl.mask = LVCF_TEXT;
			lvcl.pszText = buf;
			::SendMessage(m_hWnd, LVM_GETCOLUMN, i, (LPARAM)&lvcl);
			for(size_t j = 0; j < columnList.size(); ++j) {
				if(Util::stricmp(columnList[j]->name.c_str(), lvcl.pszText) == 0) {
					columnIndexes.push_back(static_cast<int>(j));
					break;
				}
			}
		}
	}
};

#endif // TYPED_LIST_VIEW_CTRL_H

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
