/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "UPnP.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "MainFrm.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/ConnectionManager.h"
#include "../client/LogManager.h"
#include "../client/SearchManager.h"
#include "../client/Text.h"
#include "../client/Util.h"
#include "../client/version.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
UPnP * pUPnP = NULL;
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

UPnP::UPnP() {
    sExternalIP = Util::emptyString;

    pUPnPNAT = NULL;

	lSearchPort = 0;
    lTransferPort = 0;

    ui8ActualAction = ACTUAL_ACTION_NONE;
    ui8GetSPMCCount = 0;

    bPortsOpened = false;

    ::CoCreateInstance(__uuidof(UPnPNAT), NULL, CLSCTX_INPROC_SERVER, __uuidof(IUPnPNAT), (void**)&pUPnPNAT);
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

UPnP::~UPnP() {
	if(pUPnPNAT != NULL) {
		pUPnPNAT->Release();
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void UPnP::OnTimer() {
    if(ui8ActualAction == ACTUAL_ACTION_OPEN_PORTS) {
        switch(Open()) {
            case 1: {
                size_t szsize = STRING(UPNP_CREATED_PORT_MAPPINGS).size() + 16;
                AutoArray<char> tmp(szsize);
                sprintf_s(tmp, szsize, CSTRING(UPNP_CREATED_PORT_MAPPINGS), lTransferPort, lSearchPort);
                LogManager::getInstance()->message(string(tmp));

                if(BOOLSETTING(NO_IP_OVERRIDE) == false) {
                    if(sExternalIP.empty() == false) {
                        SettingsManager::getInstance()->set(SettingsManager::EXTERNAL_IP, pUPnP->sExternalIP);
                    } else {
                        LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_GET_EXTERNAL_IP));
                    }
                }

                break;
            }
            case 2:
                //timer
                return;
            default:
                LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_CREATE_MAPPINGS));
                break;
        }

    } else if(ui8ActualAction == ACTUAL_ACTION_CLOSE_PORTS) {
        switch(Close()) {
            case 1:
                //success
                break;
            case 2:
                //timer
                return;
            default:
                LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_REMOVE_MAPPINGS));
                break;
        }
    }

    ui8ActualAction = ACTUAL_ACTION_NONE;

    return;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void UPnP::OpenPorts() {
    if(BOOLSETTING(SETTINGS_USE_UPNP) == false || SETTING(CONNECTION_TYPE) != SettingsManager::CONNECTION_ACTIVE) {
        return;
    }

    if(pUPnPNAT == NULL) {
        LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_CREATE_MAPPINGS));
        return;
    }

    ui8ActualAction = ACTUAL_ACTION_OPEN_PORTS;
    ui8GetSPMCCount = 0;

    lTransferPort = ConnectionManager::getInstance()->getPort();
    lSearchPort = SearchManager::getInstance()->getPort();

    OnTimer();
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

uint8_t UPnP::Open() {
    bool bTimer = false;

    IStaticPortMappingCollection * pSPMC = GetSPMC(bTimer);

	if(pSPMC == NULL) {
        if(bTimer == false) {
            return 0;
        }

        return 2;
    }

    BSTR bstrProtocol = ::SysAllocString(_T("TCP"));
    BSTR bstrInternalClient = ::SysAllocString(Text::toT(Util::getLocalIp()).c_str());
    BSTR bstrDescription = ::SysAllocString(Text::toT(CzDC_VERSION_STRING " transfers port").c_str());

    if(bstrProtocol == NULL || bstrInternalClient == NULL || bstrDescription == NULL) {
        ::SysFreeString(bstrProtocol);
        ::SysFreeString(bstrInternalClient);
        ::SysFreeString(bstrDescription);

        pSPMC->Release();

        return 0;
    }

    IStaticPortMapping * pSPM = NULL;

	if(pSPMC->Add(lTransferPort, bstrProtocol, lTransferPort, bstrInternalClient, VARIANT_TRUE, bstrDescription, &pSPM) != S_OK) {
        ::SysFreeString(bstrProtocol);
        ::SysFreeString(bstrInternalClient);
        ::SysFreeString(bstrDescription);

        pSPMC->Release();

        return 0;
	}

    if(pSPM != NULL) {
        pSPM->Release();
        pSPM = NULL;
    }

    ::SysFreeString(bstrProtocol);
    ::SysFreeString(bstrDescription);

    bstrProtocol = ::SysAllocString(_T("UDP"));
    bstrDescription = ::SysAllocString(Text::toT(CzDC_VERSION_STRING " searching port").c_str());

    if(bstrProtocol == NULL || bstrDescription == NULL) {
        ::SysFreeString(bstrProtocol);
        ::SysFreeString(bstrInternalClient);
        ::SysFreeString(bstrDescription);

        pSPMC->Release();

        return 0;
    }

	if(pSPMC->Add(lSearchPort, bstrProtocol, lSearchPort, bstrInternalClient, VARIANT_TRUE, bstrDescription, &pSPM) != S_OK) {
        ::SysFreeString(bstrProtocol);
        ::SysFreeString(bstrInternalClient);
        ::SysFreeString(bstrDescription);

        pSPMC->Release();

        return 0;
	}

    if(pSPM != NULL) {
        BSTR bstrExternalIPAddress = NULL;

        if(pSPM->get_ExternalIPAddress(&bstrExternalIPAddress) == S_OK) {
            if(bstrExternalIPAddress != NULL) {
                sExternalIP = Text::wideToAcp(bstrExternalIPAddress);
                ::SysFreeString(bstrExternalIPAddress);
            }
        }

        pSPM->Release();
    }

    ::SysFreeString(bstrProtocol);
    ::SysFreeString(bstrInternalClient);
    ::SysFreeString(bstrDescription);

    pSPMC->Release();

    bPortsOpened = true;

    return 1;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void UPnP::ClosePorts() {
	if(bPortsOpened == false) {
		return;
	}

    ui8ActualAction = ACTUAL_ACTION_CLOSE_PORTS;
    ui8GetSPMCCount = 0;

	OnTimer();
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

uint8_t UPnP::Close() {
    bool bTimer = false;

    IStaticPortMappingCollection * pSPMC = GetSPMC(bTimer);

	if(pSPMC == NULL) {
        if(bTimer == false) {
            return 0;
        }

        return 2;
    }

    BSTR bstrProtocol = ::SysAllocString(_T("TCP"));

	if(pSPMC->Remove(lTransferPort, bstrProtocol) != S_OK) {
        ::SysFreeString(bstrProtocol);

        pSPMC->Release();

        return 0;
    }

    ::SysFreeString(bstrProtocol);

    bstrProtocol = ::SysAllocString(_T("UDP"));

	if(pSPMC->Remove(lSearchPort, bstrProtocol) != S_OK) {
        ::SysFreeString(bstrProtocol);

        pSPMC->Release();

        return 0;
    }

    pSPMC->Release();

	lSearchPort = 0;
    lTransferPort = 0;

    bPortsOpened = false;

	return 1;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

IStaticPortMappingCollection * UPnP::GetSPMC(bool &bTimer) {
    IStaticPortMappingCollection * pSPMC = NULL;

	if(ui8GetSPMCCount < 10) {
        ui8GetSPMCCount++;

		if(pUPnPNAT->get_StaticPortMappingCollection(&pSPMC) == S_OK && pSPMC != NULL) {
            return pSPMC;
        }

        UINT_PTR uiptrret = ::SetTimer(MainFrame::pMainFrame->m_hWnd, 1, 1000, NULL);
        if(uiptrret == 0) {
            return NULL;
        }

        bTimer = true;
        return NULL;
	}

	return pSPMC;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
