/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "Resource.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/Client.h"
#include "../client/ClientManager.h"
#include "../client/FavoriteManager.h"
#include "../client/QueueManager.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "WaitingUsersFrame.h"
#include "PrivateFrame.h"
#include "FlatTabCtrl.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

int WaitingUsersFrame::columnSizes[] = { 250, 100, 75, 75, 75, 75, 100, 100 };
int WaitingUsersFrame::columnIndexes[] = { COLUMN_FILE, COLUMN_PATH, COLUMN_NICK, COLUMN_HUB, COLUMN_TRANSFERRED, COLUMN_SIZE, COLUMN_ADDED, COLUMN_WAITING };
static ResourceManager::Strings columnNames[] = { ResourceManager::FILENAME, ResourceManager::PATH, ResourceManager::NICK, ResourceManager::HUB, ResourceManager::TRANSFERRED,
    ResourceManager::SIZE, ResourceManager::ADDED, ResourceManager::WAITING_TIME
};
//----------------------------------------------------------------------------------------------------------------------------------------------------

ATOM WaitingUsersFrame::m_aAtom = 0;
UINT WaitingUsersFrame::m_uiResourceID = IDR_UPLOAD_QUEUE;
//----------------------------------------------------------------------------------------------------------------------------------------------------

WaitingUsersFrame::WaitingUsersFrame() : showTree(true), closed(false), usingUserMenu(false), contextMenu(NULL) {
    RegisterWindowClass(m_aAtom, WaitingUsersFrameWndProc, _T("WaitingUsersFrame"), m_uiResourceID);
    memset(statusSizes, 0, sizeof(statusSizes));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

WaitingUsersFrame::~WaitingUsersFrame() {
    if(contextMenu != NULL) {
        ::DestroyMenu(contextMenu);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT CALLBACK WaitingUsersFrame::WaitingUsersFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	WaitingUsersFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (WaitingUsersFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (WaitingUsersFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->WaitingUsersFrameProc(hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT WaitingUsersFrame::WaitingUsersFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SPEAKER:
            onSpeaker(wParam, lParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            if(HIWORD(wParam) == BN_CLICKED) {
                switch(LOWORD(wParam)) {
                    case IDC_GETLIST:
                        onGetList();
                        return 0;
                    case IDC_REMOVE:
                        onRemove();
                        return 0;
                    case IDC_GRANTSLOT:
                        onGrantSlot();
                        return 0;
                    case IDC_GRANTSLOT_HOUR:
                        onGrantSlotHour();
                        return 0;
                    case IDC_GRANTSLOT_DAY:
                        onGrantSlotDay();
                        return 0;
                    case IDC_GRANTSLOT_WEEK:
                        onGrantSlotWeek();
                        return 0;
                    case IDC_UNGRANTSLOT:
                        onUnGrantSlot();
                        return 0;
                    case IDC_ADD_TO_FAVORITES:
                        onAddToFavorites();
                        return 0;
                    case IDC_PRIVATEMESSAGE:
                        onPrivateMessage();
                        return 0;
                }
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_UPLOAD_QUEUE) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_GETDISPINFO: {
                        ctrlList.onGetDispInfo((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_COLUMNCLICK: {
                        ctrlList.onColumnClick((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_KEYDOWN:
                        if(onKeyDown((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                }
            } else if(((LPNMHDR)lParam)->idFrom == IDC_DIRECTORIES) {
                if(((LPNMHDR)lParam)->code == TVN_SELCHANGED) {
                    onItemChanged();
                    return 0;
                } else if(((LPNMHDR)lParam)->code == TVN_KEYDOWN) {
                    if(onKeyDownDirs((LPNMHDR)lParam) == true) {
                        return 0;
                    }
                    break;
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
    if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
    	return lResult;
    }

    if(uMsg == WM_MOUSEACTIVATE) {
        lResult = ::DefMDIChildProc(m_hWnd, uMsg, wParam, lParam);
    }

    if(BasicSplitterProc(uMsg, wParam, lParam) == true) {
    	return lResult;
    }

    return ::DefMDIChildProc(m_hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT CALLBACK WaitingUsersFrame::ShowTreeWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
		WaitingUsersFrame * pParent = (WaitingUsersFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->onShowTree(wParam);
        }
    }

    return ::CallWindowProc(OldButtonProc, hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_UPLOAD_QUEUE);
    created = true;

	showTree = BOOLSETTING(UPLOADQUEUEFRAME_SHOW_TREE);

	// status bar
    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, 0, 0, 0, 0, m_hWnd, 0, g_hInstance, NULL);

	ctrlList.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE,
        IDC_UPLOAD_QUEUE, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT);

	hWndQueued = CreateWindowEx(WS_EX_CLIENTEDGE, WC_TREEVIEW, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | TVS_HASBUTTONS | TVS_LINESATROOT | TVS_HASLINES | TVS_SHOWSELALWAYS |
        TVS_DISABLEDRAGDROP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_DIRECTORIES, g_hInstance, NULL);

	::SendMessage(hWndQueued, TVM_SETIMAGELIST, (WPARAM)TVSIL_NORMAL, (LPARAM)WinUtil::fileImages);

	::SendMessage(ctrlList.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::fileImages);

	iProportionalPos = SETTING(WAITINGFRAME_SPLITTER);

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(UPLOADQUEUEFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(UPLOADQUEUEFRAME_WIDTHS), COLUMN_LAST);

	// column names, sizes
	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		int fmt = (ui8i == COLUMN_TRANSFERRED || ui8i == COLUMN_SIZE) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		ctrlList.InsertColumn(ui8i, CTSTRING_I(columnNames[ui8i]), fmt, columnSizes[ui8i], ui8i);
	}

	ctrlList.setColumnOrderArray(COLUMN_LAST, columnIndexes);
	ctrlList.setVisible(SETTING(UPLOADQUEUEFRAME_VISIBLE));

	// colors
	::SendMessage(hWndQueued, TVM_SETBKCOLOR, 0, (LPARAM)WinUtil::bgColor);
	::SendMessage(hWndQueued, TVM_SETTEXTCOLOR, 0, (LPARAM)WinUtil::textColor);

	hWndShowTree = CreateWindowEx(0, WC_BUTTON, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_AUTOCHECKBOX, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWndStatusBar, 0, g_hInstance, NULL);
	::SetWindowLongPtr(hWndShowTree, GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWndShowTree, GWLP_WNDPROC, (LONG_PTR)ShowTreeWndProc);
    ::SendMessage(hWndShowTree, BM_SETCHECK, (showTree == true ? BST_CHECKED : BST_UNCHECKED), 0);

	// Create context menu
	contextMenu = ::CreatePopupMenu();
	::AppendMenu(contextMenu, MF_STRING, IDC_GETLIST, CTSTRING(GET_FILE_LIST));

	WinUtil::AddGrantSubMenu(contextMenu);

	::AppendMenu(contextMenu, MF_STRING, IDC_PRIVATEMESSAGE, CTSTRING(SEND_PRIVATE_MESSAGE));
	::AppendMenu(contextMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));
	::AppendMenu(contextMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(contextMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));

	statusSizes[0] = 16;
	::SendMessage(m_hWndStatusBar, SB_SETPARTS, 4, (LPARAM)statusSizes);
	UpdateLayout();

	UploadManager::getInstance()->addListener(this);
	TimerManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);

	// Load all searches
	LoadAll();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool WaitingUsersFrame::onClose() {
	if(!closed) {
		UploadManager::getInstance()->removeListener(this);
		TimerManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		HTREEITEM userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_ROOT, 0);

		while(userNode) {
			delete (UserPtr *)TreeGetItemData(hWndQueued, userNode);
			userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)userNode);
		}
		::SendMessage(ctrlList.m_hWnd, LVM_DELETEALLITEMS, 0, 0);
		::SendMessage(hWndQueued, TVM_DELETEITEM, 0, (LPARAM)TVI_ROOT);
		UQFUsers.clear();
		SettingsManager::getInstance()->set(SettingsManager::UPLOADQUEUEFRAME_SHOW_TREE, (int)::SendMessage(hWndShowTree, BM_GETCHECK, 0, 0) == BST_CHECKED);
		SettingsManager::getInstance()->set(SettingsManager::WAITINGFRAME_SPLITTER, iProportionalPos);
	    ctrlList.saveHeaderOrder(SettingsManager::UPLOADQUEUEFRAME_ORDER, SettingsManager::UPLOADQUEUEFRAME_WIDTHS,
			SettingsManager::UPLOADQUEUEFRAME_VISIBLE);

		return false;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::UpdateLayout(bool bResizeStatusBar /* = true */) {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect, bResizeStatusBar);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[4];
		::GetClientRect(m_hWndStatusBar, &sr);
		w[3] = sr.right - 16;
#define setw(x) w[x] = max(w[x+1] - statusSizes[x], 0)
		setw(2); setw(1);

		w[0] = 16;

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 4, (LPARAM)w);

		::SendMessage(m_hWndStatusBar, SB_GETRECT, 0, (LPARAM)&sr);
		::MoveWindow(hWndShowTree, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
	}

	SetSplitterRect(&rect);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onGetList() {
	if(usingUserMenu == true) {
		UserPtr User = getSelectedUser();
		if(User) {
            try {
                QueueManager::getInstance()->addList(User, QueueItem::FLAG_CLIENT_VIEW);
			} catch(const Exception&) {
                // ...
			}
		}
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
            try {
                QueueManager::getInstance()->addList(((UploadQueueItem*)ctrlList.getItemData(i))->User, QueueItem::FLAG_CLIENT_VIEW);
			} catch(const Exception&) {
                // ...
			}
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onRemove() {
	if(usingUserMenu) {
		UserPtr User = getSelectedUser();
		if(User) {
			UploadManager::getInstance()->clearUserFiles(User);
		}
	} else {
		int i = -1;
		UserList RemoveUsers;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			// Ok let's cheat here, if you try to remove more users here is not working :(
			RemoveUsers.push_back(((UploadQueueItem*)ctrlList.getItemData(i))->User);
		}
		for(UserList::iterator i = RemoveUsers.begin(); i != RemoveUsers.end(); ++i) {
			UploadManager::getInstance()->clearUserFiles(*i);
		}
	}

	updateStatus();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool WaitingUsersFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);

    {
        RECT rc;
        ::GetWindowRect((HWND)::SendMessage(ctrlList.m_hWnd, LVM_GETHEADER, 0, 0), &rc);

        POINT pt = { iX, iY };

        if(::PtInRect(&rc, pt)){
            ctrlList.showMenu(pt);
            return true;
        }
    }

	if((HWND)wParam == ctrlList.m_hWnd && (UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) > 0) {
        usingUserMenu = false;

     	if(iX == -1 && iY == -1) {
            ListViewGetContextMenuPos(ctrlList.m_hWnd, iX, iY);
    	}

		::TrackPopupMenuEx(contextMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

		return true;
	} else if((HWND)wParam == hWndQueued && (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_CARET, 0) != NULL) {
        usingUserMenu = true;

        int iX = GET_X_LPARAM(lParam),
            iY = GET_Y_LPARAM(lParam);

     	TreeGetContextMenuPos(hWndQueued, iX, iY);

        ::TrackPopupMenuEx(contextMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

		return true;
	}

	return false;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onPrivateMessage() {
	if(usingUserMenu) {
		UserPtr User = getSelectedUser();
		if(User) {
			PrivateFrame::openWindow(User);
		}
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			PrivateFrame::openWindow(((UploadQueueItem*)ctrlList.getItemData(i))->User);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onGrantSlot() {
	if(usingUserMenu) {
		UserPtr User = getSelectedUser();
		if(User) {
			UploadManager::getInstance()->reserveSlot(User);
		}
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			UploadManager::getInstance()->reserveSlot(((UploadQueueItem*)ctrlList.getItemData(i))->User);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onGrantSlotHour() {
	if(usingUserMenu) {
		UserPtr User = getSelectedUser();
		if(User) {
			UploadManager::getInstance()->reserveSlotHour(User);
		}
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			UploadManager::getInstance()->reserveSlotHour(((UploadQueueItem*)ctrlList.getItemData(i))->User);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onGrantSlotDay() {
	if(usingUserMenu) {
		UserPtr User = getSelectedUser();
		if(User) {
			UploadManager::getInstance()->reserveSlotDay(User);
		}
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			UploadManager::getInstance()->reserveSlotDay(((UploadQueueItem*)ctrlList.getItemData(i))->User);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onGrantSlotWeek() {
	if(usingUserMenu) {
		UserPtr User = getSelectedUser();
		if(User) {
			UploadManager::getInstance()->reserveSlotWeek(User);
		}
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			UploadManager::getInstance()->reserveSlotWeek(((UploadQueueItem*)ctrlList.getItemData(i))->User);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onUnGrantSlot() {
	if(usingUserMenu) {
		UserPtr User = getSelectedUser();
		if(User) {
			UploadManager::getInstance()->unreserveSlot(User);
		}
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			UploadManager::getInstance()->unreserveSlot(((UploadQueueItem*)ctrlList.getItemData(i))->User);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onAddToFavorites() {
	if(usingUserMenu) {
		UserPtr User = getSelectedUser();
		if(User) {
			FavoriteManager::getInstance()->addFavoriteUser(User);
		}
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			FavoriteManager::getInstance()->addFavoriteUser(((UploadQueueItem*)ctrlList.getItemData(i))->User);
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::LoadAll() {
	::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	::SendMessage(hWndQueued, WM_SETREDRAW, (WPARAM)FALSE, 0);

	HTREEITEM userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_ROOT, 0);

	while(userNode) {
		delete (UserPtr *)TreeGetItemData(hWndQueued, userNode);
		userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)userNode);
	}

	::SendMessage(ctrlList.m_hWnd, LVM_DELETEALLITEMS, 0, 0);
	::SendMessage(hWndQueued, TVM_DELETEITEM, 0, (LPARAM)TVI_ROOT);
	UQFUsers.clear();

	// Load queue
	UploadQueueItem::UserMap users = UploadManager::getInstance()->getQueue();
	for(UploadQueueItem::UserMapIter uit = users.begin(); uit != users.end(); ++uit) {
		UQFUsers.push_back(uit->first);
		TreeInsertItem(hWndQueued, TVIF_PARAM | TVIF_TEXT, (WinUtil::getNicks(uit->first) + _T(" - ") + WinUtil::getHubNames(uit->first).first).c_str(),
            0, 0, 0, 0, (LPARAM)(new UserItem(uit->first)), TVI_ROOT, TVI_LAST);
		for(UploadQueueItem::Iter i = uit->second.begin(); i != uit->second.end(); ++i) {
			AddFile(*i);
		}
	}
	ctrlList.resort();
	::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	::SendMessage(hWndQueued, WM_SETREDRAW, (WPARAM)TRUE, 0);
	::InvalidateRect(hWndQueued, NULL, TRUE);
	updateStatus();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::RemoveUser(const UserPtr& aUser) {
	HTREEITEM userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_ROOT, 0);

	for(UserList::iterator i = UQFUsers.begin(); i != UQFUsers.end(); ++i) {
		if(*i == aUser) {
			UQFUsers.erase(i);
			break;
		}
	}

	while(userNode) {
		UserItem *u = (UserItem *)TreeGetItemData(hWndQueued, userNode);
		if (aUser == u->u) {
			delete u;
			::SendMessage(hWndQueued, TVM_DELETEITEM, 0, (LPARAM)userNode);
			return;
		}
		userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)userNode);
	}
	updateStatus();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onItemChanged() {
	HTREEITEM userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_CARET, 0);

	while(userNode) {
		::SendMessage(ctrlList.m_hWnd, LVM_DELETEALLITEMS, 0, 0);
		UserItem *u = (UserItem *)TreeGetItemData(hWndQueued, userNode);
		UploadQueueItem::UserMap users = UploadManager::getInstance()->getQueue();
		for (UploadQueueItem::UserMapIter uit = users.begin(); uit != users.end(); ++uit) {
			if(uit->first == u->u) {
				::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
				::SendMessage(hWndQueued, WM_SETREDRAW, (WPARAM)FALSE, 0);
				for(UploadQueueItem::Iter i = uit->second.begin(); i != uit->second.end(); ++i) {
					AddFile(*i);
				}
				ctrlList.resort();
				::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
				::SendMessage(hWndQueued, WM_SETREDRAW, (WPARAM)TRUE, 0);
				::InvalidateRect(hWndQueued, NULL, TRUE);
				updateStatus();
				return;
			}
		}
		userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)userNode);
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::AddFile(UploadQueueItem* aUQI) { 
	HTREEITEM userNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_ROOT, 0);
	bool add = true;

	HTREEITEM selNode = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_CARET, 0);

	if(userNode) {
		for(UserList::iterator i = UQFUsers.begin(); i != UQFUsers.end(); ++i) {
			if(*i == aUQI->User) {
				add = false;
				break;
			}
		}
	}
	if(add) {
			UQFUsers.push_back(aUQI->User);
			userNode = TreeInsertItem(hWndQueued, TVIF_PARAM | TVIF_TEXT, (WinUtil::getNicks(aUQI->User) + _T(" - ") +
                WinUtil::getHubNames(aUQI->User).first).c_str(), 0, 0, 0, 0, (LPARAM)(new UserItem(aUQI->User)), TVI_ROOT, TVI_LAST);
	}
	if(selNode) {
		TCHAR selBuf[256];
		TreeGetItemText(hWndQueued, selNode, selBuf, 255);
		if(_tcscmp(selBuf, (WinUtil::getNicks(aUQI->User) + _T(" - ") + WinUtil::getHubNames(aUQI->User).first).c_str()) != 0) {
			return;
		}
	}
	aUQI->update();
	ctrlList.insertItem((int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0), aUQI, aUQI->imageIndex());
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

HTREEITEM WaitingUsersFrame::GetParentItem() {
	HTREEITEM item = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_CARET, 0),
        parent = (HTREEITEM)::SendMessage(hWndQueued, TVM_GETNEXTITEM, TVGN_PARENT, (LPARAM)item);
    if(parent == NULL) {
        parent = item;
    }
	::SendMessage(hWndQueued, TVM_SELECTITEM, TVGN_CARET, (LPARAM)parent);
	return parent;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::updateStatus() {
	if(::IsWindow(m_hWndStatusBar)) {
		int cnt = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
		int users = (UINT)::SendMessage(hWndQueued, TVM_GETCOUNT, 0, 0);

		string tmp[2];
		if(showTree) {
			tmp[0] = STRING(USERS) + ": " + Util::toString(users);
		} else {
			tmp[0] = "";
		}    		  
		tmp[1] = STRING(ITEMS) + ": " + Util::toString(cnt);
		bool u = false;

		for(int i = 1; i < 3; i++) {
			int w = WinUtil::getTextWidth(Text::toT(tmp[i-1]), m_hWndStatusBar);
				
			if(statusSizes[i] < w) {
				statusSizes[i] = w + 50;
				u = true;
			}

			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (i+1 | 0), (LPARAM)Text::toT(tmp[i-1]).c_str());
		}

		if(u)
			UpdateLayout(true);
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void UploadQueueItem::update() {
	columns[COLUMN_FILE] = Text::toT(Util::getFileName(File));
	columns[COLUMN_PATH] = Text::toT(Util::getFilePath(File));
	columns[COLUMN_NICK] = WinUtil::getNicks(User);
	columns[COLUMN_HUB] = WinUtil::getHubNames(User).first;
	columns[COLUMN_TRANSFERRED] = Text::toT(Util::formatBytes(pos)+" ("+Util::toString((double)pos*100.0/(double)size)+"%)");
	columns[COLUMN_SIZE] = Text::toT(Util::formatBytes(size));
	columns[COLUMN_ADDED] = Text::toT(Util::formatTime("%Y-%m-%d %H:%M", iTime));
	columns[COLUMN_WAITING] = Text::toT(Util::formatSeconds(GET_TIME() - iTime));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onSpeaker(WPARAM wParam, LPARAM lParam) {
	::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

	switch(wParam) {
    	case REMOVE_ITEM: {
    		UploadQueueItem* i = (UploadQueueItem*)lParam;
    		ctrlList.deleteItem(i);
    		updateStatus();
    		i->dec();
    		if(BOOLSETTING(BOLD_WAITING_USERS)) {
    			setDirty();
            }

    		break;
        }
        case REMOVE: {
    		RemoveUser(((UserInfoBase*)lParam)->user);
    		delete (UserInfoBase*)lParam;
    		updateStatus();
    		if(BOOLSETTING(BOLD_WAITING_USERS)) {
    			setDirty();
            }

            break;
        }	
    	case ADD_ITEM: {
    		AddFile((UploadQueueItem*)lParam);
    		updateStatus();
    		ctrlList.resort();
    		if(BOOLSETTING(BOLD_WAITING_USERS)) {
    			setDirty();
            }

            break;
        }
    	case UPDATE_ITEMS: {
    		int j = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
    		int64_t itime = GET_TIME();
    		for(int i = 0; i < j; i++) {
    			UploadQueueItem* UQI = ctrlList.getItemData(i);
    			UQI->columns[COLUMN_WAITING] = Text::toT(Util::formatSeconds(itime - UQI->iTime));
    			ctrlList.updateItem(i);
    		}
    		break;
        }
        default:
            break;
	}

	::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onRefreshSettings() {
    bool refresh = false;

    if((COLORREF)::SendMessage(ctrlList.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlList.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlList.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
        ::SendMessage(hWndQueued, TVM_SETBKCOLOR, 0, (LPARAM)WinUtil::bgColor);
        refresh = true;
    }

    if((COLORREF)::SendMessage(ctrlList.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
        ::SendMessage(ctrlList.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
        ::SendMessage(hWndQueued, TVM_SETTEXTCOLOR, 0, (LPARAM)WinUtil::textColor);
        refresh = true;
    }

    if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

int UploadQueueItem::imageIndex() {
	return WinUtil::getIconIndex(Text::toT(File));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

UserPtr WaitingUsersFrame::getSelectedUser() {
	HTREEITEM selectedItem = GetParentItem();
	return selectedItem ? ((UserItem *)TreeGetItemData(hWndQueued, selectedItem))->u : UserPtr(NULL);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool WaitingUsersFrame::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*)pnmh;
	if(kd->wVKey == VK_DELETE) {
		removeSelected();
		return true;
	}

	return false;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool WaitingUsersFrame::onKeyDownDirs(LPNMHDR pnmh) {
	NMTVKEYDOWN* kd = (NMTVKEYDOWN*) pnmh;
	if(kd->wVKey == VK_DELETE) {
		removeSelectedUser();
		return true;
	}

	return false;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::onShowTree(WPARAM wParam) {
	showTree = (wParam == BST_CHECKED);
	LoadAll();

    if(showTree == true) {
        bSplitterDisabled = false;
        ::ShowWindow(hWndQueued, SW_SHOW);
        ::EnableWindow(hWndQueued, TRUE);
    } else {
        bSplitterDisabled = true;
        ::ShowWindow(hWndQueued, SW_HIDE);
        ::EnableWindow(hWndQueued, FALSE);
    }

    UpdateSplitterParts();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::removeSelected() {
	int i = -1;
	UserList RemoveUsers;
	while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
		// Ok let's cheat here, if you try to remove more users here is not working :(
		RemoveUsers.push_back(((UploadQueueItem*)ctrlList.getItemData(i))->User);
	}
	for(UserList::iterator i = RemoveUsers.begin(); i != RemoveUsers.end(); ++i) {
		UploadManager::getInstance()->clearUserFiles(*i);
	}
	updateStatus();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::removeSelectedUser() {
	UserPtr User = getSelectedUser();
	if(User) {
		UploadManager::getInstance()->clearUserFiles(User);
	}
	updateStatus();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::on(UploadManagerListener::QueueAdd, UploadQueueItem* aUQI) throw() {
	::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)ADD_ITEM, (LPARAM)aUQI);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::on(UploadManagerListener::QueueRemove, const UserPtr& aUser) throw() {
	::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)REMOVE, (LPARAM)new UserInfoBase(aUser));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::on(UploadManagerListener::QueueItemRemove, UploadQueueItem* aUQI) throw() {
	aUQI->inc();
	::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)REMOVE_ITEM, (LPARAM)aUQI);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::on(TimerManagerListener::Second, uint64_t /*aTick*/) throw() {
	::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)UPDATE_ITEMS, NULL);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

HWND WaitingUsersFrame::GetWindowHandle() {
    return m_hWnd;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void WaitingUsersFrame::UpdateSplitterParts() {
    if(bSplitterDisabled == false) {
        ::SetWindowPos(hWndQueued, NULL, rcSplitter.left, rcSplitter.top, iSplitterPos-2, rcSplitter.bottom-rcSplitter.top, SWP_NOZORDER);
        ::SetWindowPos(ctrlList.m_hWnd, NULL, rcSplitter.left+(iSplitterPos+2), rcSplitter.top, (rcSplitter.right-rcSplitter.left)-(iSplitterPos+2), rcSplitter.bottom-rcSplitter.top,
            SWP_NOZORDER);
    } else {
        ::SetWindowPos(ctrlList.m_hWnd, NULL, rcSplitter.left, rcSplitter.top, rcSplitter.right-rcSplitter.left, rcSplitter.bottom-rcSplitter.top, SWP_NOZORDER);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
