/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#define COMPILE_MULTIMON_STUBS 1
#include <MultiMon.h>
#include <psapi.h>

#include "WinUtil.h"
#include "PrivateFrame.h"
#include "SearchFrame.h"
#include "LineDlg.h"

#include "../client/Util.h"
#include "../client/StringTokenizer.h"
#include "../client/ShareManager.h"
#include "../client/ClientManager.h"
#include "../client/TimerManager.h"
#include "../client/FavoriteManager.h"
#include "../client/ResourceManager.h"
#include "../client/QueueManager.h"
#include "../client/UploadManager.h"
#include "../client/HashManager.h"
#include "../client/LogManager.h"
#include "../client/version.h"

#include "HubFrame.h"
#include "MagnetDlg.h"
#include "winamp.h"
#include <control.h>

WinUtil::ImageMap WinUtil::fileIndexes;
int WinUtil::fileImageCount;
HBRUSH WinUtil::bgBrush = NULL;
COLORREF WinUtil::textColor = 0;
COLORREF WinUtil::bgColor = 0;
HFONT WinUtil::font = NULL;
int WinUtil::fontHeight = 0;
HFONT WinUtil::boldFont = NULL;
HFONT WinUtil::systemFont = NULL;
HFONT WinUtil::monoFont = NULL;
HIMAGELIST WinUtil::fileImages;
HIMAGELIST WinUtil::userImages;
HIMAGELIST WinUtil::speedImages;
HIMAGELIST WinUtil::hStateImages = NULL;
int WinUtil::dirIconIndex = 0;
TStringList WinUtil::lastDirs;
HWND WinUtil::mainWnd = NULL;
HWND WinUtil::mdiClient = NULL;
FlatTabCtrl* WinUtil::tabCtrl = NULL;
HHOOK WinUtil::hook = NULL;
tstring WinUtil::tth;
StringPairList WinUtil::initialDirs;
bool WinUtil::urlDcRegistered = false;
bool WinUtil::urlMagnetRegistered = false;
bool WinUtil::isAppActive = false;
CHARFORMAT2 WinUtil::m_TextStyleTimestamp;
CHARFORMAT2 WinUtil::m_ChatTextGeneral;
CHARFORMAT2 WinUtil::m_TextStyleMyNick;
CHARFORMAT2 WinUtil::m_ChatTextMyOwn;
CHARFORMAT2 WinUtil::m_ChatTextServer;
CHARFORMAT2 WinUtil::m_ChatTextSystem;
CHARFORMAT2 WinUtil::m_TextStyleBold;
CHARFORMAT2 WinUtil::m_TextStyleFavUsers;
CHARFORMAT2 WinUtil::m_TextStyleOPs;
CHARFORMAT2 WinUtil::m_TextStyleURL;
CHARFORMAT2 WinUtil::m_ChatTextPrivate;
CHARFORMAT2 WinUtil::m_ChatTextLog;
CAGEmoticonSetup* WinUtil::g_pEmoticonsSetup = NULL;
HCURSOR WinUtil::hVerticalCursor = NULL;
HCURSOR WinUtil::hHorizontalCursor = NULL;

WinUtil::tbIDImage WinUtil::ToolbarButtons[] = {
	{IDC_SHUTDOWN, 0, true, ResourceManager::TOOLBAR_SHUTDOWN},
	{ID_FILE_CONNECT, 1, false, ResourceManager::MENU_PUBLIC_HUBS},
	{ID_FILE_RECONNECT, 2, false, ResourceManager::MENU_RECONNECT},
	{IDC_FOLLOW, 3, false, ResourceManager::MENU_FOLLOW_REDIRECT},
	{IDC_TOOLBAR_AWAY, 4, true, ResourceManager::AWAY},
	{IDC_RECENTS, 5, false, ResourceManager::MENU_FILE_RECENT_HUBS},
	{IDC_FAVORITES, 6, false, ResourceManager::MENU_FAVORITE_HUBS},
	{IDC_FAVUSERS, 7, false, ResourceManager::MENU_FAVORITE_AND_IGNORED_USERS},
	{IDC_QUEUE, 8, false, ResourceManager::MENU_DOWNLOAD_QUEUE},
	{IDC_FINISHED, 9, false, ResourceManager::FINISHED_DOWNLOADS},
	{IDC_UPLOAD_QUEUE, 10, false, ResourceManager::UPLOAD_QUEUE},
	{IDC_FINISHED_UL, 11, false, ResourceManager::FINISHED_UPLOADS},
	{ID_FILE_SEARCH, 12, false, ResourceManager::MENU_SEARCH},
	{IDC_FILE_ADL_SEARCH, 13, false, ResourceManager::MENU_ADL_SEARCH},
	{IDC_SEARCH_SPY, 14, false, ResourceManager::MENU_SEARCH_SPY},
	{IDC_NET_STATS, 15, false, ResourceManager::NETWORK_STATISTICS},
	{IDC_OPEN_FILE_LIST, 17, false, ResourceManager::MENU_OPEN_FILE_LIST},
	{ID_FILE_SETTINGS, 18, false, ResourceManager::MENU_SETTINGS},
	{IDC_NOTEPAD, 19, false, ResourceManager::MENU_NOTEPAD},
	{IDC_OPEN_DOWNLOADS, 20, false, ResourceManager::MENU_OPEN_DOWNLOADS_DIR},
	{IDC_MUTE_SOUNDS_TRAY, 21, false, ResourceManager::MUTE_SOUNDS_TRAY},
	{IDC_SYSTEM_LOG, 16, false, ResourceManager::SYSTEM_LOG},
	{0, 0, false, ResourceManager::MENU_NOTEPAD}
};

HLSCOLOR RGB2HLS (COLORREF rgb) {
	unsigned char minval = min(GetRValue(rgb), min(GetGValue(rgb), GetBValue(rgb)));
	unsigned char maxval = max(GetRValue(rgb), max(GetGValue(rgb), GetBValue(rgb)));
	float mdiff = float(maxval) - float(minval);
	float msum  = float(maxval) + float(minval);

	float luminance = msum / 510.0f;
	float saturation = 0.0f;
	float hue = 0.0f;

	if ( maxval != minval ) {
		float rnorm = (maxval - GetRValue(rgb) ) / mdiff;
		float gnorm = (maxval - GetGValue(rgb) ) / mdiff;
		float bnorm = (maxval - GetBValue(rgb) ) / mdiff;

		saturation = (luminance <= 0.5f) ? (mdiff / msum) : (mdiff / (510.0f - msum));

		if (GetRValue(rgb) == maxval) hue = 60.0f * (6.0f + bnorm - gnorm);
		if (GetGValue(rgb) == maxval) hue = 60.0f * (2.0f + rnorm - bnorm);
		if (GetBValue(rgb) == maxval) hue = 60.0f * (4.0f + gnorm - rnorm);
		if (hue > 360.0f) hue = hue - 360.0f;
	}
	return HLS ((hue*255)/360, luminance*255, saturation*255);
}

static BYTE _ToRGB (float rm1, float rm2, float rh) {
	if		(rh > 360.0f) rh -= 360.0f;
	else if (rh <   0.0f) rh += 360.0f;

	if		(rh <  60.0f) rm1 = rm1 + (rm2 - rm1) * rh / 60.0f;
	else if (rh < 180.0f) rm1 = rm2;
	else if (rh < 240.0f) rm1 = rm1 + (rm2 - rm1) * (240.0f - rh) / 60.0f;

	return (BYTE)(rm1 * 255);
}

COLORREF HLS2RGB (HLSCOLOR hls) {
	float hue        = ((int)HLS_H(hls)*360)/255.0f;
	float luminance  = HLS_L(hls)/255.0f;
	float saturation = HLS_S(hls)/255.0f;

	if ( saturation == 0.0f ) {
		return RGB (HLS_L(hls), HLS_L(hls), HLS_L(hls));
	}
	float rm1, rm2;

	if ( luminance <= 0.5f ) rm2 = luminance + luminance * saturation;
	else                     rm2 = luminance + saturation - luminance * saturation;
	rm1 = 2.0f * luminance - rm2;
	BYTE red   = _ToRGB (rm1, rm2, hue + 120.0f);
	BYTE green = _ToRGB (rm1, rm2, hue);
	BYTE blue  = _ToRGB (rm1, rm2, hue - 120.0f);

	return RGB (red, green, blue);
}

COLORREF HLS_TRANSFORM (COLORREF rgb, int percent_L, int percent_S) {
	HLSCOLOR hls = RGB2HLS (rgb);
	BYTE h = HLS_H(hls);
	BYTE l = HLS_L(hls);
	BYTE s = HLS_S(hls);

	if ( percent_L > 0 ) {
		l = BYTE(l + ((255 - l) * percent_L) / 100);
	} else if ( percent_L < 0 )	{
		l = BYTE((l * (100+percent_L)) / 100);
	}
	if ( percent_S > 0 ) {
		s = BYTE(s + ((255 - s) * percent_S) / 100);
	} else if ( percent_S < 0 ) {
		s = BYTE((s * (100+percent_S)) / 100);
	}
	return HLS2RGB (HLS(h, l, s));
}

void UserInfoBase::matchQueue() {
	try {
		QueueManager::getInstance()->addList(user, QueueItem::FLAG_MATCH_QUEUE);
	} catch(const Exception& e) {
		LogManager::getInstance()->message(e.getError());
	}
}
void UserInfoBase::getList() {
	try {
		QueueManager::getInstance()->addList(user, QueueItem::FLAG_CLIENT_VIEW);
	} catch(const Exception& e) {
		LogManager::getInstance()->message(e.getError());
	}
}
void UserInfoBase::addFav() {
	FavoriteManager::getInstance()->addFavoriteUser(user);
}
void UserInfoBase::pm() {
	PrivateFrame::openWindow(user);
}
void UserInfoBase::grant() {
	UploadManager::getInstance()->reserveSlot(user);
}
void UserInfoBase::removeAll() {
	QueueManager::getInstance()->removeSource(user, QueueItem::Source::FLAG_REMOVED);
}
void UserInfoBase::grantSlotHour() {
	UploadManager::getInstance()->reserveSlotHour(user);
}
void UserInfoBase::grantSlotDay() {
	UploadManager::getInstance()->reserveSlotDay(user);
}
void UserInfoBase::grantSlotWeek() {
	UploadManager::getInstance()->reserveSlotWeek(user);
}
void UserInfoBase::ungrantSlot() {
	UploadManager::getInstance()->unreserveSlot(user);
}

bool WinUtil::getVersionInfo(OSVERSIONINFOEX& ver) {
	memset(&ver, 0, sizeof(OSVERSIONINFOEX));
	ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if(!GetVersionEx((OSVERSIONINFO*)&ver)) {
		ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		if(!GetVersionEx((OSVERSIONINFO*)&ver)) {
			return false;
		}
	}
	return true;
}

static LRESULT CALLBACK KeyboardProc(int code, WPARAM wParam, LPARAM lParam) {
	if(code == HC_ACTION) {
		if(wParam == VK_CONTROL && LOWORD(lParam) == 1) {
			if(lParam & 0x80000000) {
				WinUtil::tabCtrl->endSwitch();
			} else {
				WinUtil::tabCtrl->startSwitch();
			}
		}
	}
	return CallNextHookEx(WinUtil::hook, code, wParam, lParam);
}

void WinUtil::init(HWND hWnd) {
	mainWnd = hWnd;

	if(BOOLSETTING(USE_SYSTEM_ICONS)) {
		SHFILEINFO fi = { 0 };
		fileImages = ImageList_Create(16, 16, ILC_COLOR32 | ILC_MASK, 16, 16);
		::SHGetFileInfo(_T("folder"), FILE_ATTRIBUTE_DIRECTORY, &fi, sizeof(SHFILEINFO), SHGFI_ICON | SHGFI_SMALLICON | SHGFI_USEFILEATTRIBUTES);
		ImageList_AddIcon(fileImages, fi.hIcon);
		::DestroyIcon(fi.hIcon);
		dirIconIndex = fileImageCount++;
	} else {
		fileImages = ImageList_LoadImage(g_hInstance, MAKEINTRESOURCE(IDB_FOLDERS), 16, 4, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
		dirIconIndex = 0;
	}

    userImages = ImageList_LoadImage(g_hInstance, MAKEINTRESOURCE(IDB_USERS), 16, 4, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
    speedImages = ImageList_LoadImage(g_hInstance, MAKEINTRESOURCE(IDB_SPEEDS), 16, 4, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);

	LOGFONT lf, lf2;
	::GetObject((HFONT)GetStockObject(DEFAULT_GUI_FONT), sizeof(lf), &lf);
	SettingsManager::getInstance()->setDefault(SettingsManager::TEXT_FONT, Text::fromT(encodeFont(lf)));
	decodeFont(Text::toT(SETTING(TEXT_FONT)), lf);
	::GetObject((HFONT)GetStockObject(ANSI_FIXED_FONT), sizeof(lf2), &lf2);

	lf2.lfHeight = lf.lfHeight;
	lf2.lfWeight = lf.lfWeight;
	lf2.lfItalic = lf.lfItalic;

	font = ::CreateFontIndirect(&lf);
	fontHeight = WinUtil::getTextHeight(mainWnd, font);
	lf.lfWeight = FW_BOLD;
	boldFont = ::CreateFontIndirect(&lf);
	systemFont = (HFONT)::GetStockObject(DEFAULT_GUI_FONT);
	monoFont = (HFONT)::GetStockObject(BOOLSETTING(USE_OEM_MONOFONT)?OEM_FIXED_FONT:ANSI_FIXED_FONT);

	if(BOOLSETTING(URL_HANDLER)) {
		registerDchubHandler();
		urlDcRegistered = true;
	}
	if(BOOLSETTING(MAGNET_REGISTER)) {
		registerMagnetHandler();
		urlMagnetRegistered = true;
	}

	hook = SetWindowsHookEx(WH_KEYBOARD, &KeyboardProc, NULL, GetCurrentThreadId());

	initColors();
	initEmoticons();

    hVerticalCursor = ::LoadCursor(NULL, IDC_SIZEWE);
    hHorizontalCursor = ::LoadCursor(NULL, IDC_SIZENS);
}

void WinUtil::initColors() {
	bgBrush = CreateSolidBrush(SETTING(BACKGROUND_COLOR));
	textColor = SETTING(TEXT_COLOR);
	bgColor = SETTING(BACKGROUND_COLOR);

	CHARFORMAT2 cf;
	memset(&cf, 0, sizeof(CHARFORMAT2));
	cf.cbSize = sizeof(cf);
	cf.dwReserved = 0;
	cf.dwMask = CFM_BACKCOLOR | CFM_COLOR | CFM_BOLD | CFM_ITALIC;
	cf.dwEffects = 0;
	cf.crBackColor = SETTING(BACKGROUND_COLOR);
	cf.crTextColor = SETTING(TEXT_COLOR);

	m_TextStyleTimestamp = cf;
	m_TextStyleTimestamp.crBackColor = SETTING(TEXT_TIMESTAMP_BACK_COLOR);
	m_TextStyleTimestamp.crTextColor = SETTING(TEXT_TIMESTAMP_FORE_COLOR);
	if(SETTING(TEXT_TIMESTAMP_BOLD))
		m_TextStyleTimestamp.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_TIMESTAMP_ITALIC))
		m_TextStyleTimestamp.dwEffects |= CFE_ITALIC;

	m_ChatTextGeneral = cf;
	m_ChatTextGeneral.crBackColor = SETTING(TEXT_GENERAL_BACK_COLOR);
	m_ChatTextGeneral.crTextColor = SETTING(TEXT_GENERAL_FORE_COLOR);
	if(SETTING(TEXT_GENERAL_BOLD))
		m_ChatTextGeneral.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_GENERAL_ITALIC))
		m_ChatTextGeneral.dwEffects |= CFE_ITALIC;

	m_TextStyleBold = m_ChatTextGeneral;
	m_TextStyleBold.dwEffects = CFE_BOLD;
	
	m_TextStyleMyNick = cf;
	m_TextStyleMyNick.crBackColor = SETTING(TEXT_MYNICK_BACK_COLOR);
	m_TextStyleMyNick.crTextColor = SETTING(TEXT_MYNICK_FORE_COLOR);
	if(SETTING(TEXT_MYNICK_BOLD))
		m_TextStyleMyNick.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_MYNICK_ITALIC))
		m_TextStyleMyNick.dwEffects |= CFE_ITALIC;

	m_ChatTextMyOwn = cf;
	m_ChatTextMyOwn.crBackColor = SETTING(TEXT_MYOWN_BACK_COLOR);
	m_ChatTextMyOwn.crTextColor = SETTING(TEXT_MYOWN_FORE_COLOR);
	if(SETTING(TEXT_MYOWN_BOLD))
		m_ChatTextMyOwn.dwEffects       |= CFE_BOLD;
	if(SETTING(TEXT_MYOWN_ITALIC))
		m_ChatTextMyOwn.dwEffects       |= CFE_ITALIC;

	m_ChatTextPrivate = cf;
	m_ChatTextPrivate.crBackColor = SETTING(TEXT_PRIVATE_BACK_COLOR);
	m_ChatTextPrivate.crTextColor = SETTING(TEXT_PRIVATE_FORE_COLOR);
	if(SETTING(TEXT_PRIVATE_BOLD))
		m_ChatTextPrivate.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_PRIVATE_ITALIC))
		m_ChatTextPrivate.dwEffects |= CFE_ITALIC;

	m_ChatTextSystem = cf;
	m_ChatTextSystem.crBackColor = SETTING(TEXT_SYSTEM_BACK_COLOR);
	m_ChatTextSystem.crTextColor = SETTING(TEXT_SYSTEM_FORE_COLOR);
	if(SETTING(TEXT_SYSTEM_BOLD))
		m_ChatTextSystem.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_SYSTEM_ITALIC))
		m_ChatTextSystem.dwEffects |= CFE_ITALIC;

	m_ChatTextServer = cf;
	m_ChatTextServer.crBackColor = SETTING(TEXT_SERVER_BACK_COLOR);
	m_ChatTextServer.crTextColor = SETTING(TEXT_SERVER_FORE_COLOR);
	if(SETTING(TEXT_SERVER_BOLD))
		m_ChatTextServer.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_SERVER_ITALIC))
		m_ChatTextServer.dwEffects |= CFE_ITALIC;

	m_ChatTextLog = m_ChatTextGeneral;
	m_ChatTextLog.crTextColor = blendColors(SETTING(TEXT_GENERAL_BACK_COLOR), SETTING(TEXT_GENERAL_FORE_COLOR), 0.4);

	m_TextStyleFavUsers = cf;
	m_TextStyleFavUsers.crBackColor = SETTING(TEXT_FAV_BACK_COLOR);
	m_TextStyleFavUsers.crTextColor = SETTING(TEXT_FAV_FORE_COLOR);
	if(SETTING(TEXT_FAV_BOLD))
		m_TextStyleFavUsers.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_FAV_ITALIC))
		m_TextStyleFavUsers.dwEffects |= CFE_ITALIC;

	m_TextStyleOPs = cf;
	m_TextStyleOPs.crBackColor = SETTING(TEXT_OP_BACK_COLOR);
	m_TextStyleOPs.crTextColor = SETTING(TEXT_OP_FORE_COLOR);
	if(SETTING(TEXT_OP_BOLD))
		m_TextStyleOPs.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_OP_ITALIC))
		m_TextStyleOPs.dwEffects |= CFE_ITALIC;

	m_TextStyleURL = cf;
	m_TextStyleURL.dwMask = CFM_COLOR | CFM_BOLD | CFM_ITALIC | CFM_BACKCOLOR | CFM_LINK | CFM_UNDERLINE;
	m_TextStyleURL.dwEffects = CFE_LINK | CFE_UNDERLINE;
	m_TextStyleURL.crBackColor = SETTING(TEXT_URL_BACK_COLOR);
	m_TextStyleURL.crTextColor = SETTING(TEXT_URL_FORE_COLOR);
	if(SETTING(TEXT_URL_BOLD))
		m_TextStyleURL.dwEffects |= CFE_BOLD;
	if(SETTING(TEXT_URL_ITALIC))
		m_TextStyleURL.dwEffects |= CFE_ITALIC;
}

void WinUtil::initEmoticons() {
	delete g_pEmoticonsSetup;

	g_pEmoticonsSetup = new CAGEmoticonSetup();
	if(g_pEmoticonsSetup == NULL || g_pEmoticonsSetup->Create(SETTING(EMOTICONFILE)) == false) {
		dcassert("Emoticons creating failed!");
	}
}

void WinUtil::uninit() {
	ImageList_Destroy(fileImages);
	ImageList_Destroy(userImages);
	ImageList_Destroy(speedImages);

	if(hStateImages != NULL) {
        ImageList_Destroy(hStateImages);
    }

	::DeleteObject(font);
	::DeleteObject(boldFont);
	::DeleteObject(bgBrush);

	delete g_pEmoticonsSetup;
	g_pEmoticonsSetup = NULL;

	UnhookWindowsHookEx(hook);
}

void WinUtil::decodeFont(const tstring& setting, LOGFONT &dest) {
	StringTokenizer<tstring> st(setting, _T(','));
	TStringList &sl = st.getTokens();

	::GetObject((HFONT)GetStockObject(DEFAULT_GUI_FONT), sizeof(dest), &dest);
	tstring face;
	if(sl.size() == 4)
	{
		face = sl[0];
		dest.lfHeight = Util::toInt(Text::fromT(sl[1]));
		dest.lfWeight = Util::toInt(Text::fromT(sl[2]));
		dest.lfItalic = (BYTE)Util::toInt(Text::fromT(sl[3]));
	}

	if(!face.empty()) {
		::ZeroMemory(dest.lfFaceName, LF_FACESIZE);
		_tcscpy_s(dest.lfFaceName, face.c_str());
	}
}

int CALLBACK WinUtil::browseCallbackProc(HWND hwnd, UINT uMsg, LPARAM /*lp*/, LPARAM pData) {
	if(uMsg == BFFM_INITIALIZED) {
		::SendMessage(hwnd, BFFM_SETSELECTION, TRUE, pData);
	}

	return 0;
}
bool WinUtil::browseDirectory(tstring& target, HWND owner /* = NULL */) {
    TCHAR buf[MAX_PATH];

    BROWSEINFO bi = { 0 };
    bi.hwndOwner = owner;
    bi.pszDisplayName = buf;
    bi.lpszTitle = CTSTRING(CHOOSE_FOLDER);
    bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
	bi.lpfn = &browseCallbackProc;
	bi.lParam = (LPARAM)target.c_str();

    LPITEMIDLIST pidl = ::SHBrowseForFolder(&bi);

    if(pidl == NULL) {
        return false;
    }

    ::SHGetPathFromIDList(pidl, buf);
    target = buf;

    if(target.empty() == false && target[target.size()-1] != _T('\\')) {
        target += _T('\\');
    }

    ::CoTaskMemFree(pidl);

    return true;
}

bool WinUtil::browseFile(tstring& target, HWND owner /* = NULL */, bool save /* = true */, const tstring& initialDir /* = Util::emptyString */, const TCHAR* types /* = NULL */, const TCHAR* defExt /* = NULL */) {
	TCHAR buf[MAX_PATH];
	target = Text::toT(Util::validateFileName(Text::fromT(target)));
	_tcscpy_s(buf, MAX_PATH, target.c_str());

	// Initialize OPENFILENAME
	OPENFILENAME ofn = { 0 }; // common dialog box structure
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = owner;
	ofn.lpstrFile = buf;
	ofn.lpstrFilter = types;
	ofn.lpstrDefExt = defExt;
	ofn.nFilterIndex = 1;

	if(!initialDir.empty()) {
		ofn.lpstrInitialDir = initialDir.c_str();
	}
	ofn.nMaxFile = sizeof(buf);
	ofn.Flags = (save ? 0: OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST);

	// Display the Open dialog box.
	if ( (save ? GetSaveFileName(&ofn) : GetOpenFileName(&ofn) ) ==TRUE) {
		target = ofn.lpstrFile;
		return true;
	}
	return false;
}

tstring WinUtil::encodeFont(LOGFONT const& font)
{
	tstring res(font.lfFaceName);
	res += L',';
	res += Text::toT(Util::toString(font.lfHeight));
	res += L',';
	res += Text::toT(Util::toString(font.lfWeight));
	res += L',';
	res += Text::toT(Util::toString(font.lfItalic));
	return res;
}

void WinUtil::setClipboard(const tstring& str) {
	if(!::OpenClipboard(mainWnd)) {
		return;
	}

	EmptyClipboard();

#ifdef UNICODE
	OSVERSIONINFOEX ver;
	if( WinUtil::getVersionInfo(ver) ) {
		if( ver.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS ) {
			string tmp = Text::wideToAcp(str);

			HGLOBAL hglbCopy = GlobalAlloc(GMEM_MOVEABLE, (tmp.size() + 1) * sizeof(char)); 
			if (hglbCopy == NULL) {
				CloseClipboard();
				return;
			}

			// Lock the handle and copy the text to the buffer.
			char* lptstrCopy = (char*)GlobalLock(hglbCopy);
			strcpy(lptstrCopy, tmp.c_str());
			GlobalUnlock(hglbCopy);

			SetClipboardData(CF_TEXT, hglbCopy);

			CloseClipboard();

			return;
		}
	}
#endif

	// Allocate a global memory object for the text.
	HGLOBAL hglbCopy = GlobalAlloc(GMEM_MOVEABLE, (str.size() + 1) * sizeof(TCHAR)); 
	if (hglbCopy == NULL) {
		CloseClipboard();
		return;
	}

	// Lock the handle and copy the text to the buffer.
	TCHAR* lptstrCopy = (TCHAR*)GlobalLock(hglbCopy);
	_tcscpy(lptstrCopy, str.c_str());
	GlobalUnlock(hglbCopy);

	// Place the handle on the clipboard.
#ifdef UNICODE
	SetClipboardData(CF_UNICODETEXT, hglbCopy);
#else
	SetClipboardData(CF_TEXT hglbCopy);
#endif

	CloseClipboard();
}

void WinUtil::splitTokens(int* array, const string& tokens, int maxItems /* = -1 */) throw() {
	StringTokenizer<string> t(tokens, _T(','));
	StringList& l = t.getTokens();
	if(maxItems == -1)
		maxItems = (int)l.size();

	int k = 0;
	for(StringList::const_iterator i = l.begin(); i != l.end() && k < maxItems; ++i, ++k) {
		array[k] = Util::toInt(*i);
	}
}

bool WinUtil::getUCParams(HWND parent, const UserCommand& uc, StringMap& sm) throw() {
	string::size_type i = 0;
	StringMap done;

	while( (i = uc.getCommand().find("%[line:", i)) != string::npos) {
		i += 7;
		string::size_type j = uc.getCommand().find(']', i);
		if(j == string::npos)
			break;

		string name = uc.getCommand().substr(i, j-i);
		if(done.find(name) == done.end()) {
			LineDlg dlg;
			dlg.title = Text::toT(uc.getName());
			dlg.description = Text::toT(name);
			dlg.line = Text::toT(sm["line:" + name]);
			if(dlg.DoModal(parent) == IDOK) {
				sm["line:" + name] = Text::fromT(dlg.line);
				done[name] = Text::fromT(dlg.line);
			} else {
				return false;
			}
		}
		i = j + 1;
	}
	i = 0;
	while( (i = uc.getCommand().find("%[kickline:", i)) != string::npos) {
		i += 11;
		string::size_type j = uc.getCommand().find(']', i);
		if(j == string::npos)
			break;

		string name = uc.getCommand().substr(i, j-i);
		if(done.find(name) == done.end()) {
			KickDlg dlg;
			dlg.title = Text::toT(uc.getName());
			dlg.description = Text::toT(name);
			if(dlg.DoModal(parent) == IDOK) {
				sm["kickline:" + name] = Text::fromT(dlg.line);
				done[name] = Text::fromT(dlg.line);
			} else {
				return false;
			}
		}
		i = j + 1;
	}
	return true;
}

tstring WinUtil::commands = _T("/refresh, /slots #, /search <string>, /czdc, /away <msg>, /back, /google <searchstring>, /yahoo <searchstring>, /imdb <searchstring>, /u <url>, /rebuild, /csfd <searchstring>, /wikipedia <searchstring>");

bool WinUtil::checkCommand(tstring& cmd, tstring& param, tstring& message, tstring& status) {
	string::size_type i = cmd.find(' ');
	if(i != string::npos) {
		param = cmd.substr(i+1);
		cmd = cmd.substr(1, i - 1);
	} else {
		cmd = cmd.substr(1);
	}

	if(Util::stricmp(cmd.c_str(), _T("log")) == 0) {
		if(Util::stricmp(param.c_str(), _T("system")) == 0) {
			WinUtil::openFile(Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + "system.log")));
		} else if(Util::stricmp(param.c_str(), _T("downloads")) == 0) {
			WinUtil::openFile(Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatTime(SETTING(LOG_FILE_DOWNLOAD), time(NULL)))));
		} else if(Util::stricmp(param.c_str(), _T("uploads")) == 0) {
			WinUtil::openFile(Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatTime(SETTING(LOG_FILE_UPLOAD), time(NULL)))));
		} else {
			return false;
		}
	} else if(Util::stricmp(cmd.c_str(), _T("refresh"))==0) {
		ShareManager::getInstance()->setDirty();
		ShareManager::getInstance()->refresh(true);
	} else if(Util::stricmp(cmd.c_str(), _T("slots"))==0) {
		int j = Util::toInt(Text::fromT(param));
		if(j > 0) {
			SettingsManager::getInstance()->set(SettingsManager::SLOTS, j);
			Util::setSlots(j);
			status = TSTRING(SLOTS_SET);
			ClientManager::getInstance()->infoUpdated(false);
		} else {
			status = TSTRING(INVALID_NUMBER_OF_SLOTS);
		}
	} else if(Util::stricmp(cmd.c_str(), _T("search")) == 0) {
		if(!param.empty()) {
            SearchOnDC(param);
		} else {
			status = TSTRING(SPECIFY_SEARCH_STRING);
		}
	} else if(Util::stricmp(cmd.c_str(), _T("czdc")) == 0) {
		message = _T("\r\n-- I'm a happy CzDC user. You could be happy too.\r\n-- http://www.czdc.org <") _T(CzDC_VERSION_STRING) _T(">");
	} else if(Util::stricmp(cmd.c_str(), _T("away")) == 0) {
		if(Util::getAway() && param.empty()) {
			Util::setAway(false);
			changeToolbarAway(false);
			status = TSTRING(AWAY_MODE_OFF);
		} else {
			Util::setAway(true);
			changeToolbarAway(true);
			Util::setAwayMessage(Text::fromT(param));
			status = TSTRING(AWAY_MODE_ON) + Text::toT(Util::getAwayMessage());
		}
		ClientManager::getInstance()->infoUpdated(true);
	} else if(Util::stricmp(cmd.c_str(), _T("back")) == 0) {
		Util::setAway(false);
		changeToolbarAway(false);
		ClientManager::getInstance()->infoUpdated(true);
		status = TSTRING(AWAY_MODE_OFF);
	} else if((Util::stricmp(cmd.c_str(), _T("g")) == 0) || (Util::stricmp(cmd.c_str(), _T("google")) == 0)) {
		if(param.empty()) {
			status = TSTRING(SPECIFY_SEARCH_STRING);
		} else {
			SearchOnGoogle(param);
		}
	} else if((Util::stricmp(cmd.c_str(), _T("y")) == 0) || (Util::stricmp(cmd.c_str(), _T("yahoo")) == 0)) {
		if(param.empty()) {
			status = TSTRING(SPECIFY_SEARCH_STRING);
		} else {
            SearchOnYahoo(param);
		}
	} else if(Util::stricmp(cmd.c_str(), _T("imdb")) == 0) {
		if(param.empty()) {
			status = TSTRING(SPECIFY_SEARCH_STRING);
		} else {
            SearchOnIMDB(param);
		}
	} else if((Util::stricmp(cmd.c_str(), _T("wikipedia")) == 0) || (Util::stricmp(cmd.c_str(), _T("wikip")) == 0)) {
		if(param.empty()) {
			status = TSTRING(SPECIFY_SEARCH_STRING);
		} else {
            SearchOnWikipedia(param);
		}
	} else if(Util::stricmp(cmd.c_str(), _T("u")) == 0) {
		if (param.empty()) {
			status = TSTRING(SPECIFY_URL);
		} else {
			WinUtil::openLink(Text::toT(Util::encodeURI(Text::fromT(param))));
		}
	} else if(Util::stricmp(cmd.c_str(), _T("rebuild")) == 0) {
		HashManager::getInstance()->rebuild();
		status = TSTRING(HASH_REBUILT);
	} else if((Util::stricmp(cmd.c_str(), _T("winamp")) == 0) || (Util::stricmp(cmd.c_str(), _T("w")) == 0)) {
		HWND hwndWinamp = FindWindow(_T("Winamp v1.x"), NULL);
		if (hwndWinamp) {
			StringMap params;
			int waVersion = (int)SendMessage(hwndWinamp,WM_USER, 0, IPC_GETVERSION),
				majorVersion, minorVersion;
			majorVersion = waVersion >> 12;
			if (((waVersion & 0x00F0) >> 4) == 0) {
				minorVersion = ((waVersion & 0x0f00) >> 8) * 10 + (waVersion & 0x000f);
			} else {
				minorVersion = ((waVersion & 0x00f0) >> 4) * 10 + (waVersion & 0x000f);
			}
			params["version"] = Util::toString(majorVersion + minorVersion / 100.0);
			int state = (int)SendMessage(hwndWinamp,WM_USER, 0, IPC_ISPLAYING);
			switch (state) {
				case 0: params["state"] = "stopped";
					break;
				case 1: params["state"] = "playing";
					break;
				case 3: params["state"] = "paused";
			};
			TCHAR titleBuffer[2048];
			int buffLength = sizeof(titleBuffer);
			GetWindowText(hwndWinamp, titleBuffer, buffLength);
			tstring title = titleBuffer;
			params["rawtitle"] = Text::fromT(title);
			// fix the title if scrolling is on, so need to put the stairs to the end of it
			tstring titletmp = title.substr(title.find(_T("***"))+2, title.size());
			title = titletmp + title.substr(0, title.size()-titletmp.size());
			title = title.substr(title.find(_T("."))+2, title.size());
			if (title.rfind(_T("-")) != string::npos) {
				params["title"] = Text::fromT(title.substr(0, title.rfind(_T('-')) - 1));
			}
			int curPos = (int)SendMessage(hwndWinamp,WM_USER, 0, IPC_GETOUTPUTTIME);
			int length = (int)SendMessage(hwndWinamp,WM_USER, 1, IPC_GETOUTPUTTIME);
			if (curPos == -1) {
				curPos = 0;
			} else {
				curPos /= 1000;
			}
			int intPercent;
			if (length > 0 ) {
				intPercent = curPos * 100 / length;
			} else {
				length = 0;
				intPercent = 0;
			}
			params["percent"] = Util::toString(intPercent) + "%";
			params["elapsed"] = Util::formatSeconds(curPos, true);
			params["length"] = Util::formatSeconds(length, true);
			int numFront = (int)min(max(intPercent / 10, 0), 10),
				numBack = (int)min(max(10 - 1 - numFront, 0), 10);
			string inFront = string(numFront, '-'),
				inBack = string(numBack, '-');
			params["bar"] = "[" + inFront + (state ? "|" : "-") + inBack + "]";
			int waSampleRate = (int)SendMessage(hwndWinamp,WM_USER, 0, IPC_GETINFO),
				waBitRate = (int)SendMessage(hwndWinamp,WM_USER, 1, IPC_GETINFO),
				waChannels = (int)SendMessage(hwndWinamp,WM_USER, 2, IPC_GETINFO);
			params["bitrate"] = Util::toString(waBitRate) + "kbps";
			params["sample"] = Util::toString(waSampleRate) + "kHz";
			params["channels"] = (waChannels==2?"stereo":"mono"); // 3+ channels? 0 channels?
			message = Text::toT(Util::formatParams(SETTING(WINAMP_FORMAT), params, false));
		} else {
			status = _T("Supported version of Winamp is not running");
		}
	} else if(Util::stricmp(cmd.c_str(), _T("csfd")) == 0) {
		if(param.empty()) {
			status = TSTRING(SPECIFY_SEARCH_STRING);
		} else {
            SearchOnCSFD(param);
		}
	} else {
		return false;
	}

	return true;
}

void WinUtil::copyMagnet(const TTHValue* aHash, int64_t aSize, const tstring& aFile) {
	if(aHash != NULL && !aFile.empty()) {
        string sFile = Util::encodeURI(Text::fromT(aFile));
        char* buf = new char[128+sFile.length()];
		sprintf_s(buf, 128+sFile.length(), "magnet:?xt=urn:tree:tiger:%s&xl=%I64d&dn=%s", aHash->toBase32().c_str(), 
			aSize, sFile.c_str());
		setClipboard(Text::toT(buf));
		delete [] buf;
	}
}

void WinUtil::searchHash(const TTHValue* aHash) {
	if(aHash != NULL) {
		SearchFrame::OpenWindow(Text::toT(aHash->toBase32()), 0, SearchManager::SIZE_DONTCARE, SearchManager::TYPE_TTH);
	}
}

void WinUtil::registerDchubHandler() {
	HKEY hk;
    TCHAR Buf[512];
    Buf[0] = 0;

	if(::RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\Classes\\dchub\\Shell\\Open\\Command"), 0, KEY_WRITE | KEY_READ, &hk) == ERROR_SUCCESS) {
		DWORD bufLen = sizeof(Buf);
		::RegQueryValueEx(hk, NULL, NULL, NULL, (LPBYTE)Buf, &bufLen);
		::RegCloseKey(hk);
	}

    tstring app = _T("\"") + Text::toT(getAppName()) + _T("\" %1");

	if(Util::stricmp(app.c_str(), Buf) == 0) {
        return;
    }

    ::SHDeleteKey(HKEY_CURRENT_USER, _T("Software\\Classes\\dchub"));

    if(::RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Classes\\dchub"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, NULL)) {
    	LogManager::getInstance()->message(STRING(ERROR_CREATING_REGISTRY_KEY_DCHUB));
    	return;
	}
	
	TCHAR* tmp = _T("URL:Direct Connect Protocol");
	::RegSetValueEx(hk, NULL, 0, REG_SZ, (LPBYTE)tmp, (DWORD)(sizeof(TCHAR) * (_tcslen(tmp) + 1)));
	::RegSetValueEx(hk, _T("URL Protocol"), 0, REG_SZ, (LPBYTE)_T(""), sizeof(TCHAR));
	::RegCloseKey(hk);

	::RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Classes\\dchub\\Shell\\Open\\Command"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, NULL);
	::RegSetValueEx(hk, _T(""), 0, REG_SZ, (LPBYTE)app.c_str(), (DWORD)(sizeof(TCHAR) * (app.length() + 1)));
	::RegCloseKey(hk);

	::RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Classes\\dchub\\DefaultIcon"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, NULL);
	app = Text::toT(getAppName());
	::RegSetValueEx(hk, _T(""), 0, REG_SZ, (LPBYTE)app.c_str(), (DWORD)(sizeof(TCHAR) * (app.length() + 1)));
	::RegCloseKey(hk);
}

void WinUtil::unRegisterDchubHandler() {
	::SHDeleteKey(HKEY_CURRENT_USER, _T("Software\\Classes\\dchub"));
}

void WinUtil::registerMagnetHandler() {
	HKEY hk;
	TCHAR buf[512];
	buf[0] = 0;

	// what command is set up to handle magnets right now?
	if(::RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\Classes\\magnet\\shell\\open\\command"), 0, KEY_READ, &hk) == ERROR_SUCCESS) {
		DWORD bufLen = sizeof(TCHAR) * sizeof(buf);
		::RegQueryValueEx(hk, NULL, NULL, NULL, (LPBYTE)buf, &bufLen);
		::RegCloseKey(hk);
	}

	tstring openCmd = buf;
	tstring magnetLoc = _T('"') + Text::toT(getAppName()) + _T('"');

	// (re)register the handler if CzDC is handling it
	if(BOOLSETTING(MAGNET_REGISTER) && _tcsicmp(openCmd.c_str(), (magnetLoc + _T(" %1")).c_str()) == 0) {
        return;
    }

	::SHDeleteKey(HKEY_CURRENT_USER, _T("Software\\Classes\\magnet"));

    if(::RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Classes\\magnet"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, NULL) != ERROR_SUCCESS) {
    	LogManager::getInstance()->message(STRING(ERROR_CREATING_REGISTRY_KEY_MAGNET));
    	return;
	}

	::RegSetValueEx(hk, NULL, NULL, REG_SZ, (LPBYTE)CTSTRING(MAGNET_SHELL_DESC), (DWORD)(sizeof(TCHAR)*(TSTRING(MAGNET_SHELL_DESC).length()+1)));
	::RegSetValueEx(hk, _T("URL Protocol"), NULL, REG_SZ, NULL, NULL);
	::RegCloseKey(hk);

	::RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Classes\\magnet\\DefaultIcon"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, NULL);
	::RegSetValueEx(hk, NULL, NULL, REG_SZ, (LPBYTE)magnetLoc.c_str(), (DWORD)(sizeof(TCHAR)*(magnetLoc.length()+1)));
	::RegCloseKey(hk);

	magnetLoc += _T(" %1");

	::RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Classes\\magnet\\shell\\open\\command"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, NULL);
	::RegSetValueEx(hk, NULL, NULL, REG_SZ, (LPBYTE)magnetLoc.c_str(), (DWORD)(sizeof(TCHAR)*(magnetLoc.length()+1)));
	::RegCloseKey(hk);
}

void WinUtil::unRegisterMagnetHandler() {
	::SHDeleteKey(HKEY_CURRENT_USER, _T("Software\\Classes\\magnet"));
}

void WinUtil::openLink(const tstring& url) {
	HKEY key;
	TCHAR regbuf[MAX_PATH];
	ULONG len = MAX_PATH;
	if(_strnicmp(Text::fromT(url).c_str(), "magnet:?", 8) == 0) {
		parseMagnetUri(url);
		return;
	}
	if(_strnicmp(Text::fromT(url).c_str(), "dchub://", 8) == 0) {
		parseDchubUrl(url);
		return;
	}
	tstring x;
	tstring::size_type i = url.find(_T("://"));
	if(i != string::npos) {
		x = url.substr(0, i);
	} else {
		x = _T("http");
	}
	x += _T("\\shell\\open\\command");
	if(::RegOpenKeyEx(HKEY_CLASSES_ROOT, x.c_str(), 0, KEY_READ, &key) == ERROR_SUCCESS) {
		DWORD dwType;
		if(::RegQueryValueEx(key, NULL, NULL, &dwType, (LPBYTE)regbuf, &len)) {
			/*
			 * Various values (for http handlers):
			 *  C:\PROGRA~1\MOZILL~1\FIREFOX.EXE -url "%1"
			 *  "C:\Program Files\Internet Explorer\iexplore.exe" -nohome
			 *  "C:\Apps\Opera7\opera.exe"
			 *  C:\Programy\Seamonkey\Seamonkey.exe -url "%1"
			 *  C:\PROGRA~1\NETSCAPE\NETSCAPE\NETSCP.EXE -url "%1"
			 */
			tstring cmd(regbuf); // otherwise you consistently get two trailing nulls

			if(cmd.length() > 1) {
				string::size_type start,end;
				if(cmd[0] == '"') {
					start = 1;
					end = cmd.find('"', 1);
				} else {
					start = 0;
					end = cmd.find(' ', 1);
				}
				if(end == string::npos)
					end = cmd.length();

				tstring cmdLine(cmd);
				cmd = cmd.substr(start, end-start);
				size_t arg_pos;
				if((arg_pos = cmdLine.find(_T("%1"))) != string::npos) {
					cmdLine.replace(arg_pos, 2, url);
				} else {
					cmdLine.append(_T(" \"") + url + _T('\"'));
				}

				STARTUPINFO si = { sizeof(si), 0 };
				PROCESS_INFORMATION pi = { 0 };
				int iLen = (int)(cmdLine.length() + 1);
				AutoArray<TCHAR> buf(iLen);
				_tcscpy_s(buf, iLen, cmdLine.c_str());
				if(::CreateProcess(cmd.c_str(), buf, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi)) {
					::CloseHandle(pi.hThread);
					::CloseHandle(pi.hProcess);
					return;
				}
			}
		}
	}

	::ShellExecute(NULL, NULL, url.c_str(), NULL, NULL, SW_SHOWNORMAL);
}

void WinUtil::parseDchubUrl(const tstring& aUrl) {
	string server, file;
	uint16_t port = 411;
	Util::decodeUrl(Text::fromT(aUrl), server, port, file);
	if(!server.empty()) {
		HubFrame::openWindow(Text::toT(server + ":" + Util::toString(port)));
	}
	if(!file.empty()) {
		if(file[0] == '/') // Remove any '/' in from of the file
			file = file.substr(1);
		try {
			UserPtr user = ClientManager::getInstance()->findLegacyUser(file);
			if(user)
				QueueManager::getInstance()->addList(user, QueueItem::FLAG_CLIENT_VIEW);
			// @todo else report error
		} catch(const Exception&) {
			// ...
		}
	}
}

void WinUtil::parseMagnetUri(const tstring& aUrl, bool /*aOverride*/) {
	// official types that are of interest to us
	//  xt = exact topic
	//  xs = exact substitute
	//  as = acceptable substitute
	//  dn = display name
	//  xl = exact length
	if (Util::strnicmp(aUrl.c_str(), _T("magnet:?"), 8) == 0) {
		LogManager::getInstance()->message(STRING(MAGNET_DLG_TITLE) + ": " + Text::fromT(aUrl), true);
		StringTokenizer<tstring> mag(aUrl.substr(8), _T('&'));
		typedef map<tstring, tstring> MagMap;
		MagMap hashes;
		tstring fname, fhash, type, param;
		int64_t fsize = 0;
		for(TStringList::iterator idx = mag.getTokens().begin(); idx != mag.getTokens().end(); ++idx) {
			// break into pairs
			string::size_type pos = idx->find(_T('='));
			if(pos != string::npos) {
				type = Text::toT(Text::toLower(Util::encodeURI(Text::fromT(idx->substr(0, pos)), true)));
				param = Text::toT(Util::encodeURI(Text::fromT(idx->substr(pos+1)), true));
			} else {
				type = Text::toT(Util::encodeURI(Text::fromT(*idx), true));
				param.clear();
			}
			// extract what is of value
			if(param.length() == 85 && Util::strnicmp(param.c_str(), _T("urn:bitprint:"), 13) == 0) {
				hashes[type] = param.substr(46);
			} else if(param.length() == 54 && Util::strnicmp(param.c_str(), _T("urn:tree:tiger:"), 15) == 0) {
				hashes[type] = param.substr(15);
			} else if(param.length() == 55 && Util::strnicmp(param.c_str(), _T("urn:tree:tiger/:"), 16) == 0) {
				hashes[type] = param.substr(16);
			} else if(param.length() == 59 && Util::strnicmp(param.c_str(), _T("urn:tree:tiger/1024:"), 20) == 0) {
				hashes[type] = param.substr(20);
			} else if(type.length() == 2 && Util::strnicmp(type.c_str(), _T("dn"), 2) == 0) {
				fname = param;
			} else if(type.length() == 2 && Util::strnicmp(type.c_str(), _T("xl"), 2) == 0) {
				fsize = _tstoi64(param.c_str());
			}
		}
		// pick the most authoritative hash out of all of them.
		if(hashes.find(_T("as")) != hashes.end()) {
			fhash = hashes[_T("as")];
		}
		if(hashes.find(_T("xs")) != hashes.end()) {
			fhash = hashes[_T("xs")];
		}
		if(hashes.find(_T("xt")) != hashes.end()) {
			fhash = hashes[_T("xt")];
		}
		if(!fhash.empty() && Encoder::isBase32(Text::fromT(fhash).c_str())) {
			// ok, we have a hash, and maybe a filename.
			if(!BOOLSETTING(MAGNET_ASK) && fsize > 0 && fname.length() > 0) {
				switch(SETTING(MAGNET_ACTION)) {
					case SettingsManager::MAGNET_AUTO_DOWNLOAD:
						try {
							QueueManager::getInstance()->add(Text::fromT(fname), fsize, Text::fromT(fhash));
						} catch(const Exception& e) {
							LogManager::getInstance()->message(e.getError());
						}
						break;
					case SettingsManager::MAGNET_AUTO_SEARCH:
						SearchFrame::OpenWindow(fhash, 0, SearchManager::SIZE_DONTCARE, SearchManager::TYPE_TTH);
						break;
				};
			} else {
                // use aOverride to force the display of the dialog.  used for auto-updating
				MagnetDlg dlg(fhash, fname, fsize);
				dlg.DoModal(mainWnd);
			}
		} else {
			MessageBox(mainWnd, CTSTRING(MAGNET_DLG_TEXT_BAD), CTSTRING(MAGNET_DLG_TITLE), MB_OK | MB_ICONEXCLAMATION);
		}
	}
}

int WinUtil::textUnderCursor(POINT p, HWND hWnd, tstring& x) {
	int i = (int)((short)LOWORD((DWORD)::SendMessage(hWnd, EM_CHARFROMPOS, 0, MAKELPARAM(p.x, p.y))));
	int line = (int)::SendMessage(hWnd, EM_LINEFROMCHAR, i, 0);
	int c = LOWORD(i) - (int)::SendMessage(hWnd, EM_LINEINDEX, line, 0);
	int len = (int)::SendMessage(hWnd, EM_LINELENGTH, i, 0) + 1;
	if(len < 3) {
		return 0;
	}

	TCHAR *buf = new TCHAR[len];

	*(LPWORD)buf = (WORD)len;
	::SendMessage(hWnd, EM_GETLINE, line, (LPARAM)buf);

	x = tstring(buf, len-1);

	delete[] buf;

	string::size_type start = x.find_last_of(_T(" <\t\r\n"), c);
	if(start == string::npos)
		start = 0;
	else
		start++;

	return (int)start;
}

bool WinUtil::parseDBLClick(const tstring& aString, string::size_type start, string::size_type end) {
	if((Util::strnicmp(aString.c_str() + start, _T("http://"), 7) == 0) ||
		(Util::strnicmp(aString.c_str() + start, _T("www."), 4) == 0) ||
		(Util::strnicmp(aString.c_str() + start, _T("ftp://"), 6) == 0) ||
		(Util::strnicmp(aString.c_str() + start, _T("irc://"), 6) == 0) ||
		(Util::strnicmp(aString.c_str() + start, _T("https://"), 8) == 0) ||
		(Util::strnicmp(aString.c_str() + start, _T("mailto:"), 7) == 0)) {
		openLink(aString.substr(start, end-start));
		return true;
	} else if(Util::strnicmp(aString.c_str() + start, _T("dchub://"), 8) == 0) {
		parseDchubUrl(aString.substr(start, end-start));
		return true;
	} else if(Util::strnicmp(aString.c_str() + start, _T("magnet:?"), 8) == 0) {
		parseMagnetUri(aString.substr(start, end-start));
		return true;
	}
	return false;
}

void WinUtil::saveHeaderOrder(HWND hWnd, SettingsManager::StrSetting order, SettingsManager::StrSetting widths, int n, int* indexes, int* sizes) throw() {
	string tmp;

	::SendMessage(hWnd, LVM_GETCOLUMNORDERARRAY, n, (LPARAM)indexes);

	int i;
	for(i = 0; i < n; ++i) {
		tmp += Util::toString(indexes[i]);
		tmp += ',';
	}
	tmp.erase(tmp.size()-1, 1);
	SettingsManager::getInstance()->set(order, tmp);
	tmp.clear();
	int nHeaderItemsCount = (int)::SendMessage((HWND)::SendMessage(hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0);
	for(i = 0; i < n; ++i) {
		sizes[i] = (int)::SendMessage(hWnd, LVM_GETCOLUMNWIDTH, i, 0);
		if(i >= nHeaderItemsCount) // Not exist column
			sizes[i] = 0;
			
		tmp += Util::toString(sizes[i]);
		tmp += ',';
	}
	tmp.erase(tmp.size()-1, 1);
	SettingsManager::getInstance()->set(widths, tmp);
}

int WinUtil::getIconIndex(const tstring& aFileName) {
	if(BOOLSETTING(USE_SYSTEM_ICONS)) {
		SHFILEINFO fi;
		memset(&fi, 0, sizeof(SHFILEINFO));
		string x = Text::fromT(Text::toLower(Util::getFileExt(aFileName)));
		if(!x.empty()) {
			ImageIter j = fileIndexes.find(x);
			if(j != fileIndexes.end())
				return j->second;
		}
		tstring fn = Text::toLower(Util::getFileName(aFileName));
		::SHGetFileInfo(fn.c_str(), FILE_ATTRIBUTE_NORMAL, &fi, sizeof(SHFILEINFO), SHGFI_ICON | SHGFI_SMALLICON | SHGFI_USEFILEATTRIBUTES);
		ImageList_AddIcon(fileImages, fi.hIcon);
		::DestroyIcon(fi.hIcon);

		fileIndexes[x] = fileImageCount++;
		return fileImageCount - 1;
	} else {
		return 2;
	}
}

double WinUtil::toBytes(TCHAR* aSize) {
	double bytes = _tstof(aSize);

	if (_tcsstr(aSize, CTSTRING(PB))) {
		return bytes * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
	} else if (_tcsstr(aSize, CTSTRING(TB))) {
		return bytes * 1024.0 * 1024.0 * 1024.0 * 1024.0;
	} else if (_tcsstr(aSize, CTSTRING(GB))) {
		return bytes * 1024.0 * 1024.0 * 1024.0;
	} else if (_tcsstr(aSize, CTSTRING(MB))) {
		return bytes * 1024.0 * 1024.0;
	} else if (_tcsstr(aSize, CTSTRING(KB))) {
		return bytes * 1024.0;
	} else {
		return bytes;
	}
}

tstring WinUtil::getNicks(const CID& cid) throw() {
	return Text::toT(Util::toString(ClientManager::getInstance()->getNicks(cid)));
}

pair<tstring, bool> WinUtil::getHubNames(const CID& cid) throw() {
	StringList hubs = ClientManager::getInstance()->getHubNames(cid);
	if(hubs.empty()) {
		return make_pair(TSTRING(OFFLINE), false);
	} else {
		return make_pair(Text::toT(Util::toString(hubs)), true);
	}
}

void WinUtil::openFolder(const tstring& file) {
	if (File::getSize(Text::fromT(file)) != -1)
		::ShellExecute(NULL, NULL, Text::toT("explorer.exe").c_str(), Text::toT("/e, /select, \"" + (Text::fromT(file)) + "\"").c_str(), NULL, SW_SHOWNORMAL);
	else
		::ShellExecute(NULL, NULL, Text::toT("explorer.exe").c_str(), Text::toT("/e, \"" + Util::getFilePath(Text::fromT(file)) + "\"").c_str(), NULL, SW_SHOWNORMAL);
}

int WinUtil::SetupPreviewMenu(HMENU previewMenu, string extension){
	int PreviewAppsSize = 0;
	PreviewApplication::List lst = FavoriteManager::getInstance()->getPreviewApps();
	if(lst.size() > 0){		
		PreviewAppsSize = 0;
		for(PreviewApplication::Iter i = lst.begin(); i != lst.end(); i++) {
			StringList tok = StringTokenizer<string>((*i)->getExtension(), ';').getTokens();
			bool add = false;
			if(tok.size() == 0) {
                add = true;
            }

			for(StringIter si = tok.begin(); si != tok.end(); ++si) {
				if(_stricmp(extension.c_str(), si->c_str()) == 0){
					add = true;
					break;
				}
			}

			if(add == true) {
                ::AppendMenu(previewMenu, MF_STRING, IDC_PREVIEW_APP + PreviewAppsSize, Text::toT(((*i)->getName())).c_str());
            }
			PreviewAppsSize++;
		}
	}
	return PreviewAppsSize;
}

void WinUtil::RunPreviewCommand(unsigned int index, const string &target) {
	PreviewApplication::List lst = FavoriteManager::getInstance()->getPreviewApps();

	if(index <= lst.size()) {
		string application = lst[index]->getApplication();
		string arguments = lst[index]->getArguments();
		StringMap ucParams;				

        if(File::FileExist(target+".antifrag")) {	
            ucParams["file"] = "\"" + target + ".antifrag\"";
        } else {
            ucParams["file"] = "\"" + target + "\"";
        }
		ucParams["dir"] = "\"" + Util::getFilePath(target) + "\"";

		::ShellExecute(NULL, NULL, Text::toT(application).c_str(), Text::toT(Util::formatParams(arguments, ucParams, false)).c_str(), Text::toT(ucParams["dir"]).c_str(), SW_SHOWNORMAL);
	}
}

string WinUtil::formatTime(uint64_t rest) {
    static char time[256];
	time[0] = NULL;
	char buf[128];
	uint64_t n, i;
	i = 0;
    n = rest / (24*3600*365);
	rest %= (24*3600*365);
	if(n) {
		if(n >= 2)
			sprintf_s(buf, 128, "%d years", n);
		else
			sprintf_s(buf, 128, "%d year", n);
		strcat_s(time, 256, buf);
		i++;
	}
    n = rest / (24*3600*30);
	rest %= (24*3600*30);
	if(n) {
		if(i > 0) {
			if(n >= 2)
				sprintf_s(buf, 128, " %d months", n);
			else
				sprintf_s(buf, 128, " %d month", n);
		} else {
			if(n >= 2)
				sprintf_s(buf, 128, "%d months", n);
			else
				sprintf_s(buf, 128, "%d month", n);
		}
		strcat_s(time, 256, buf);
		i++;
	}
	n = rest / (24*3600);
	rest %= (24*3600);
	if(n) {
		if(i > 0) {
			if(n >= 2)
				sprintf_s(buf, 128, " %d days", n); 
			else
				sprintf_s(buf, 128, " %d day", n);
		} else {
			if(n >= 2)
				sprintf_s(buf, 128, "%d days", n); 
			else
				sprintf_s(buf, 128, "%d day", n);
		}
		strcat_s(time, 256, buf);
		i++;
	}
    n = rest / (3600);
	rest %= (3600);
	if(n) {
		if(i > 0) {
			if(n >= 2)
				sprintf_s(buf, 128, " %d hours", n);
			else
				sprintf_s(buf, 128, " %d hour", n);
		} else {
			if(n >= 2)
				sprintf_s(buf, 128, "%d hours", n);
			else
				sprintf_s(buf, 128, "%d hour", n);
		}
		strcat_s(time, 256, buf);
		i++;
	}
    n = rest / (60);
	rest %= (60);
	if(n) {
		if(i > 0) {
			sprintf_s(buf, 128, " %d min", n);
		} else {
			sprintf_s(buf, 128, "%d min", n);
		}
		strcat_s(time, 256, buf);
		i++;
	}
	n = rest;
	if(++i <= 3 && n > 0) {
		if(i > 0) {
			sprintf_s(buf, 128, " %d sec ", n);
		} else {
			sprintf_s(buf, 128, "%d sec ", n);
		}
		strcat_s(time, 256, buf);
	}
	return time;
}

uint8_t WinUtil::getImage(const Identity& u, bool bNoOPs/* = false*/) {
	uint8_t image = u.getcType();
	if(u.isOp() && bNoOPs == false) {
		image = 0;
	} else if(u.isSet(Identity::FIREBALL)) {
		image = 10;
	} else if(u.isSet(Identity::SERVER)) {
		image = 9;
	} 
	if(u.isSet(Identity::AWAY)) {
		image+=12;
	}
	if(u.getUser()->isSet(User::DCPLUSPLUS)) {
		image+=24;
	}
	if(u.getUser()->isSet(User::PASSIVE) || (u.getMode() == "P") || (u.getMode() == "5")) {
		image+=48;
	}
	return image;
}

uint8_t WinUtil::getSpeedImage(int conn) {
	if(conn == 1 || conn == 2) {
		return 1;
	} else if(conn == 3 || conn == 4 || conn == 5) {
		return 2;
	} else if(conn == 6) {
		return 3;
	}
	return 0;
}

COLORREF WinUtil::blendColors(const COLORREF& cr1, const COLORREF& cr2, double balance) {
	BYTE r1 = getRValue(cr1);
	BYTE g1 = getGValue(cr1);
	BYTE b1 = getBValue(cr1);
	BYTE r2 = getRValue(cr2);
	BYTE g2 = getGValue(cr2);
	BYTE b2 = getBValue(cr2);
	return RGB(
		(r1*(balance*2) + (r2*((1-balance)*2))) / 2,
		(g1*(balance*2) + (g2*((1-balance)*2))) / 2,
		(b1*(balance*2) + (b2*((1-balance)*2))) / 2
	);
}

void WinUtil::changeToolbarAway(bool Away) {
	if(Away) {
		if(::SendMessage(MainFrame::pMainFrame->hwndToolbar, TB_ISBUTTONCHECKED, IDC_TOOLBAR_AWAY, 0) == 0) {
            ::SendMessage(MainFrame::pMainFrame->hwndToolbar, TB_CHECKBUTTON, IDC_TOOLBAR_AWAY, MAKELPARAM(TRUE, 0));
        }
	} else {
		if(::SendMessage(MainFrame::pMainFrame->hwndToolbar, TB_ISBUTTONCHECKED, IDC_TOOLBAR_AWAY, 0) != 0) {
            ::SendMessage(MainFrame::pMainFrame->hwndToolbar, TB_CHECKBUTTON, IDC_TOOLBAR_AWAY, MAKELPARAM(FALSE, 0));
        }
	}
}

string WinUtil::generateStats() {
	char buf[2048];
	if(Util::OsMajor >= 5) {
		PROCESS_MEMORY_COUNTERS pmc;
        memset(&pmc, 0, sizeof(PROCESS_MEMORY_COUNTERS));
		pmc.cb = sizeof(pmc);

		::GetProcessMemoryInfo(GetCurrentProcess(), &pmc, sizeof(pmc));

		FILETIME tmpa, tmpb, kernelTimeFT, userTimeFT;
		::GetProcessTimes(GetCurrentProcess(), &tmpa, &tmpb, &kernelTimeFT, &userTimeFT);
		int64_t kernelTime = kernelTimeFT.dwLowDateTime | (((int64_t)kernelTimeFT.dwHighDateTime) << 32);
		int64_t userTime = userTimeFT.dwLowDateTime | (((int64_t)userTimeFT.dwHighDateTime) << 32);
		double dCpuPercents = ((double)(kernelTime + userTime) / (10I64 * 1000I64 * 1000I64))/(((double)Util::getUptime())/100);

        sprintf_s(buf, 2048, "-=[ " CzDC_VERSION_STRING
#ifdef _M_X64
            " (x64)"
#endif
            " ]=-\r\n-=[ Uptime: %s ]=-\r\n-=[ CPU time: %s (Avg. %0.2f %%) ]=-\r\n-=[ Memory usage (peak): %s (%s) ]=-\r\n-=[ Virtual memory usage (peak): %s (%s) ]=-\r\n"
            "-=[ Downloaded: %s ][ Uploaded: %s ]=-\r\n-=[ Total download: %s ][ Total upload: %s ]=-\r\n-=[ Total Uptime: %s ]=-\r\n-=[ System Uptime: %s ]=-",
			formatTime(Util::getUptime()).c_str(), Util::formatSeconds((kernelTime + userTime) / (10I64 * 1000I64 * 1000I64)).c_str(), dCpuPercents, 
			Util::formatBytes(pmc.WorkingSetSize).c_str(), Util::formatBytes(pmc.PeakWorkingSetSize).c_str(), 
			Util::formatBytes(pmc.PagefileUsage).c_str(), Util::formatBytes(pmc.PeakPagefileUsage).c_str(), 
			Util::formatBytes(Socket::getTotalDown()).c_str(), Util::formatBytes(Socket::getTotalUp()).c_str(), 
			Util::formatBytes(SETTING(TOTAL_DOWNLOAD)).c_str(), Util::formatBytes(SETTING(TOTAL_UPLOAD)).c_str(), 
			formatTime(SETTING(TOTAL_UPTIME)).c_str(), formatTime(::GetTickCount()/1000).c_str());
		return buf;
	} else {
		sprintf_s(buf, 2048, "-=[ " CzDC_VERSION_STRING " ]=-\r\n-=[ Uptime: %s ]=-\r\n-=[ Downloaded: %s ][ Uploaded: %s ]=-\r\n-=[ Total download: %s ][ Total upload: %s ]=-\r\n"
            "-=[ Total Uptime: %s ]=-\r\n-=[ System Uptime: %s ]=-",
			formatTime(Util::getUptime()).c_str(), Util::formatBytes(Socket::getTotalDown()).c_str(), Util::formatBytes(Socket::getTotalUp()).c_str(),
			Util::formatBytes(SETTING(TOTAL_DOWNLOAD)).c_str(), Util::formatBytes(SETTING(TOTAL_UPLOAD)).c_str(),
            formatTime(SETTING(TOTAL_UPTIME)).c_str(), formatTime(::GetTickCount()/1000).c_str());
		return buf;
	}
}

void WinUtil::SearchOnDC(tstring sSearchString) {
    SearchFrame::OpenWindow(sSearchString);
}

void WinUtil::SearchOnGoogle(tstring sSearchString) {
    openLink(_T("http://www.google.com/search?q=") + Text::toT(Util::encodeURI(Text::fromT(sSearchString))));
}

void WinUtil::SearchOnYahoo(tstring sSearchString) {
    openLink(_T("http://search.yahoo.com/search?p=") + Text::toT(Util::encodeURI(Text::fromT(sSearchString))));
}

void WinUtil::SearchOnIMDB(tstring sSearchString) {
    openLink(_T("http://www.imdb.com/find?q=") + Text::toT(Util::encodeURI(Text::fromT(sSearchString))));
}

void WinUtil::SearchOnCSFD(tstring sSearchString) {
    openLink(_T("http://www.csfd.cz/search_pg.php?search=") + Text::toT(Util::encodeURI(Text::fromT(sSearchString))));
}

void WinUtil::SearchOnWikipedia(tstring sSearchString) {
    openLink(_T("http://en.wikipedia.org/wiki/Special:Search?search=") + Text::toT(Util::encodeURI(Text::fromT(sSearchString))) + _T("&fulltext=Search"));
}

void WinUtil::AddSearchSubMenu(HMENU menu) {
    HMENU searchMenu = ::CreatePopupMenu();
    ::AppendMenu(searchMenu, MF_STRING, IDC_SEARCH_DC, CTSTRING(SEARCH_DC));
    ::AppendMenu(searchMenu, MF_STRING, IDC_SEARCH_GOOGLE, CTSTRING(SEARCH_GOOGLE));
    ::AppendMenu(searchMenu, MF_STRING, IDC_SEARCH_YAHOO, CTSTRING(SEARCH_YAHOO));
    ::AppendMenu(searchMenu, MF_STRING, IDC_SEARCH_IMDB, CTSTRING(SEARCH_IMDB));
    ::AppendMenu(searchMenu, MF_STRING, IDC_SEARCH_CSFD, CTSTRING(SEARCH_CSFD));
    ::AppendMenu(searchMenu, MF_STRING, IDC_SEARCH_WIKI, CTSTRING(SEARCH_WIKI));

    ::AppendMenu(menu, MF_POPUP, (UINT_PTR)searchMenu, CTSTRING(SEARCH));
}

void WinUtil::AddPrioritySubMenu(HMENU menu, tstring tsPriority) {
    HMENU priorityMenu = ::CreatePopupMenu();
    ::AppendMenu(priorityMenu, MF_STRING, IDC_PRIORITY_PAUSED, CTSTRING(PAUSED));
	::AppendMenu(priorityMenu, MF_STRING, IDC_PRIORITY_LOWEST, CTSTRING(LOWEST));
	::AppendMenu(priorityMenu, MF_STRING, IDC_PRIORITY_LOW, CTSTRING(LOW));
	::AppendMenu(priorityMenu, MF_STRING, IDC_PRIORITY_NORMAL, CTSTRING(NORMAL));
	::AppendMenu(priorityMenu, MF_STRING, IDC_PRIORITY_HIGH, CTSTRING(HIGH));
	::AppendMenu(priorityMenu, MF_STRING, IDC_PRIORITY_HIGHEST, CTSTRING(HIGHEST));

    ::AppendMenu(menu, MF_POPUP, (UINT_PTR)priorityMenu, tsPriority.c_str());
}

void WinUtil::AddGrantSubMenu(HMENU menu) {
    HMENU grantMenu = ::CreatePopupMenu();
    ::AppendMenu(grantMenu, MF_STRING, IDC_GRANTSLOT, CTSTRING(GRANT_EXTRA_SLOT));
    ::AppendMenu(grantMenu, MF_STRING, IDC_GRANTSLOT_HOUR, CTSTRING(GRANT_EXTRA_SLOT_HOUR));
    ::AppendMenu(grantMenu, MF_STRING, IDC_GRANTSLOT_DAY, CTSTRING(GRANT_EXTRA_SLOT_DAY));
    ::AppendMenu(grantMenu, MF_STRING, IDC_GRANTSLOT_WEEK, CTSTRING(GRANT_EXTRA_SLOT_WEEK));
    ::AppendMenu(grantMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(grantMenu, MF_STRING, IDC_UNGRANTSLOT, CTSTRING(REMOVE_EXTRA_SLOT));

    ::AppendMenu(menu, MF_POPUP, (UINT_PTR)grantMenu, CTSTRING(GRANT_SLOTS_MENU));
}

void WinUtil::CreateStateImages() {
    HTHEME hTheme = ::OpenThemeData(NULL, L"TREEVIEW");
    if(hTheme == NULL) {
        return;
    }

    SIZE sz = { 0, 0 };
    if(::GetThemePartSize(hTheme, NULL, TVP_GLYPH, GLPS_CLOSED, NULL, TS_TRUE, &sz) == S_OK) {
        HDC hWindowDC = ::GetWindowDC(NULL);
        if(hWindowDC == NULL) {
            ::CloseThemeData(hTheme);
            return;
        }

        HBITMAP hState = ::CreateCompatibleBitmap(hWindowDC, Util::OsMajor < 6 ? 16 : sz.cx, Util::OsMajor < 6 ? 16 : sz.cy);
        if(hState == NULL) {
            ::CloseThemeData(hTheme);
            return;
        }

        HDC hDC = ::CreateCompatibleDC(hWindowDC);
        if(hDC == NULL) {
            ::DeleteObject(hState);
            ::CloseThemeData(hTheme);
            return;
        }

        HBITMAP hOldBitmap = (HBITMAP)::SelectObject(hDC, hState);

        if(Util::OsMajor < 6) {
            HBRUSH hBG = ::CreateSolidBrush(bgColor);
            RECT rect = { 0, 0, 16, 16 };
            ::FillRect(hDC, &rect, hBG);
            ::DeleteObject(hBG);
        }

        RECT rc = { Util::OsMajor < 6 ? 4 : 0, Util::OsMajor < 6 ? 3 : 0, Util::OsMajor < 6 ? sz.cx+4 : sz.cx, Util::OsMajor < 6 ? sz.cy+3 : sz.cy };
        if(::DrawThemeBackground(hTheme, hDC, TVP_GLYPH, GLPS_CLOSED, &rc, NULL) != S_OK) {
            ::DeleteObject(hState);
            ::CloseThemeData(hTheme);
            return;
        }

        ::SelectObject(hDC, hOldBitmap);

        if(Util::OsMajor < 6) {
            hStateImages = ::ImageList_Create(16, 16, ILC_COLOR16, 2, 0);
        } else {
            hStateImages = ::ImageList_Create(sz.cx, sz.cy, ILC_COLOR32, 2, 0);
        }

        if(hStateImages == NULL) {
            ::DeleteObject(hState);
            ::CloseThemeData(hTheme);
            return;
        }

        ::ImageList_Add(hStateImages, hState, NULL);

        ::DeleteObject(hState);

        hState = ::CreateCompatibleBitmap(hWindowDC, Util::OsMajor < 6 ? 16 : sz.cx, Util::OsMajor < 6 ? 16 : sz.cy);
        if(hState == NULL) {
            ::ImageList_Destroy(hStateImages);
            hStateImages = NULL;
            ::CloseThemeData(hTheme);
            return;
        }

        hOldBitmap = (HBITMAP)::SelectObject(hDC, hState);

        if(Util::OsMajor < 6) {
            HBRUSH hBG = ::CreateSolidBrush(bgColor);
            RECT rect = { 0, 0, 16, 16 };
            ::FillRect(hDC, &rect, hBG);
            ::DeleteObject(hBG);
        }

        if(::DrawThemeBackground(hTheme, hDC, TVP_GLYPH, GLPS_OPENED, &rc, NULL) != S_OK) {
            ::ImageList_Destroy(hStateImages);
            hStateImages = NULL;
            ::DeleteObject(hState);
            ::CloseThemeData(hTheme);
            return;
        }

        ::SelectObject(hDC, hOldBitmap);

        ::ImageList_Add(hStateImages, hState, NULL);

        ::DeleteObject(hState);
        ::DeleteDC(hDC);
        ::ReleaseDC(NULL, hWindowDC);
    }

    ::CloseThemeData(hTheme);
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
